@@ -24,6 +24,7 @@
 #include "config.h"
 
 #include "Account.h"
+#include "AccountP.h"
 #include "Group.h"
 #include "LedgerUtils.h"
 #include "MultiLedger.h"
@@ -32,6 +33,7 @@
 #include "Transaction.h"
 #include "util.h"
 
+
 /** GLOBALS *********************************************************/
 /* These are globals because they describe the state of the entire session.
  * The is, there must be only one instance of these per GUI session.
@@ -41,6 +43,7 @@ static xaccLedgerDisplay **regList = NULL;     /* single-account registers */
 static xaccLedgerDisplay **ledgerList = NULL;  /* multiple-account registers */
 static xaccLedgerDisplay **fullList = NULL;    /* all registers */
 
+static short module = MOD_LEDGER;
 
 /********************************************************************\
  * Ledger utilities                                                 *
@@ -52,7 +55,7 @@ static xaccLedgerDisplay **fullList = NULL;    /* all registers */
  * present a rather natural place for the locks to be placed.       *
 \********************************************************************/
 
-int 
+static int 
 ledgerListCount (xaccLedgerDisplay **list)
 {
    int n = 0;
@@ -63,7 +66,7 @@ ledgerListCount (xaccLedgerDisplay **list)
 
 /* ------------------------------------------------------ */
 
-xaccLedgerDisplay ** 
+static xaccLedgerDisplay ** 
 ledgerListAdd (xaccLedgerDisplay **oldlist, xaccLedgerDisplay *addreg)
 {
    xaccLedgerDisplay **newlist;
@@ -93,7 +96,7 @@ ledgerListAdd (xaccLedgerDisplay **oldlist, xaccLedgerDisplay *addreg)
 
 /* ------------------------------------------------------ */
 
-void
+static void
 ledgerListRemove (xaccLedgerDisplay **list, xaccLedgerDisplay *delreg)
 {
    int n, i;
@@ -185,7 +188,7 @@ xaccLedgerDisplaySimple (Account *acc)
       reg_type = CURRENCY_REGISTER;
       break;
     default:
-      PERR (" xaccLedgerDisplaySimple(): unknown account type\n");
+      PERR ("xaccLedgerDisplaySimple(): unknown account type %d\n", acc_type);
       return NULL;
   }
 
@@ -197,7 +200,7 @@ xaccLedgerDisplaySimple (Account *acc)
   }
 
 /********************************************************************\
- * xaccLedgerDisplayAccGroup                                             *
+ * xaccLedgerDisplayAccGroup                                        *
  *   opens up a register window to display an account, and all      *
  *   of its children, in the same window                            *
  *                                                                  *
@@ -207,7 +210,7 @@ xaccLedgerDisplaySimple (Account *acc)
 
 xaccLedgerDisplay *
 xaccLedgerDisplayAccGroup (Account *acc)
-  {
+{
   xaccLedgerDisplay *retval;
   Account **list;
   int ledger_type;
@@ -257,7 +260,7 @@ xaccLedgerDisplayAccGroup (Account *acc)
        break;
 
     default:
-      PERR (" xaccLedgerDisplayAccGroup(): unknown account type \n");
+      PERR ("xaccLedgerDisplayAccGroup(): unknown account type \n");
       _free (list);
       return NULL;
   }
@@ -269,10 +272,38 @@ xaccLedgerDisplayAccGroup (Account *acc)
 
   if (list) _free (list);
   return retval;
-  }
+}
+
+static gncUIWidget
+xaccLedgerDisplayParent(void *user_data)
+{
+  xaccLedgerDisplay *regData = user_data;
+
+  if (regData == NULL)
+    return NULL;
+
+  if (regData->get_parent == NULL)
+    return NULL;
+
+  return (regData->get_parent)(regData);
+}
+
+static void
+xaccLedgerDisplaySetHelp(void *user_data, const char *help_str)
+{
+  xaccLedgerDisplay *regData = user_data;
+
+  if (regData == NULL)
+    return;
+
+  if (regData->set_help == NULL)
+    return;
+
+  (regData->set_help)(regData, help_str);
+}
 
 /********************************************************************\
- * xaccLedgerDisplayLedger                                               *
+ * xaccLedgerDisplayLedger                                          *
  *   opens up a ledger window for a list of accounts                *
  *                                                                  *
  * Args:   lead_acc - the account associated with this register     *
@@ -284,8 +315,8 @@ xaccLedgerDisplayAccGroup (Account *acc)
 
 xaccLedgerDisplay *
 xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
-  {
-  xaccLedgerDisplay   *regData = NULL;
+{
+  xaccLedgerDisplay *regData = NULL;
 
   /******************************************************************\
   \******************************************************************/
@@ -323,6 +354,8 @@ xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
   regData->leader = lead_acc;
   regData->redraw = NULL;
   regData->destroy = NULL;
+  regData->get_parent = NULL;
+  regData->set_help = NULL;
   regData->gui_hook = NULL;
   regData->dirty = 0;
   regData->balance = 0.0;
@@ -338,7 +371,9 @@ xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
   /* set up the query filter */
   regData->query = xaccMallocQuery();
   xaccQuerySetAccounts (regData->query, regData->displayed_accounts);
-  xaccQueryAddAccount (regData->query, regData->leader);
+  if ((regData->leader != NULL) &&
+      !accListHasAccount(regData->displayed_accounts, regData->leader))
+    xaccQueryAddAccount (regData->query, regData->leader);
 
   /* by default, display only thirty transactions */
   xaccQuerySetMaxSplits (regData->query, MAX_QUERY_SPLITS);
@@ -350,10 +385,14 @@ xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
    * The main register window itself                                *
   \******************************************************************/
 
-  /* MallocBasicRegister will malloc & initialize the register,
+  /* xaccMallocSplitRegister will malloc & initialize the register,
    * but will not do the gui init */
   regData->ledger = xaccMallocSplitRegister (ledger_type);
-  
+
+  xaccSRSetData(regData->ledger, regData,
+                xaccLedgerDisplayParent,
+                xaccLedgerDisplaySetHelp);
+
   regData->dirty = 1;
   xaccLedgerDisplayRefresh (regData);
   
@@ -367,8 +406,6 @@ xaccLedgerDisplayGeneral (Account *lead_acc, Account **acclist, int ledger_type)
 void 
 xaccLedgerDisplayRefresh (xaccLedgerDisplay *regData)
 {
-   int typo;
-
    /* If we don't really need the redraw, don't do it. */
    if (!(regData->dirty)) return;
    regData->dirty = 0;  /* mark clean */
@@ -377,42 +414,43 @@ xaccLedgerDisplayRefresh (xaccLedgerDisplay *regData)
     * assign a default source account for a "blank split"
     * that is attached to the bottom of the register.
     * The "blank split" is what the user edits to create 
-    * new splits and get them into the system.
-    */
+    * new splits and get them into the system. */
    xaccSRLoadRegister (regData->ledger, 
-                     xaccQueryGetSplits (regData->query),
-                     regData->leader);
-
+                       xaccQueryGetSplits (regData->query),
+                       regData->leader);
 
   /* hack alert -- this computation of totals is incorrect 
    * for multi-account ledgers */
 
-  /* provide some convenience data for the ture GUI window.
-   * If the GUI wants to display yet other stuff, its on its own.
-   */
-  xaccAccountRecomputeBalance(regData->leader);
+  /* provide some convenience data for the the GUI window.
+   * If the GUI wants to display yet other stuff, it's on its own. */
   regData->balance = xaccAccountGetBalance (regData->leader);
   regData->clearedBalance = xaccAccountGetClearedBalance (regData->leader);
   regData->reconciledBalance = xaccAccountGetReconciledBalance (regData->leader);
 
-  /* for income and expense acounts, we have to reverse
-   * the meaning of balance, since, in a dual entry
-   * system, income will show up as a credit to a
-   * bank account, and a debit to the income account.
-   * Thus, positive and negative are interchanged */
-  typo = regData->type & REG_TYPE_MASK;
-  if ((INCOME_REGISTER == typo) ||
-      (EXPENSE_REGISTER == typo)) { 
-    regData->balance = - (regData->balance);
-    regData->clearedBalance = - (regData->clearedBalance);
-    regData->reconciledBalance = - (regData->reconciledBalance);
-  }
-
   /* OK, now tell this specific GUI window to redraw itself ... */
-  if (regData->redraw) {
-     (regData->redraw) (regData);
-   }
+  if (regData->redraw)
+    (regData->redraw) (regData);
+}
 
+/********************************************************************\
+ * refresh all the register windows, but only with the gui callback *
+\********************************************************************/
+
+void 
+xaccRegisterRefreshAllGUI (void)
+{
+   xaccLedgerDisplay *regData;
+   int n;
+
+   if (!fullList) return;
+
+   n = 0; regData = fullList[n];
+   while (regData) {
+     if (regData->redraw)
+       (regData->redraw) (regData);
+      n++; regData = fullList[n];
+   }
 }
 
 /********************************************************************\
@@ -439,31 +477,6 @@ xaccRegisterRefresh (SplitRegister *splitreg)
    }
 }
 
-/********************************************************************\
- * sort of a quick hack involving the layout of the register.
-\********************************************************************/
-
-void 
-xaccRegisterCountHack (SplitRegister *splitreg)
-{
-   xaccLedgerDisplay *regData;
-   int n;
-
-   if (!fullList) return;
-
-   /* find the ledger which contains this register */
-   n = 0; regData = fullList[n];
-   while (regData) {
-      if (splitreg == regData->ledger) {
-        xaccSRCountRows (splitreg, 
-                      xaccAccountGetSplitList (regData->leader),
-                      regData->leader);      
-        return;
-      }
-      n++; regData = fullList[n];
-   }
-}
-
 /********************************************************************\
  * mark dirty *all* register windows which contain this account      * 
 \********************************************************************/
@@ -573,19 +586,23 @@ xaccDestroyLedgerDisplay (Account *acc)
    xaccLedgerDisplay *regData;
    int n;
 
+   if (!acc) return;
+
    /* find the single-account window for this account, if any */
    FIND_IN_LIST (xaccLedgerDisplay, regList, acc, leader, regData);
    if (regData) {
       if (regData->destroy) { (regData->destroy) (regData); }
+      xaccLedgerDisplayClose (regData);
    } 
 
    /* find the multiple-account window for this account, if any */
    FIND_IN_LIST (xaccLedgerDisplay, ledgerList, acc, leader, regData);
    if (regData) {
       if (regData->destroy) { (regData->destroy) (regData); }
+      xaccLedgerDisplayClose (regData);
    } 
 
-   /* cruise throught the miscellanous account windows */
+   /* cruise throught the miscellaneous account windows */
    if (!fullList) return;
    n = 0;
    regData = fullList[n];
@@ -595,6 +612,8 @@ xaccDestroyLedgerDisplay (Account *acc)
       got_one = ledgerIsMember (regData, acc);
       if (got_one) {
          if (regData->destroy) { (regData->destroy) (regData); }
+         xaccLedgerDisplayClose (regData);
+         n = -1;  /* since the above alters this list */
       }
       n++;
       regData = fullList[n];
@@ -606,24 +625,23 @@ xaccDestroyLedgerDisplay (Account *acc)
  *   frees memory allocated for an regWindow, and other cleanup     *
  *   stuff                                                          *
  *                                                                  *
- * Args:   mw - the widget that called us                           *
- *         cd - regData - the data struct for this register         *
- *         cb -                                                     *
+ * Args:   regData - ledger display structure                       *
  * Return: none                                                     *
 \********************************************************************/
 void 
 xaccLedgerDisplayClose (xaccLedgerDisplay *regData)
 {
-  Account *acc = regData->leader;
+  Account *acc;
   
-  /* Save any unsaved changes */
-  xaccSRSaveRegEntry (regData->ledger);
+  if (!regData) return;
+  acc = regData->leader;
 
-  /* refresh the register windows if there were changes */
-  xaccSRRedrawRegEntry (regData->ledger);
+  /* Save any unsaved changes */
+  if (xaccSRSaveRegEntry (regData->ledger, NULL))
+    xaccSRRedrawRegEntry (regData->ledger);
 
   xaccDestroySplitRegister (regData->ledger);
-  
+
   /* whether this is a single or multi-account window, remove it */
   REMOVE_FROM_LIST (xaccLedgerDisplay, regList, acc, leader);
   REMOVE_FROM_LIST (xaccLedgerDisplay, ledgerList, acc, leader);
