@@ -1,7 +1,7 @@
 /********************************************************************\
  * Account.c -- the Account data structure                          *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997, 1998 Linas Vepstas                           *
+ * Copyright (C) 1997, 1998, 1999 Linas Vepstas                     *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -34,12 +34,22 @@
 #include "GroupP.h"
 #include "date.h"
 #include "messages.h"
+#include "Queue.h"
 #include "Transaction.h"
 #include "TransactionP.h"
 #include "util.h"
 
+/* The unsafe_ops flag allows certain unsafe manipulations to be 
+ * performed on the data structures. Normally, this is disabled,
+ * as it can lead to scrambled data.
+ * hack alert -- this should be a configurable parameter.
+ */
+int unsafe_ops = 1;
+
 int next_free_unique_account_id = 0;
 
+static short module = MOD_ENGINE; 
+
 #ifndef FALSE
 #define FALSE 0
 #endif
@@ -75,13 +85,14 @@ xaccInitAccount (Account * acc)
 
   acc->flags = 0;
   acc->type  = -1;
+  acc->accInfo = NULL;
   
-  acc->accountName = NULL;
-  acc->accountCode = NULL;
-  acc->description = NULL;
-  acc->notes       = NULL;
-  acc->currency    = NULL;
-  acc->security    = NULL;
+  acc->accountName = strdup("");
+  acc->accountCode = strdup("");
+  acc->description = strdup("");
+  acc->notes       = strdup("");
+  acc->currency    = strdup("");
+  acc->security    = strdup("");
   
   acc->numSplits   = 0;
   acc->splits      = (Split **) _malloc (sizeof (Split *));
@@ -114,16 +125,10 @@ xaccFreeAccount( Account *acc )
 
   if (NULL == acc) return;
     
-  /* recursively free children */
+  /* First, recursively free children */
   xaccFreeAccountGroup (acc->children);
-
-  if (acc->accountName) free (acc->accountName);
-  if (acc->accountCode) free (acc->accountCode);
-  if (acc->description) free (acc->description);
-  if (acc->notes) free (acc->notes);
-  if (acc->currency) free (acc->currency);
-  if (acc->security) free (acc->security);
   
+  /* Next, clean up the splits */
   /* any split pointing at this account needs to be unmarked */
   for (i=0; i<acc->numSplits; i++) {
     s = acc->splits[i];
@@ -148,6 +153,17 @@ xaccFreeAccount( Account *acc )
   acc->splits = NULL;
   acc->numSplits = 0;
   
+  /* Finally, clean up the account info */
+  if (acc->accInfo) xaccFreeAccInfo (acc->accInfo); 
+  acc->accInfo = NULL;
+
+  if (acc->accountName) free (acc->accountName);
+  if (acc->accountCode) free (acc->accountCode);
+  if (acc->description) free (acc->description);
+  if (acc->notes) free (acc->notes);
+  if (acc->currency) free (acc->currency);
+  if (acc->security) free (acc->security);
+
   /* zero out values, just in case stray 
    * pointers are pointing here. */
 
@@ -188,7 +204,7 @@ void
 xaccAccountCommitEdit (Account *acc)
 {
    if (!acc) return;
-   acc->changed = 1;
+   acc->changed |= ACC_INVALIDATE_ALL;
    acc->open = 0;
 }
 
@@ -205,6 +221,16 @@ xaccGetAccountID (Account *acc)
 /********************************************************************\
 \********************************************************************/
 
+char
+xaccGetAccountFlags (Account *acc)
+{
+  if (!acc) return -1;
+  return acc->flags;
+}
+
+/********************************************************************\
+\********************************************************************/
+
 #define CHECK(acc) {					\
    if (0 == acc->open) {				\
       /* not today, some day in the future ... */	\
@@ -248,7 +274,7 @@ disable for now till we figure out what the right thing is.
 */
 
   /* mark the account as having changed */
-  acc -> changed = TRUE;
+  acc -> changed |= ACC_INVALIDATE_ALL;
 
   /* if this split belongs to another acount, remove it from 
    * there first.  We don't want to ever leave the system
@@ -331,7 +357,7 @@ disable for now till we figure out what the right thing is.
 
 void
 xaccAccountRemoveSplit ( Account *acc, Split *split )
-  {
+{
   int  i,j;
 
   if (!acc) return;
@@ -342,7 +368,7 @@ xaccAccountRemoveSplit ( Account *acc, Split *split )
   CHECK (acc);
 
   /* mark the account as having changed */
-  acc -> changed = TRUE;
+  acc -> changed |= ACC_INVALIDATE_ALL;
   
   for( i=0,j=0; j<acc->numSplits; i++,j++ ) {
     acc->splits[i] = acc->splits[j];
@@ -367,8 +393,8 @@ xaccAccountRemoveSplit ( Account *acc, Split *split )
  * The way the computation is done depends on whether the partial
  * balances are for a monetary account (bank, cash, etc.) or a 
  * certificate account (stock portfolio, mutual fund).  For bank
- * accounts, the invarient amount is the dollar amount. For share
- * accounts, the invarient amount is the number of shares. For
+ * accounts, the invariant amount is the dollar amount. For share
+ * accounts, the invariant amount is the number of shares. For
  * share accounts, the share price fluctuates, and the current 
  * value of such an account is the number of shares times the current 
  * share price.
@@ -401,7 +427,8 @@ xaccAccountRecomputeBalance( Account * acc )
   Split *split, *last_split = NULL;
   
   if( NULL == acc ) return;
-  if (FALSE == acc->changed) return;
+  if (0x0 == (ACC_INVALID_BALN & acc->changed)) return;
+  acc->changed &= ~ACC_INVALID_BALN;
 
   split = acc->splits[0];
   while (split) {
@@ -421,8 +448,8 @@ xaccAccountRecomputeBalance( Account * acc )
       dreconciled_balance += amt * (split->share_price);
     }
 
-    /* For bank accounts, the invarient subtotal is the dollar
-     * amount.  For stock accoounts, the invarient is the share amount */
+    /* For bank accounts, the invariant subtotal is the dollar
+     * amount.  For stock accounts, the invariant is the share amount */
     if ( (STOCK == acc->type) || ( MUTUAL == acc->type) ) {
       split -> share_balance = share_balance;
       split -> share_cleared_balance = share_cleared_balance;
@@ -438,6 +465,8 @@ xaccAccountRecomputeBalance( Account * acc )
       split -> cleared_balance = dcleared_balance;
       split -> reconciled_balance = dreconciled_balance;
     }
+    /* invalidate the cost basis; this has to be computed with other routine */
+    split -> cost_basis = 0.0;
 
     last_split = split;
     i++;
@@ -463,6 +492,46 @@ xaccAccountRecomputeBalance( Account * acc )
   return;
 }
 
+/********************************************************************\
+\********************************************************************/
+
+void
+xaccAccountRecomputeCostBasis( Account * acc )
+{
+  int  i = 0; 
+  double  amt = 0.0;
+  Split *split = NULL;
+  Queue *q;
+  
+  if( NULL == acc ) return;
+  if (0x0 == (ACC_INVALID_COSTB & acc->changed)) return;
+  acc->changed &= ~ACC_INVALID_COSTB;
+
+  /* create the FIFO queue */
+  q = xaccMallocQueue ();
+
+  /* loop over all splits in this account */
+  split = acc->splits[0];
+  while (split) {
+
+    /* positive amounts are a purchase, negative are a sale. 
+     * Use FIFO accounting: purchase to head, sale from tail. */
+    amt = split->damount;
+    if (0.0 < amt) {
+       xaccQueuePushHead (q, split);
+    } else 
+    if (0.0 > amt) {
+       xaccQueuePopTailShares (q, -amt);
+    }
+    split->cost_basis = xaccQueueGetValue (q);
+
+    i++;
+    split = acc->splits[i];
+  }
+
+  xaccFreeQueue (q);
+}
+
 /********************************************************************\
  * xaccCheckDateOrder                                               *
  *   check this split to see if the date is in correct order        *
@@ -509,10 +578,10 @@ xaccCheckDateOrder (Account * acc, Split *split )
 
   /* figure out if the transactions are out of order */
   if (NULL != prevSplit) {
-    if( xaccTransOrder (&(prevSplit->parent), &(split->parent)) >0 ) outOfOrder = TRUE;
+    if( xaccSplitDateOrder (&prevSplit, &split) > 0 ) outOfOrder = TRUE;
   }
   if (NULL != nextSplit) {
-    if( xaccTransOrder (&(split->parent), &(nextSplit->parent)) >0 ) outOfOrder = TRUE;
+    if( xaccSplitDateOrder (&split, &nextSplit) > 0 ) outOfOrder = TRUE;
   }
 
   /* take care of re-ordering, if necessary */
@@ -771,6 +840,7 @@ xaccConsolidateTransactions (Account * acc)
 {
    Split *sa, *sb;
    Transaction *ta, *tb;
+   Timespec ts;
    int i,j;
    int retval;
 
@@ -784,27 +854,70 @@ xaccConsolidateTransactions (Account * acc)
          sb = acc->splits[j];
          tb = sb->parent;
 
+         /* A single transaction can have multiple splits in the same
+          * account.  For instance, a split deposit in AccountA of two
+          * checks from AccountB creates two splits in AccountB with
+          * the same parent transaction.  Skip this case.
+          */
+         if (ta == tb) continue;
+
          /* if no match, then continue on in the loop.
           * we really must match everything to get a duplicate */
          retval = xaccTransMatch (&ta, &tb);
          if (retval) continue;
 
          /* OK, looks like the two splits are a matching pair. 
-          * Blow one of them, and its entie associated transaction, away. 
-          * (We blow away the trasnaction because not only do the splits 
-          * match, but so do all of thier partner-splits. )
+          * Blow one of them, and its entire associated transaction, away.
+          * (We blow away the transaction because not only do the splits 
+          * match, but so do all of their partner-splits.)
+          *
+          * But, before we blow it away, we go through each split and
+          * update the reconciled flag and date of the split in the
+          * remaining transaction with those in the one being deleted,
+          * but only if the remaining transaction has an NREC reconciled
+          * flag. In other words, the two splits match on everything 
+          * but the reconciled flags and dates, so we assume that the
+          * one which is not NREC is 'more correct'. This is true in
+          * the case of importing two QIF files with overlapping
+          * transactions. Each file will have one 'half' of the
+          * transaction, but the other half will be generated by the
+          * QIF importing routines, but with default values for the
+          * reconciled data. When we load the other file, we need to
+          * replace the generated 'half' with the real one.
           */
 
-          xaccTransBeginEdit (tb, 1);
-          xaccTransDestroy (tb);
-          xaccTransCommitEdit (tb);
+         xaccTransBeginEdit (ta, 1);
 
-          /* It should be safe to just "break" here, as all splits
-           * wwith index i or less have been checked already and couldn't
-           * have been dupes.  So index i is still valid, although j is 
-           * not.  Note that numSplits changed ...
-           */
-          break;
+         for (i=xaccTransCountSplits(ta); i>0; i--) {
+           sa = xaccTransGetSplit (ta, i - 1);
+
+           /* If the remaining split is reconciled as something
+              other than NREC, just leave it alone. */
+           if (xaccSplitGetReconcile(sa) != NREC)
+             continue;
+
+           /* We get the matching split using the tickee value
+              generated by xaccTransMatch above. */
+           sb = xaccTransGetSplit(tb, sa->tickee);
+
+           xaccSplitSetReconcile (sa, xaccSplitGetReconcile(sb));
+           xaccSplitGetDateReconciledTS (sb, &ts);
+           xaccSplitSetDateReconciledTS (sa, &ts);
+         }
+
+         xaccTransCommitEdit (ta);
+
+         xaccTransBeginEdit (tb, 1);
+         xaccTransDestroy (tb);
+         xaccTransCommitEdit (tb);
+
+
+         /* It should be safe to just "break" here, as all splits
+          * with index i or less have been checked already and couldn't
+          * have been dupes.  So index i is still valid, although j is 
+          * not.  Note that numSplits changed ...
+          */
+         break;
       }
    }
 }
@@ -818,18 +931,20 @@ xaccAccountSetType (Account *acc, int tip)
    if (!acc) return;
    CHECK (acc);
 
-   /* After an account type has been set, it cannot be changed */
-   if (-1 < acc->type) {
-      printf ("Error: xaccAccountSetType(): "
-              "the type of the account cannot be changed "
-              "after its been set! \n"
-             );
-      return;
-   }
-
    /* refuse invalid account types */
    if (NUM_ACCOUNT_TYPES <= tip) return;
+
+   /* Don't bother if it already is. */
+   if (acc->type == tip) return;
+
    acc->type = tip;
+
+   /* initialize the auxilliary account info as well */
+   if (acc->accInfo) xaccFreeAccInfo (acc->accInfo); 
+   acc->accInfo = xaccMallocAccInfo (tip);
+
+   /* Changing the type can change the way the balances are computed. */
+   xaccAccountRecomputeBalance(acc);
 }
 
 void 
@@ -891,11 +1006,17 @@ xaccAccountSetCurrency (Account *acc, char *str)
    CHECK (acc);
 
    if (acc->currency && (0x0 != acc->currency[0])) {
-      printf ("Error: xacAccountSetCurrency(): "
-              "the currency denomination of an account "
-              "cannot be changed!\n"
-             );
-      return;
+      if (unsafe_ops) {
+         PWARN ("xaccAccountSetCurrency(): "
+                "it is dangerous to change the currency denomination of an account! \n"
+                "\tAccount=%s old currency=%s new currency=%s \n",
+                acc->accountName, acc->currency, str);
+      } else {
+         PERR ("xaccAccountSetCurrency(): "
+               "the currency denomination of an account cannot be changed!\n"
+                "\tAccount=%s \n", acc->accountName);
+         return;
+      }
    }
    /* free the zero-length string */
    if (acc->currency) free (acc->currency);
@@ -909,11 +1030,17 @@ xaccAccountSetSecurity (Account *acc, char *str)
    CHECK (acc);
 
    if (acc->security && (0x0 != acc->security[0])) {
-      printf ("Error: xacAccountSetCurrency(): "
-              "the security traded in an account "
-              "cannot be changed!\n"
-             );
-      return;
+      if (unsafe_ops) {
+         PWARN ("xaccAccountSetSecurity(): "
+                "it is dangerous to change the security denomination of an account! \n"
+                "\tAccount=%s old security=%s new security=%s \n",
+                acc->accountName, acc->security, str);
+      } else {
+         PERR ("xaccAccountSetSecurity(): "
+               "the security denomination of an account cannot be changed!\n"
+                "\tAccount=%s \n", acc->accountName);
+         return;
+      }
    }
    /* free the zero-length string */
    if (acc->security) free (acc->security);
@@ -923,6 +1050,13 @@ xaccAccountSetSecurity (Account *acc, char *str)
 /********************************************************************\
 \********************************************************************/
 
+AccInfo *
+xaccAccountGetAccInfo (Account *acc)
+{
+   if (!acc) return NULL;
+   return (acc->accInfo);
+}
+
 AccountGroup *
 xaccAccountGetChildren (Account *acc)
 {
@@ -937,6 +1071,13 @@ xaccAccountGetParent (Account *acc)
    return (acc->parent);
 }
 
+Account *
+xaccAccountGetParentAccount (Account * acc)
+{
+  if (!acc) return NULL;
+  return xaccGroupGetParentAccount(acc->parent);
+}
+
 int
 xaccAccountGetType (Account *acc)
 {
@@ -951,6 +1092,67 @@ xaccAccountGetName (Account *acc)
    return (acc->accountName);
 }
 
+char *
+xaccAccountGetFullName(Account *account, const char separator)
+{
+  Account *a;
+  char *fullname;
+  char *name;
+  char *p;
+  int length;
+
+  if (account == NULL)
+  {
+    fullname = strdup("");
+    assert(fullname != NULL);
+    return fullname;
+  }
+
+  /* Figure out how much space is needed */
+  length = 0;
+  a = account;
+  while (a != NULL)
+  {
+    name = xaccAccountGetName(a);
+
+    length += strlen(name) + 1; /* plus one for the separator */
+
+    a = xaccAccountGetParentAccount(a);
+  }
+
+  /* length has one extra separator in it, that's ok, because it will
+   * hold the null character at the end. */
+
+  /* allocate the memory */
+  fullname = malloc(length * sizeof(char));
+  assert(fullname != 0);
+
+  /* go to end of string */
+  p = fullname + length - 1;
+
+  /* put in the null character and move to the previous char */
+  *p-- = 0;
+
+  a = account;
+  while (a != NULL)
+  {
+    name = xaccAccountGetName(a);
+    length = strlen(name);
+
+    /* copy the characters going backwards */
+    while (length > 0)
+      *p-- = name[--length];
+
+    a = xaccAccountGetParentAccount(a);
+
+    /* if we're not at the root, add another separator */
+    if (a != NULL)
+      *p-- = separator;
+  }
+
+  return fullname;
+}
+
 char *
 xaccAccountGetCode (Account *acc)
 {
@@ -1030,4 +1232,50 @@ xaccAccountGetNumSplits (Account *acc)
    return (acc->numSplits);
 }
 
+/********************************************************************\
+\********************************************************************/
+
+Account * 
+IthAccount (Account **list, int i)
+{
+   if (!list || 0 > i) return NULL;
+   return list[i];
+}
+
+/********************************************************************\
+\********************************************************************/
+
+gncBoolean
+xaccAccountsHaveCommonCurrency(Account *account_1, Account *account_2)
+{
+  if ((account_1 == NULL) || (account_2 == NULL))
+    return GNC_F;
+
+  return xaccIsCommonCurrency(account_1->currency, account_1->security,
+			      account_2->currency, account_2->security);
+}
+
+/********************************************************************\
+\********************************************************************/
+
+gncBoolean
+xaccAccountHasAncestor (Account *account, Account * ancestor)
+{
+  Account *parent;
+
+  if ((account == NULL) || (ancestor == NULL))
+    return GNC_F;
+
+  parent = xaccAccountGetParentAccount(account);
+  while (parent != NULL)
+  {
+    if (parent == ancestor)
+      return GNC_T;
+
+    parent = xaccAccountGetParentAccount(parent);
+  }
+
+  return GNC_F;
+}
+
 /*************************** END OF FILE **************************** */
