@@ -1,7 +1,7 @@
 /********************************************************************\
  * Group.c -- the main data structure of the program                *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997, 1998 Linas Vepstas                           *
+ * Copyright (C) 1997-2000 Linas Vepstas                            *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -24,6 +24,7 @@
 \********************************************************************/
 
 #include <assert.h>
+#include <string.h>
 
 #include "config.h"
 
@@ -33,18 +34,11 @@
 #include "GroupP.h"
 #include "TransactionP.h"
 #include "util.h"
-
-#ifndef FALSE
-#define FALSE 0
-#endif
-
-#ifndef TRUE
-#define TRUE 1
-#endif
+#include "gnc-common.h"
 
 /********************************************************************\
  * Because I can't use C++ for this project, doesn't mean that I    *
- * can't pretend too!  These functions perform actions on the       *
+ * can't pretend to!  These functions perform actions on the        *
  * AccountGroup data structure, in order to encapsulate the         *
  * knowledge of the internals of the AccountGroup in one file.      *
 \********************************************************************/
@@ -53,9 +47,8 @@
 \********************************************************************/
 void
 xaccInitializeAccountGroup (AccountGroup *grp)
-  {
-  grp->saved       = TRUE;
-  grp->new         = FALSE;
+{
+  grp->saved       = GNC_T;
   
   grp->parent      = NULL;
   grp->numAcc      = 0;
@@ -63,19 +56,19 @@ xaccInitializeAccountGroup (AccountGroup *grp)
   grp->account[0]  = NULL;   /* null-terminated array */
   
   grp->balance     = 0.0;
-  }
+}
 
 /********************************************************************\
 \********************************************************************/
 AccountGroup *
 xaccMallocAccountGroup( void )
-  {
+{
   AccountGroup *grp = (AccountGroup *)_malloc(sizeof(AccountGroup));
   
   xaccInitializeAccountGroup (grp);
 
   return grp;
-  }
+}
 
 /********************************************************************\
 \********************************************************************/
@@ -97,7 +90,7 @@ xaccFreeAccountGroup( AccountGroup *grp )
   grp->numAcc      = 0;
   grp->account     = NULL;
   grp->balance     = 0.0;
-
+  
   _free(grp);
 }
 
@@ -109,13 +102,22 @@ xaccAccountGroupMarkSaved (AccountGroup *grp)
    int i;
 
    if (!grp) return;
-   grp->saved = TRUE;
+   grp->saved = GNC_T;
 
    for (i=0; i<grp->numAcc; i++) {
       xaccAccountGroupMarkSaved (grp->account[i]->children); 
    }
 }
 
+/********************************************************************\
+\********************************************************************/
+void
+xaccAccountGroupMarkNotSaved (AccountGroup *grp)
+{
+   if (!grp) return;
+   grp->saved = GNC_F;
+}
+
 /********************************************************************\
 \********************************************************************/
 int
@@ -125,7 +127,7 @@ xaccAccountGroupNotSaved (AccountGroup *grp)
    int i;
 
    if (!grp) return 0;
-   if (FALSE == grp->saved) return 1;
+   if (GNC_F == grp->saved) return 1;
 
    for (i=0; i<grp->numAcc; i++) {
       not_saved = xaccAccountGroupNotSaved (grp->account[i]->children); 
@@ -156,7 +158,49 @@ xaccGetNumAccounts ( AccountGroup *root )
 }
 
 /********************************************************************\
- * Fetch an account, given only it's ID number                      *
+ * Get all of the accounts, including subaccounts                   *
+\********************************************************************/
+
+int
+xaccFillInAccounts ( AccountGroup *root, Account **arr )
+{
+  int num_acc = 0;
+  int i,j;
+
+  if (!root || !arr) return 0;
+
+  num_acc = root->numAcc;
+  for (i=0, j=0; i<num_acc; i++) {
+    arr[j] = root->account[i];
+    j++;
+    j += xaccFillInAccounts (root->account[i]->children, &arr[j]);
+  }
+
+  arr[j] = NULL;
+  return j;
+}
+
+Account ** 
+xaccGetAccounts ( AccountGroup *root )
+{
+  Account **arr;
+  int num_acc = 0;
+  int num_done;
+
+  if (NULL == root) return NULL;
+
+  num_acc = xaccGetNumAccounts (root);
+  arr = (Account **) malloc ((num_acc+1)*sizeof (Account *));
+
+  num_done = xaccFillInAccounts (root, arr);
+  assert (num_done == num_acc);
+
+  arr[num_acc] = NULL;
+  return arr;
+}
+
+/********************************************************************\
+ * Fetch an account, given only its ID number                      *
 \********************************************************************/
 
 Account *
@@ -211,7 +255,7 @@ xaccGetAccountRoot (Account * acc)
 }
 
 /********************************************************************\
- * Fetch an account, given only it's ID number                      *
+ * Fetch an account, given only its ID number                      *
 \********************************************************************/
 
 Account *
@@ -233,7 +277,7 @@ xaccGetPeerAccountFromID ( Account *acc, int acc_id )
 }
 
 /********************************************************************\
- * Fetch an account, given it's name                                *
+ * Fetch an account, given its name                                *
 \********************************************************************/
 
 Account *
@@ -262,7 +306,84 @@ xaccGetAccountFromName ( AccountGroup *root, const char * name )
 }
 
 /********************************************************************\
- * Fetch an account, given it's name                                *
+ * Fetch an account, given its full name                            *
+\********************************************************************/
+
+Account *
+xaccGetAccountFromFullName (AccountGroup *root,
+                            const char *name,
+                            const char separator)
+{
+  Account *account;
+  Account *found;
+  char *p;
+  int i;
+
+  if (NULL == root) return NULL;
+  if (NULL == name) return NULL;
+
+  p = (char *) name;
+  found = NULL;
+
+  while (1)
+  {
+    /* Look for the first separator. */
+    p = strchr(p, separator);
+
+    /* If found, switch it to the null char. */
+    if (p != NULL)
+      *p = 0;
+
+    /* Now look for that name in the children. */
+    for (i = 0; i < root->numAcc; i++) {
+      account = root->account[i];
+      if (safe_strcmp(account->accountName, name) == 0)
+      {
+        /* We found an account.
+         * If p == NULL, there is nothing left
+         * in the name, so just return the account.
+         * We don't need to put back the separator,
+         * because it was never erased (p == NULL). */
+        if (p == NULL)
+          return account;
+
+        /* There's stuff left to search for.
+         * Search recursively after the separator. */
+        found = xaccGetAccountFromFullName(account->children,
+                                           p + 1, separator);
+
+        /* If we found the account, break out. */
+        if (found != NULL)
+          break;
+      }
+    }
+
+    /* If found != NULL, an account was found. */
+    /* If p == NULL, there are no more separators left. */
+
+    /* Put back the separator. */
+    if (p != NULL)
+      *p = separator;
+
+    /* If we found the account, return it. */
+    if (found != NULL)
+      return found;
+
+    /* We didn't find the account. If there
+     * are no more separators, return NULL. */
+    if (p == NULL)
+      return NULL;
+
+    /* If we are here, we didn't find anything and there
+     * must be more separators. So, continue looking with
+     * a longer name, in case there is a name with the
+     * separator character in it. */
+    p++;
+  }
+}
+
+/********************************************************************\
+ * Fetch an account, given its name                                *
 \********************************************************************/
 
 Account *
@@ -283,6 +404,29 @@ xaccGetPeerAccountFromName ( Account *acc, const char * name )
   return peer_acc;
 }
 
+/********************************************************************\
+ * Fetch an account, given its full name                            *
+\********************************************************************/
+
+Account *
+xaccGetPeerAccountFromFullName ( Account *acc, const char * name,
+                                 const char separator )
+{
+  AccountGroup * root;
+  Account *peer_acc;
+
+  if (NULL == acc) return NULL;
+  if (NULL == name) return NULL;
+
+  /* first, find the root of the account group structure */
+  root = xaccGetAccountRoot (acc);
+
+  /* now search all acounts hanging off the root */
+  peer_acc = xaccGetAccountFromFullName (root, name, separator);
+
+  return peer_acc;
+}
+
 /********************************************************************\
 \********************************************************************/
 
@@ -304,7 +448,7 @@ xaccRemoveGroup (AccountGroup *grp)
    grp = acc -> parent;
    if (!grp) return;
 
-   grp->saved = FALSE;
+   grp->saved = GNC_F;
 }
 
 /********************************************************************\
@@ -337,7 +481,7 @@ xaccRemoveAccount (Account *acc)
    nacc --;
    arr[nacc] = NULL;
    grp->numAcc = nacc;
-   grp->saved = FALSE;
+   grp->saved = GNC_F;
 
    /* if this was the last account in a group, delete
     * the group as well (unless its a root group) */
@@ -391,7 +535,7 @@ xaccGroupInsertAccount( AccountGroup *grp, Account *acc )
     if (grp == acc->parent) ralo = 0;
     xaccRemoveAccount (acc);
   }
-  grp->saved = FALSE;
+  grp->saved = GNC_F;
 
   /* set back-pointer to the accounts parent */
   acc->parent = grp;
@@ -592,10 +736,11 @@ xaccGroupDepthAutoCode (AccountGroup *grp)
 void
 xaccGroupAutoCode (AccountGroup *grp, int depth)
 {
-   int i;
+   int i, n;
    if (!grp || (0>depth)) return;
 
-   for (i=0; i<grp->numAcc; i++) {
+   n = grp->numAcc;
+   for (i=0; i<n; i++) {
       Account *acc = grp->account[i];
       xaccAccountAutoCode (acc, depth);
       xaccGroupAutoCode (acc->children, depth);
@@ -609,18 +754,22 @@ void
 xaccConcatGroups (AccountGroup *togrp, AccountGroup *fromgrp)
 {
    Account * acc;
-   int i;
+   int numAcc;
 
    if (!togrp) return;
    if (!fromgrp) return;
    
-   for (i=0; i<fromgrp->numAcc; i++) {
-      acc = fromgrp->account[i];
+   /* The act of inserting the account into togrp also causes
+    * it to automatically be deleted from fromgrp. But use a
+    * saved copy of fromgrp's numAcc member since, after the
+    * last insertion, fromgrp will be pointing to freed memory.
+    */
+   numAcc = fromgrp->numAcc;
+   while (numAcc) {
+      acc = fromgrp->account[0];
       xaccGroupInsertAccount (togrp, acc);
-      fromgrp->account[i] = NULL;
+      numAcc--;
    }
-   fromgrp->account[0] = NULL;
-   fromgrp->numAcc = 0;
 }
 
 /********************************************************************\
@@ -630,7 +779,7 @@ void
 xaccMergeAccounts (AccountGroup *grp)
 {
    Account *acc_a, *acc_b;
-   int i,j, k;
+   int i, j, k;
 
    if (!grp) return;
    
@@ -659,7 +808,6 @@ xaccMergeAccounts (AccountGroup *grp)
                } else {
                   xaccConcatGroups (ga, gb);
                   acc_b->children = NULL;
-                  xaccFreeAccountGroup (gb);
                }
             }
 
@@ -726,6 +874,13 @@ xaccGroupGetAccount (AccountGroup *grp, int i)
    return (grp->account[i]);
 }
 
+Account *
+xaccGroupGetParentAccount (AccountGroup * grp)
+{
+  if (!grp) return NULL;
+  return grp->parent;
+}
+
 double
 xaccGroupGetBalance (AccountGroup * grp)
 {
@@ -751,4 +906,102 @@ xaccGroupGetDepth (AccountGroup *grp)
    return maxdepth;
 }
 
+/********************************************************************\
+\********************************************************************/
+
+void
+xaccGroupBeginStagedTransactionTraversals (AccountGroup *grp) 
+{
+  unsigned int numAcc;
+  unsigned int i;
+  
+  if (!grp) return;
+  
+  numAcc = grp->numAcc;
+  for(i = 0; i < numAcc; i++) {
+    unsigned int n = 0;
+    Account *acc;
+    Split *s = NULL;
+    acc = xaccGroupGetAccount(grp, i);
+    
+    if(!acc) return;
+    
+    /* recursively do sub-accounts */
+    xaccGroupBeginStagedTransactionTraversals(acc->children);
+    
+    s = acc->splits[0];
+    while (s) {
+      Transaction *trans = s->parent;
+      trans->marker = 0;
+      n++;
+      s = acc->splits[n];
+    }
+  }
+}
+
+int
+xaccAccountStagedTransactionTraversal (Account *acc,
+                        unsigned int stage,
+                        int (*callback)(Transaction *t, void *cb_data),
+                        void *cb_data) 
+{
+  unsigned int n = 0;
+  Split *s = NULL;
+  
+  if(!acc) return 0;
+  
+  s = acc->splits[0];
+  if (callback) {
+    int retval;
+    while (s) {
+      Transaction *trans = s->parent;
+      if (trans && (trans->marker < stage)) {
+        trans->marker = stage;
+        retval = callback(trans, cb_data);
+        if (retval) return retval;
+      }
+      n++;
+      s = acc->splits[n];
+    }
+  } else {
+    while (s) {
+      Transaction *trans = s->parent;
+      if (trans && (trans->marker < stage)) {
+        trans->marker = stage;
+      }
+      n++;
+      s = acc->splits[n];
+    }
+  }
+  return 0;
+}
+
+
+int
+xaccGroupStagedTransactionTraversal(AccountGroup *grp,
+                                    unsigned int stage,
+                                    int (*callback)(Transaction *t, void *cb_data),
+                                    void *cb_data) 
+{
+  unsigned int numAcc;
+  unsigned int i;
+  
+  if (!grp) return 0;
+  
+  numAcc = grp->numAcc;
+  for(i = 0; i < numAcc; i++) {
+    int retval;
+    Account *acc;
+    acc = xaccGroupGetAccount(grp, i);
+    
+    /* recursively do sub-accounts */
+    retval = xaccGroupStagedTransactionTraversal 
+               (acc->children, stage, callback, cb_data);
+    if (retval) return retval;
+    retval = xaccAccountStagedTransactionTraversal (acc, stage, callback, cb_data);
+    if (retval) return retval;
+  }
+  return 0;
+}
+
 /****************** END OF FILE *************************************/
