@@ -28,7 +28,6 @@
 \********************************************************************/
 
 /* hack alert -- stocks probably not handled correctly
- * this stuff still needs work 
  * also, check out a stock split tooo
  */
 
@@ -50,10 +49,28 @@
 #define WFLAGS  (O_WRONLY | O_CREAT | O_TRUNC)
 #define RFLAGS  O_RDONLY
 
+/* Cleared values found in QIF files */
+#define QRECCLEAR '*'     /* Cleared, per Quicken */
+#define QRECREC   'x'     /* Reconciled, per Quicken */
+#define QRECRECM  'X'     /* Reconciled, per MS Money */
+#define QRECBUDG  '?'     /* Budgeted amount, per CBB */
+#define QRECBUDP  '!'     /* OLD Budgeted amount per CBB */
+
 /** GLOBALS *********************************************************/
 
+/* XXX hack alert -- this default currency should be made configurable
+ * somewhere, somehow, somewhen
+ */
+static char *gnc_qif_import_currency = "USD";
+
 static int          error_code=0; /* error code, if error occurred */
 
+/* This static indicates the debugging module that this .o belongs to.  */
+static short        module = MOD_IO;
+static int FindDateDelimiter (char *str);
+static void TryToFixDate (struct tm *date);
+static int FavorDateType (int value);
+
 /*******************************************************/
 
 int 
@@ -120,16 +137,13 @@ char * xaccReadQIFDiscard( int fd )
      if (NSTRNCMP(qifline, "^^")) {
         qifline = xaccReadQIFLine (fd);
         return qifline;
-     } else
-     if (NSTRNCMP(qifline, "^\n")) {
+     } else if (NSTRNCMP(qifline, "^\n")) {
         qifline = xaccReadQIFLine (fd);
         return qifline;
-     } else
-     if (NSTRNCMP(qifline, "^\r")) {
+     } else if (NSTRNCMP(qifline, "^\r")) {
         qifline = xaccReadQIFLine (fd);
         return qifline;
-     } else
-     if ('!' == qifline [0]) return qifline;
+     } else if ('!' == qifline [0]) return qifline;
      qifline = xaccReadQIFLine (fd);
    }
    return qifline;
@@ -168,66 +182,76 @@ char * xaccReadQIFCategory (int fd, Account * acc)
    char * qifline;
    char * tmp;
 
-
    if (!acc) return NULL;
 
    xaccAccountBeginEdit (acc, 0);
    xaccAccountSetType (acc, -1);
-   xaccAccountSetName (acc, "");
-   xaccAccountSetDescription (acc, "");
-   xaccAccountSetNotes (acc, "");
 
    qifline = xaccReadQIFLine (fd);
    if (!qifline) return NULL;
    if ('!' == qifline [0]) return qifline;
 
    /* scan for account name, description, type */
-   while (qifline) {
+   while (qifline) 
+     {
 
-     /* N == Name */
-     if ('N' == qifline [0]) {
-        XACC_PREP_STRING (tmp);
-        xaccAccountSetName (acc, tmp);
-     } else 
+       switch (qifline[0]) {
+	 /* N == Name */
+         case 'N':
+	   XACC_PREP_STRING (tmp);
+	   xaccAccountSetName (acc, tmp);
+	   break;
 
-     /* D == Description */
-     if ('D' == qifline [0]) {
-        XACC_PREP_STRING (tmp);
-        xaccAccountSetDescription (acc, tmp);
-     } else 
+	 /* D == Description */
+         case 'D':
+	   XACC_PREP_STRING (tmp);
+	   xaccAccountSetDescription (acc, tmp);
+	   break;
+  
+	 /* T == Taxable -- this income is taxable */
+         case 'T':
+	   break;
 
-     /* T == Taxable -- this income is taxable */
-     if ('T' == qifline [0]) {
-     } else 
+	 /* E == Expense Category */
+         case 'E':
+	   xaccAccountSetType (acc, EXPENSE);
+	   break;
 
-     /* E == Expense Category */
-     if ('E' == qifline [0]) {
-        xaccAccountSetType (acc, EXPENSE);
-     } else 
+	 /* I == Income Category */
+         case 'I':
+	   xaccAccountSetType (acc, INCOME);
+	   break;
 
-     /* I == Income Category */
-     if ('I' == qifline [0]) {
-        xaccAccountSetType (acc, INCOME);
-     } else 
+	 /* R == Tax Rate Indicator; -- some number ... */
+         case 'R':
+	   break;
 
-     /* R == Tax Rate Indicator; -- some number ... */
-     if ('R' == qifline [0]) {
-     } else 
+         /* B == Budget Amount -- not (yet ?) used in GnuCash */
+         case 'B':
+	   break;
 
-     /* check for end-of-transaction marker */
-     if (NSTRNCMP(qifline, "^^")) {
-        break;
-     } else
-     if (NSTRNCMP(qifline, "^\n")) {
-        break;
-     } else
-     if (NSTRNCMP(qifline, "^\r")) {
-        break;
-     } else
-     if ('!' == qifline [0]) break;
+         case '^':
+           break;    /* see below */
 
-     qifline = xaccReadQIFLine (fd);
-   }
+         default:
+           PWARN ("Unknown transaction component %s\n", qifline);
+           break;
+       }
+
+       /* check for end-of-transaction marker */
+       if (NSTRNCMP(qifline, "^^")) {
+	 break;
+       } else
+	 if (NSTRNCMP(qifline, "^\n")) {
+	   break;
+	 } else
+	   if (NSTRNCMP(qifline, "^\r")) {
+	     break;
+	   } else
+	     if ('!' == qifline [0]) break;
+
+       qifline = xaccReadQIFLine (fd);
+     }
 
    xaccAccountCommitEdit (acc);
    return qifline;
@@ -251,63 +275,63 @@ char * xaccReadQIFAccount (int fd, Account * acc)
 
    xaccAccountBeginEdit (acc, 0);
    xaccAccountSetType (acc, -1);
-   xaccAccountSetName (acc, "");
-   xaccAccountSetDescription (acc, "");
-   xaccAccountSetNotes (acc, "");
 
    qifline = xaccReadQIFLine (fd);
    if (!qifline) return NULL;
    if ('!' == qifline [0]) return qifline;
 
    /* scan for account name, description, type */
-   while (qifline) {
-     if ('N' == qifline [0]) {
-        XACC_PREP_STRING (tmp);
-        xaccAccountSetName (acc, tmp);
-     } else 
-     if ('D' == qifline [0]) {
-        XACC_PREP_STRING (tmp);
-        xaccAccountSetDescription (acc, tmp);
-     } else 
-     if ('T' == qifline [0]) {
+   while (qifline) 
+     {
+       switch (qifline[0]) 
+	 {
+	   case 'N':
+	     XACC_PREP_STRING (tmp);
+	     xaccAccountSetName (acc, tmp);
+	     break;
+	   case 'D':
+	     XACC_PREP_STRING (tmp);
+	     xaccAccountSetDescription (acc, tmp);
+	     break;
+	   case 'T':
+	     if (NSTRNCMP (&qifline[1], "Bank")) {
+	       xaccAccountSetType (acc, BANK);
+	     } else
+	       if (NSTRNCMP (&qifline[1], "Cash")) {
+		 xaccAccountSetType (acc, CASH);
+	       } else
+		 if (NSTRNCMP (&qifline[1], "CCard")) {
+		   xaccAccountSetType (acc, CREDIT);
+		 } else
+		   if (NSTRNCMP (&qifline[1], "Invst")) {
+		     xaccAccountSetType (acc, STOCK);
+		   } else
+		     if (NSTRNCMP (&qifline[1], "Oth A")) {
+		       xaccAccountSetType (acc, ASSET);
+		     } else 
+		       if (NSTRNCMP (&qifline[1], "Oth L")) {
+			 xaccAccountSetType (acc, LIABILITY);
+		       } else 
+			 {
+			   printf ("QIF Parse: Unsupported account type %s \n", &qifline[1]);
+			   xaccAccountSetType (acc, -1);    /* hack alert -- */
+			 }
+	   break;
+	 }
 
-        if (NSTRNCMP (&qifline[1], "Bank")) {
-           xaccAccountSetType (acc, BANK);
-        } else
-        if (NSTRNCMP (&qifline[1], "Cash")) {
-           xaccAccountSetType (acc, CASH);
-        } else
-        if (NSTRNCMP (&qifline[1], "CCard")) {
-           xaccAccountSetType (acc, CREDIT);
-        } else
-        if (NSTRNCMP (&qifline[1], "Invst")) {
-           xaccAccountSetType (acc, STOCK);
-        } else
-        if (NSTRNCMP (&qifline[1], "Oth A")) {
-           xaccAccountSetType (acc, ASSET);
-        } else 
-        if (NSTRNCMP (&qifline[1], "Oth L")) {
-           xaccAccountSetType (acc, LIABILITY);
-        } else 
-        {
-           printf ("QIF Parse: Unsupported account type %s \n", &qifline[1]);
-           xaccAccountSetType (acc, -1);    /* hack alert -- */
-        }
-     } else 
-
-     /* check for end-of-transaction marker */
-     if (NSTRNCMP (qifline, "^^")) {
-        break;
-     } else
-     if (NSTRNCMP (qifline, "^\n")) {
-        break;
-     } else
-     if (NSTRNCMP (qifline, "^\r")) {
-        break;
-     } else
-     if ('!' == qifline [0]) break;
-     qifline = xaccReadQIFLine (fd);
-   }
+       /* check for end-of-transaction marker */
+       if (NSTRNCMP (qifline, "^^")) {
+	 break;
+       } else
+	 if (NSTRNCMP (qifline, "^\n")) {
+	   break;
+	 } else
+	   if (NSTRNCMP (qifline, "^\r")) {
+	     break;
+	   } else
+	     if ('!' == qifline [0]) break;
+       qifline = xaccReadQIFLine (fd);
+     }
 
    xaccAccountCommitEdit (acc);
    return qifline;
@@ -327,6 +351,7 @@ char * xaccReadQIFAccList (int fd, AccountGroup *grp, int cat)
    if (!grp) return 0x0;
    do { 
       acc = xaccMallocAccount();
+      xaccAccountSetCurrency (acc, gnc_qif_import_currency);
       if (cat) { 
          qifline = xaccReadQIFCategory (fd, acc);
       } else {
@@ -360,7 +385,7 @@ char * xaccReadQIFAccList (int fd, AccountGroup *grp, int cat)
 
             /* trim off the parent account name ... */
             tok += sizeof(char);  /* get rid of the semi-colon */
-            xaccAccountSetName (acc, str);
+            xaccAccountSetName (acc, tok);
 
             xaccInsertSubAccount( parent, acc );
          } else {
@@ -382,7 +407,7 @@ char * xaccReadQIFAccList (int fd, AccountGroup *grp, int cat)
  *   parses date of the form MM/DD/YY                               *
  *                                                                  * 
  * Args:   str -- pointer to string rep of date                     * 
- * Return: void                                                     * 
+ * Return: time_t -- date in format used by UNIX time_t             * 
 \********************************************************************/
 
 time_t
@@ -391,16 +416,28 @@ xaccParseQIFDate (char * str)
    char * tok;
    time_t secs;
    struct tm dat;
+   int favechar = 0;   /* Which delimiter do we have? */
+   gncBoolean add_2k = GNC_F;
 
-   if (!str) return 0;
-   tok = strchr (str, '/');
+   if (!str) return 0;  /* If the string is null, we're done. */
+
+   /* First, figure out the delimiter. */
+   /* Choices: "." or "-" or "/" */
+   favechar = FindDateDelimiter(str);
+   if (!favechar) return 0;
+
+   tok = strchr (str, favechar);
    if (!tok) return 0;
    *tok = 0x0;
    dat.tm_mon = atoi (str) - 1;
 
    str = tok+sizeof(char);
-   tok = strchr (str, '/');
-   if (!tok) return 0;
+   tok = strchr (str, favechar);
+   if (!tok) {
+     tok = strchr (str, '\'');
+     if (!tok) return 0;
+     add_2k = GNC_T;
+   }
    *tok = 0x0;
    dat.tm_mday = atoi (str);
 
@@ -412,11 +449,18 @@ xaccParseQIFDate (char * str)
    }
    *tok = 0x0;
    dat.tm_year = atoi (str);
+   if (add_2k)
+     dat.tm_year += 2000;
+
+   TryToFixDate(&dat);
 
    /* a quickie Y2K fix: assume two digit dates with
     * a value less than 50 are in the 21st century. */
    if (50 > dat.tm_year) dat.tm_year += 100;
 
+   /* For 4 digit dates */
+   if (dat.tm_year >= 1900) dat.tm_year -= 1900;
+
    dat.tm_sec = 0;
    dat.tm_min = 0;
    dat.tm_hour = 11;
@@ -426,6 +470,124 @@ xaccParseQIFDate (char * str)
    return secs;
 }
 
+
+/********************************************************************\
+ * FindDateDelimiter                                                *
+ *  determines which character is the delimiter for a date          *
+ *  typical would be "MM/DD/YY", "MM-DD-YY", "MM.DD.YY"             *
+ *                                                                  * 
+ * Args:   str -- pointer to string rep of date                     * 
+ * Return: int containing '/', '.', '-', or NULL indicating failure * 
+\********************************************************************/
+
+static int FindDateDelimiter (char *str) {
+  char *tok;
+  if (!str) return 0;
+  tok = strchr (str, '/');
+  if (tok)
+    return '/';
+  tok = strchr (str, '.');
+  if (tok)
+    return '.';
+  tok = strchr (str, '-');
+  if (tok)
+    return '-';
+  return 0;
+}
+
+/********************************************************************\
+ * TryToFixDate                                                     *
+ *  Swaps around date components based on some heuristics           *
+ *                                                                  * 
+ * Args:   date -- pointer to time_t structure                      *
+ * Return: void                                                     *
+\********************************************************************/
+static void TryToFixDate (struct tm *date) 
+{
+  int first, second, third;
+  int st_first, st_second, st_third;
+  int mon, mday, year;
+  int which[5] = {0,0,0,0,0};
+
+  first = date->tm_mon;
+  second = date->tm_mday;
+  third = date->tm_year;
+
+  /* See what sort of date is favored by each component */
+  st_first = FavorDateType(first);
+  st_second = FavorDateType(second);
+  st_third = FavorDateType(third);
+
+  /* Plunk the values down into which[] */
+  which[st_first] = 1;
+  which[st_second] = 2;
+  which[st_third] = 3;
+
+  switch (which[4]) {   /* Year */
+  case 1:
+    year = first;
+    break;
+  case 2:
+    year = second;
+    break;
+  case 3:
+    year = third;
+    break;
+  default:
+    return;   /* No date component looks like a year --> ABORT */
+  }
+
+  switch (which[2]) {   /* month */
+  case 1:
+    mon = first;
+    break;
+  case 2:
+    mon = second;
+    break;
+  case 3:
+    mon = third;
+    break;
+  default:
+    return;   /* No date component looks like a month --> ABORT */
+  }
+
+  switch (which[1]) {   /* mday */
+  case 1:
+    mday = first;
+    break;
+  case 2:
+    mday = second;
+    break;
+  case 3:
+    mday = third;
+    break;
+  default:
+    return;   /* No date component looks like a day of the month - ABORT */
+  }
+
+  date->tm_mon = mon;
+  date->tm_mday = mday;
+  date->tm_year = year;
+
+  return;    
+}
+
+static int FavorDateType (int value) 
+{
+  int favoring;
+
+  favoring = 2;  /* Month */
+  if (value > 30)
+    favoring = 4;  /* Year */
+  if (value < 0) 
+    favoring = 4;  /* Year */
+  if (value < 31)
+    if (value > 11)
+      favoring = 1;  /* Day of month */
+
+  return favoring;
+}
+
 /********************************************************************\
 \********************************************************************/
 
@@ -496,8 +658,7 @@ GetSubQIFAccount (AccountGroup *rootgrp, char *qifline, int acc_type)
    if (!xfer_acc) {
       xfer_acc = xaccMallocAccount ();
       xaccAccountSetName (xfer_acc, qifline);
-      xaccAccountSetDescription (xfer_acc, "");
-      xaccAccountSetNotes (xfer_acc, "");
+      xaccAccountSetCurrency (xfer_acc, gnc_qif_import_currency);
 
       if (0 > acc_type) acc_type = GuessAccountType (qifline);
       xaccAccountSetType (xfer_acc, acc_type);
@@ -581,8 +742,7 @@ xaccGetSecurityQIFAccount (Account *acc, char *qifline)
    if (!xfer_acc) {
       xfer_acc = xaccMallocAccount ();
       xaccAccountSetName (xfer_acc, qifline);
-      xaccAccountSetDescription (xfer_acc, "");
-      xaccAccountSetNotes (xfer_acc, "");
+      xaccAccountSetCurrency (xfer_acc, gnc_qif_import_currency);
 
       xaccAccountSetType (xfer_acc, STOCK);
       xaccInsertSubAccount (acc, xfer_acc);
@@ -600,6 +760,10 @@ xaccGetSecurityQIFAccount (Account *acc, char *qifline)
  *                                                                  * 
  * Args:   fd -- file descriptor                                    * 
  * Args:   acc -- account structure to fill in                      * 
+ * Args:   guess_name -- true if we should try and guess the name   *
+ *                       based on an opening balance entry          * 
+ * Args:   first_trans -- true if this is the first transaction to  *
+ *                        be processed in this account              *
  * Return: first new line after end of transaction                  * 
 \********************************************************************/
 
@@ -610,13 +774,15 @@ xaccGetSecurityQIFAccount (Account *acc, char *qifline)
 
 
 char * 
-xaccReadQIFTransaction (int fd, Account *acc, int *name_not_yet_set)
+xaccReadQIFTransaction (int fd, Account *acc, int guess_name,
+			int first_trans)
 {
    Transaction *trans;
    Split *source_split;
    Split *split = NULL;
    char * qifline;
    char * tmp;
+   int opening_balance = 0;
    int isneg = 0;
    int got_share_quantity = 0;
    int share_xfer = 0;
@@ -624,11 +790,13 @@ xaccReadQIFTransaction (int fd, Account *acc, int *name_not_yet_set)
    Account *sub_acc = 0x0;
    Account *xfer_acc = 0x0;
    double adjust = 0.0;
+   char secondchar;
 
    if (!acc) return NULL;
 
    qifline = xaccReadQIFLine (fd);
    if (!qifline) return NULL;
+
    if ('!' == qifline [0]) return qifline;
 
    trans = xaccMallocTransaction ();
@@ -637,267 +805,268 @@ xaccReadQIFTransaction (int fd, Account *acc, int *name_not_yet_set)
 
    /* scan for transaction date, description, type */
    while (qifline) {
-     /* C == Cleared / Reconciled */
-     if ('C' == qifline [0]) {  
-         /* Quicken uses C* and Cx, while MS Money uses CX.
-          * C* means cleared (but not yet reconciled)
-          * Cx or CX means reconciled
-          */
-        if (('x' == qifline[1]) || ('X' == qifline[1])) { 
-           xaccSplitSetReconcile (source_split, YREC);
-        } else {
-           xaccSplitSetReconcile (source_split, CREC);
-        }
-     } else 
-
-     /* D == date */
-     if ('D' == qifline [0]) {  
-         time_t secs;
-         secs = xaccParseQIFDate (&qifline[1]);
-         xaccTransSetDateSecs (trans, secs);
-     } else 
-
-     /* E == memo for split */
-     if ('E' == qifline [0]) {   
+     switch (qifline[0]) 
+       {
+       case 'C':   /* Cleared flag */
+	 secondchar = qifline[1];
+	 switch(secondchar) 
+	   {
+	   case QRECCLEAR:
+	     xaccSplitSetReconcile(source_split, CREC);
+	     break;
+	   case QRECREC:
+	   case QRECRECM:
+	     xaccSplitSetReconcile(source_split, YREC);
+	     break;
+	   case QRECBUDP:
+	   case QRECBUDG:
+	     xaccSplitSetReconcile(source_split, NREC);
+	     break;
+	   default:
+	     xaccSplitSetReconcile(source_split, NREC);
+	   }
+	 break;
+       case 'D':   /* D == date */
+	 {
+	   time_t secs;
+	   secs = xaccParseQIFDate (&qifline[1]);
+	   xaccTransSetDateSecs (trans, secs);
+	   xaccTransSetDateEnteredSecs (trans, secs);
+	 }
+	 break;
+       case 'E':    /* E == memo for split */
         if (split) {
-           XACC_PREP_STRING (tmp);
-           xaccSplitSetMemo (split, tmp);
+	  XACC_PREP_STRING (tmp);
+	  xaccSplitSetMemo (split, tmp);
         }
-     } else 
+	break;
+       case 'I':      /* I == share price */
+	 {
+	   double amt = xaccParseQIFAmount (&qifline[1]); 
+	   xaccSplitSetSharePrice (source_split, amt);
+	 }
+	 break;
+       case 'L': /* L == name of acount from which transfer occured */
+	 /* MSMoney uses a cute trick to overcome the lack of an account name
+	  * in the QIF format.  Basically, if the very very first transaction
+	  * has a payee field of "Opening Balance", then the L field is the name
+	  * of this account, and not the transfer account.  But this only works
+	  * for the very, very first transaction. This also seems to be the case
+	  * for Quicken 5.0 (and others?).
+	  */
+	 if (opening_balance) {
+	   if (guess_name) {
+	     /* remove square brackets from name, remove carriage return ... */
+	     qifline = &qifline[1];
+	     if ('[' == qifline[0]) {
+	       qifline = &qifline[1];
+	       tmp = strchr (qifline, ']');
+	       if (tmp) *tmp = 0x0;
+	     }
+	     tmp = strchr (qifline, '\r');
+	     if (tmp) *tmp = 0x0;
+	     tmp = strchr (qifline, '\n');
+	     if (tmp) *tmp = 0x0;
+	     xaccAccountSetName (acc, qifline);
+	   }
+	 } else {
+	   /* locate the transfer account */
+	   xfer_acc = xaccGetXferQIFAccount (acc, qifline);
+	 }
+	 break;
+       case 'M':
+	 /* M == memo field */
+	 XACC_PREP_STRING (tmp);
+	 xaccSplitSetMemo (source_split, tmp);
+	 break;
+       case 'N': 
+	 /* N == check numbers for Banks, but Action for portfolios */
+	 if (!strncmp (qifline, "NSell", 5)) isneg = 1;
+	 if (!strncmp (qifline, "NSell", 5)) share_xfer = 1;
+	 if (!strncmp (qifline, "NBuy", 4)) share_xfer = 1;
+	 
+	 /* if a recognized action, convert to our cannonical names */
+	 XACC_ACTION ("Buy", "Buy")
+	   XACC_ACTION ("Sell", "Sell")
+	   XACC_ACTION ("Div", "Div")
+	   XACC_ACTION ("CGLong", "LTCG")
+	   XACC_ACTION ("CGShort", "STCG")
+	   XACC_ACTION ("IntInc", "Int")
+	   XACC_ACTION ("DEP", "Deposit")
+	   XACC_ACTION ("XIn", "Deposit")
+	   XACC_ACTION ("XOut", "Withdraw")
+	   {
+	     XACC_PREP_STRING (tmp);
+	     xaccTransSetNum (trans, tmp);
+	   }
+	 break;
+       case 'O':
+	 /* O == adjustments */
+	 /* hack alert -- sometimes adjustments are quite large.
+	  * I have no clue why, and what to do about it.  For what 
+	  * it's worth, I can prove that Quicken version 3.0 makes 
+	  * math errors ... */
+	 {
+	   double pute;
+	   adjust = xaccParseQIFAmount (&qifline[1]);
+	   pute = xaccSplitGetValue (source_split);
+	   if (isneg) pute = -pute;
 
-     /* I == share price */
-     if ('I' == qifline [0]) {   
-         double amt = xaccParseUSAmount (&qifline[1]); 
-         xaccSplitSetSharePrice (source_split, amt);
-     } else 
-
-     /* L == name of acount from which transfer occured */
-     if ('L' == qifline [0]) {   
-        /* MSMoney uses a cute trick to overcome the lack of an account name
-         * in the QIF format.  Basically, if the very very first transaction
-         * has a payee field of "Opening Balance", then the L field is the name
-         * of this account, and not the transfer account.  But this only works
-         * for the very, very first transaction.
-         */
-        if (*name_not_yet_set) {
-            *name_not_yet_set = 0;
-            /* remove square brackets from name, remove carriage return ... */
-            qifline = &qifline[1];
-            if ('[' == qifline[0]) {
-               qifline = &qifline[1];
-               tmp = strchr (qifline, ']');
-               if (tmp) *tmp = 0x0;
-            }
-            tmp = strchr (qifline, '\r');
-            if(tmp) *tmp = 0x0;
-            tmp = strchr (qifline, '\n');
-            if(tmp) *tmp = 0x0;
-            xaccAccountSetName (acc, qifline);
-        } else {
-            /* locate the transfer account */
-            xfer_acc = xaccGetXferQIFAccount (acc, qifline);
-        }
-     } else 
-
-     /* M == memo field */
-     if ('M' == qifline [0]) {  
-        XACC_PREP_STRING (tmp);
-        xaccSplitSetMemo (source_split, tmp);
-     } else 
-
-     /* N == check numbers for Banks, but Action for portfolios */
-     if ('N' == qifline [0]) {   
-
-        if (!strncmp (qifline, "NSell", 5)) isneg = 1;
-        if (!strncmp (qifline, "NSell", 5)) share_xfer = 1;
-        if (!strncmp (qifline, "NBuy", 4)) share_xfer = 1;
-
-        /* if a recognized action, convert to our cannonical names */
-        XACC_ACTION ("Buy", "Buy")
-        XACC_ACTION ("Sell", "Sell")
-        XACC_ACTION ("Div", "Div")
-        XACC_ACTION ("CGLong", "LTCG")
-        XACC_ACTION ("CGShort", "STCG")
-        XACC_ACTION ("IntInc", "Int")
-        XACC_ACTION ("DEP", "Deposit")
-        XACC_ACTION ("XIn", "Deposit")
-        XACC_ACTION ("XOut", "Withdraw")
-        {
-          XACC_PREP_STRING (tmp);
-          xaccTransSetNum (trans, tmp);
-        }
-     } else
-
-     /* O == adjustments */
-     /* hack alert -- sometimes adjustments are quite large.
-      * I have no clue why, and what to do about it.  For what 
-      * its worth, I can prove that Quicken version 3.0 makes 
-      * math errors ... */
-     if ('O' == qifline [0]) {   
-        double pute;
-        adjust = xaccParseUSAmount (&qifline[1]);
-        pute = xaccSplitGetValue (source_split);
-        if (isneg) pute = -pute;
-
-        printf ("QIF Warning: Adjustment of %.2f to amount %.2f not handled \n", adjust, pute);
-     } else 
-
-     /* P == Payee, for Bank accounts */
-     if ('P' == qifline [0]) {   
-        XACC_PREP_STRING (tmp);
-        xaccTransSetDescription (trans, tmp);
-
-        /* MSMoney uses a cute trick to overcome the lack of an account name
-         * in the QIF format.  Basically, if the very very first transaction
-         * has a payee field of "Opening Balance", then the L field is the name
-         * of this account, and not the transfer account.  But this only works
-         * for the very, very first transaction.
-         */
-        if (*name_not_yet_set) {
-           if (! NSTRNCMP (qifline, "POpening Balance")) *name_not_yet_set = 0;
-        }
-     } else
-
-     /* Q == number of shares */
-     if ('Q' == qifline [0]) {   
-         double amt = xaccParseUSAmount (&qifline[1]);  
-         if (isneg) amt = -amt;
-         xaccSplitSetShareAmount (source_split, amt);
-         got_share_quantity = 1;
-     } else 
-
-     /* S == split, name of debited account */
-     if ('S' == qifline [0]) {   
-         split = xaccMallocSplit();
-
-         xaccTransAppendSplit (trans, split);
-         xfer_acc = xaccGetXferQIFAccount (acc, qifline);
-         xaccAccountInsertSplit (xfer_acc, split);
+	   printf ("QIF Warning: Adjustment of %.2f to amount %.2f not handled \n", adjust, pute);
+	 }
+	 break;
+       case 'P':
+	 XACC_PREP_STRING (tmp);
+	 xaccTransSetDescription (trans, tmp);
+	   
+	 /* MSMoney uses a cute trick to overcome the lack of an account name
+	  * in the QIF format.  Basically, if the very very first transaction
+	  * has a payee field of "Opening Balance", then the L field is the name
+	  * of this account, and not the transfer account.  But this only works
+	  * for the very, very first transaction. This also seems to be the case
+	  * for Quicken 5.0 (and others?).
+	  */
+	 if (first_trans)
+	   if (NSTRNCMP (qifline, "POpening Balance"))
+	     opening_balance = GNC_T;
+	 break;
 
+       case 'Q':
+	 /* Q == number of shares */
+	 {
+	   double amt = xaccParseQIFAmount (&qifline[1]);  
+	   if (isneg) amt = -amt;
+	   xaccSplitSetShareAmount (source_split, amt);
+	   got_share_quantity = 1;
+	 }
+	 break;
+       case 'S':
+	 /* S == split, name of debited account */
+	 split = xaccMallocSplit();
+	 xaccTransAppendSplit (trans, split);
+	 xfer_acc = xaccGetXferQIFAccount (acc, qifline);
+	 xaccAccountInsertSplit (xfer_acc, split);
+	 
          /* set xfer account to NULL, so that we don't
           * end up adding spurious splits */
-         xfer_acc = NULL;
-     } else 
-
-     /* T == total */
-     if ('T' == qifline [0]) {   
-         /* ignore T for stock transactions, since T is a dollar amount */
-         if (0 == got_share_quantity) {
-            double amt = xaccParseUSAmount (&qifline[1]);  
-            if (isneg) amt = -amt;
-            xaccSplitSetValue (source_split, amt);
-         }
-     } else 
-
-     /* Y == Name of Security */
-     if ('Y' == qifline [0]) {   
-        XACC_PREP_STRING (tmp);
-        xaccTransSetDescription (trans, tmp);
-
-        is_security = 1;
-        if (share_xfer) {
-           /* locate or create the sub-account account */
-           sub_acc = xaccGetSecurityQIFAccount (acc, qifline);
-        }
-
-     } else
-
-     /* $ == dollar amount */
-     if ('$' == qifline [0]) {   
-        /* for splits, $ records the part of the total for each split */
-        if (split) {
-           double amt = xaccParseUSAmount (&qifline[1]);  
-           amt = -amt;
-           xaccSplitSetValue (split, amt);
-        } else {
-           /* Currently, it appears that the $ amount is a redundant 
-            * number that we can safely ignore.  To get fancy,
-            * we use it to double-check the above work, since it 
-            * appears to always appear as the last entry in the
-            * transaction.  Round things out to pennies, to 
-            * handle round-off errors. 
-            */
-           double parse, pute;
-           int got, wanted;
-           parse = xaccParseUSAmount (&qifline[1]);
-           pute = xaccSplitGetValue (source_split);
-           if (isneg) pute = -pute;
-   
-           wanted = (int) (100.0 * parse + 0.5);
-           got = (int) (100.0 * (pute+adjust) + 0.5);
-           if (wanted != got) {
-              printf ("QIF Parse Error: wanted %f got %f \n", parse, pute);
-           }
-        }
-     } else 
-
+	 xfer_acc = NULL;
+	 break;
+	 
+       case 'T':
+	 /* T == total */
+	 /* ignore T for stock transactions, since T is a dollar amount */
+	 if (0 == got_share_quantity) {
+	   double amt = xaccParseQIFAmount (&qifline[1]);  
+	   if (isneg) amt = -amt;
+	   xaccSplitSetValue (source_split, amt);
+	 }
+	 break;
+       case 'Y':
+	 /* Y == Name of Security */
+	 XACC_PREP_STRING (tmp);
+	 xaccTransSetDescription (trans, tmp);
+	 
+	 is_security = 1;
+	 if (share_xfer) {
+	   /* locate or create the sub-account account */
+	   sub_acc = xaccGetSecurityQIFAccount (acc, qifline);
+	 }
+	 break;
+       case '$':
+	 /* $ == dollar amount */
+	 /* for splits, $ records the part of the total for each split */
+	 if (split) {
+	   double amt = xaccParseQIFAmount (&qifline[1]);  
+	   amt = -amt;
+	   xaccSplitSetValue (split, amt);
+	 } else {
+	   /* Currently, it appears that the $ amount is a redundant 
+	    * number that we can safely ignore.  To get fancy,
+	    * we use it to double-check the above work, since it 
+	    * appears to always appear as the last entry in the
+	    * transaction.  Round things out to pennies, to 
+	    * handle round-off errors. 
+	    */
+	   double parse, pute;
+	   int got, wanted;
+	   parse = xaccParseQIFAmount (&qifline[1]);
+	   pute = xaccSplitGetValue (source_split);
+	   if (isneg) pute = -pute;
+	   
+	   wanted = (int) (100.0 * parse + 0.5);
+	   got = (int) (100.0 * (pute+adjust) + 0.5);
+	     if (wanted != got) {
+	       printf ("QIF Parse Error: wanted %f got %f \n", parse, pute);
+	     }
+	 }
+       }
+     
      /* check for end-of-transaction marker */
      if (NSTRNCMP (qifline, "^^")) {
-        break;
+       break;
      } else
-     if (NSTRNCMP (qifline, "^\n")) {
-        break;
-     } else
-     if (NSTRNCMP (qifline, "^\r")) {
-        break;
-     } else
-     if ('!' == qifline [0]) break;
+       if (NSTRNCMP (qifline, "^\n")) {
+	 break;
+       } else
+	 if (NSTRNCMP (qifline, "^\r")) {
+	   break;
+	 } else
+	   if ('!' == qifline [0]) break;
      qifline = xaccReadQIFLine (fd);
-
+     
    }
-
+   
    /* at this point, we should see an end-of-transaction marker
     * if we see something else, assume the worst, free the last 
     * transaction, and return */
    if (!qifline || ('!' == qifline[0])) {
-      xaccTransDestroy (trans);
-      xaccTransCommitEdit (trans);
-      return qifline;
+     xaccTransDestroy (trans);
+     xaccTransCommitEdit (trans);
+     return qifline;
    }
 
-   /* fundamentally differnt handling for securities and non-securities */
+   /* fundamentally different handling for securities and non-securities */
    if (is_security) {
-
-      /* if the transaction  is a sale/purchase of a security, 
-       * then it is a defacto transfer between the brokerage account 
-       * and the stock account.  */
-      if (share_xfer) {
-         if (!split) {
-            split = xaccMallocSplit ();
-            xaccTransAppendSplit (trans, split);
-         }
-
-         /* Insert the transaction into the main brokerage 
-          * account as a debit, unless an alternate account
-          * was specified. */
-         if (xfer_acc) {
-            xaccAccountInsertSplit (xfer_acc, split);
+     /* if the transaction  is a sale/purchase of a security, 
+      * then it is a defacto transfer between the brokerage account 
+      * and the stock account.  */
+     if (share_xfer) {
+       if (!split) {
+	 split = xaccMallocSplit ();
+	 xaccTransAppendSplit (trans, split);
+       }
+       
+       /* Insert the transaction into the main brokerage 
+	* account as a debit, unless an alternate account
+	* was specified. */
+       if (xfer_acc) {
+	 xaccAccountInsertSplit (xfer_acc, split);
+       } else {
+	 xaccAccountInsertSplit (acc, split);
+       }
+       
+       /* normally, the security account is pointed at by 
+	* sub_acc; the security account is credited.
+	* But, just in case its missing, avoid a core dump */
+       if (sub_acc) {
+	 /* xxx hack alert --- is this right ??? */
+	 xaccAccountInsertSplit (sub_acc, source_split);
+       }
+     } else {
+       
+       /* else, we are here if its not a share transfer. 
+	* It is probably dividend or other income */
+       
+       /* if a transfer account is specified, the transfer 
+	* account gets the dividend credit; otherwise, the 
+	* main account gets it */
+       if (xfer_acc) {
+	 xaccAccountInsertSplit (xfer_acc, source_split);
          } else {
-            xaccAccountInsertSplit (acc, split);
+	   xaccAccountInsertSplit (acc, source_split);
          }
-
-         /* normally, the security account is pointed at by 
-          * sub_acc; the security account is credited.
-          * But, just in case its missing, avoid a core dump */
-         if (sub_acc) {
-/* xxx hack alert --- is this right ??? */
-            xaccAccountInsertSplit (sub_acc, source_split);
-         }
-      } else {
-
-         /* else, we are here if its not a share transfer. 
-          * It is probably dividend or other income */
-
-         /* if a transfer account is specified, the transfer 
-          * account gets the dividend credit; otherwise, the 
-          * main account gets it */
-         if (xfer_acc) {
-            xaccAccountInsertSplit (xfer_acc, source_split);
-         } else {
-            xaccAccountInsertSplit (acc, source_split);
-         }
-      }
-
+     }
+     
    } else {
       /* if we are here, its not a security, but an ordinary account */
       /* if a transfer account was specified,  it is the debited account */
@@ -924,15 +1093,15 @@ xaccReadQIFTransaction (int fd, Account *acc, int *name_not_yet_set)
  * the indicated account
 \********************************************************************/
 
-char * xaccReadQIFTransList (int fd, Account *acc, int *acc_name_not_yet_set)
+char * xaccReadQIFTransList (int fd, Account *acc, int guess_name)
 {
    char * qifline;
 
    if (!acc) return 0x0;
-   qifline = xaccReadQIFTransaction (fd, acc, acc_name_not_yet_set);
+   qifline = xaccReadQIFTransaction (fd, acc, guess_name, GNC_T);
    while (qifline) {
       if ('!' == qifline[0]) break;
-      qifline = xaccReadQIFTransaction (fd, acc, acc_name_not_yet_set);
+      qifline = xaccReadQIFTransaction (fd, acc, guess_name, GNC_F);
    } 
    return qifline;
 }
@@ -942,7 +1111,7 @@ char * xaccReadQIFTransList (int fd, Account *acc, int *acc_name_not_yet_set)
 \********************************************************************/
 
 /********************************************************************\
- * xaccReadQIFAccountGroup                                                  * 
+ * xaccReadQIFAccountGroup                                          * 
  *   reads in the data from file datafile                           *
  *                                                                  * 
  * Args:   datafile - the file to load the data from                * 
@@ -1009,13 +1178,13 @@ xaccReadQIFAccountGroup( char *datafile )
      } 
         
      if (name) {
-        int bogus_acc_name = 1;
         Account * acc = xaccMallocAccount();
         xaccAccountSetType (acc, typo);
         xaccAccountSetName (acc, name);
+        xaccAccountSetCurrency (acc, gnc_qif_import_currency);
 
         xaccGroupInsertAccount( grp, acc );
-        qifline = xaccReadQIFTransList (fd, acc, &bogus_acc_name);
+        qifline = xaccReadQIFTransList (fd, acc, GNC_T);
         typo = -1; name = NULL;
         continue;
      } else
@@ -1032,7 +1201,8 @@ xaccReadQIFAccountGroup( char *datafile )
         continue;
      } else
 
-     if (STRSTR (qifline, "Type:Memorized")) {
+     if ((STRSTR (qifline, "Type:Memorized")) ||
+         (STRSTR (qifline, "Type:Memorised")) ) {
         DEBUG ("got memorized\n");
         qifline = xaccReadQIFDiscard (fd);
         continue;
@@ -1065,10 +1235,10 @@ xaccReadQIFAccountGroup( char *datafile )
            /* read account name, followed by dollar data ... */
            char * acc_name;
            Account *preexisting;
-           Account *acc   = xaccMallocAccount();
-           int guess_acc_name = 0;
+           Account *acc = xaccMallocAccount();
 
            DEBUG ("got account\n");
+           xaccAccountSetCurrency (acc, gnc_qif_import_currency);
            qifline = xaccReadQIFAccount (fd, acc);
            if (!qifline) {  /* free up malloced data if the read bombed. */
               xaccFreeAccount(acc); 
@@ -1080,7 +1250,7 @@ xaccReadQIFAccountGroup( char *datafile )
            acc_name = xaccAccountGetName (acc);
            preexisting = xaccGetAccountFromName (grp, acc_name);
            if (preexisting) 
-            {
+	   {
               xaccFreeAccount (acc);
               acc = preexisting;
            } 
@@ -1103,7 +1273,7 @@ xaccReadQIFAccountGroup( char *datafile )
    
            /* read transactions */
            /* note, we have a real account name, so no need to go guessing it. */
-           if (qifline) qifline = xaccReadQIFTransList (fd, acc, &guess_acc_name);
+           if (qifline) qifline = xaccReadQIFTransList (fd, acc, GNC_F);
         }    
         continue;
      } else
