@@ -1,16 +1,3 @@
-/*
- * FILE:
- * Query.c
- *
- * DESCRIPTION:
- * Provide a simple query engine interface.
- * Note that the query engine is officially a part of the transaction engine, 
- * and thus has direct access to internal structures.
- *
- * HISTORY:
- * created by Linas Vepstas Sept 1998
- * Copyright (c) 1998 Linas Vepstas
- */
 /********************************************************************\
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -27,8 +14,23 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 \********************************************************************/
 
+/*
+ * FILE:
+ * Query.c
+ *
+ * DESCRIPTION:
+ * Provide a simple query engine interface.
+ * Note that the query engine is officially a part of the transaction engine, 
+ * and thus has direct access to internal structures.
+ *
+ * HISTORY:
+ * created by Linas Vepstas Sept 1998
+ * Copyright (c) 1998-2000 Linas Vepstas
+ */
+
 #include <limits.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <strings.h>
 
 #include "config.h"
@@ -40,6 +42,11 @@
 #include "TransactionP.h"
 #include "util.h"
 
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_ENGINE;
+
+/* ================================================== */
+
 struct _Query {
    Account ** acc_list;
 
@@ -72,7 +79,14 @@ xaccMallocQuery (void)
 
 /* ================================================== */
 
-static int Sort_DATE_NUM_AMOUNT (Split **, Split **);
+static int Sort_STANDARD_NONE_NONE (Split **, Split **);
+
+#ifndef LONG_LONG_MAX
+#define LONG_LONG_MAX 0x7fffffffffffffffLL
+#endif
+#ifndef LONG_LONG_MIN
+#define LONG_LONG_MIN (- (0x7fffffffffffffffLL) -1)
+#endif
 
 void 
 xaccInitQuery (Query *q)
@@ -82,28 +96,22 @@ xaccInitQuery (Query *q)
    q->acc_list = NULL;
    q->split_list = NULL;
    q->changed = 0; 
-   q->max_num_splits = INT_MAX ;
+   q->max_num_splits = INT_MAX;
 
-   q->earliest.tv_sec = 0; 
+   q->earliest.tv_sec = LONG_LONG_MIN;
    q->earliest.tv_nsec = 0; 
 
-/* hack alert HACK ALERT Y2K problem -- danger danger.
- * For some stupid reason,  struct Timespec uses long int
- * instead of time_t for seconds. This means we will have 
- * overflow in year 2004 which is stupid, and needs to be 
- * fixed.
- */
-   /* q->latest.tv_sec = ULONG_MAX; */
-   q->latest.tv_sec = LONG_MAX;
+   /* Its a signed, 64-bit int */
+   q->latest.tv_sec = LONG_LONG_MAX;
    q->latest.tv_nsec = 0; 
 
-   q->earliest_found.tv_sec = LONG_MAX; 
+   q->earliest_found.tv_sec = LONG_LONG_MAX;
    q->earliest_found.tv_nsec = 0; 
 
-   q->latest_found.tv_sec = 0;
+   q->latest_found.tv_sec = LONG_LONG_MIN;
    q->latest_found.tv_nsec = 0; 
 
-   q->sort_func = (int (*)(const void*, const void *)) Sort_DATE_NUM_AMOUNT;
+   q->sort_func = (int (*)(const void*, const void *)) Sort_STANDARD_NONE_NONE;
 }
 
 /* ================================================== */
@@ -210,6 +218,85 @@ xaccQuerySetDateRange (Query *q, time_t early, time_t late)
    q->latest.tv_sec = late;
 }
 
+void  
+xaccQuerySetDateRangeL (Query *q, long long early, long long late)
+{
+   if (!q) return;
+   q->changed = 1; 
+   q->earliest.tv_sec = early;
+   q->latest.tv_sec = late;
+}
+
+void
+xaccQuerySetEarliest (Query *q, time_t earliest)
+{
+   if (!q) return;
+   q->changed = 1; 
+   q->earliest.tv_sec = earliest;
+}
+
+void
+xaccQuerySetLatest (Query *q, time_t latest)
+{
+   if (!q) return;
+   q->changed = 1; 
+   q->latest.tv_sec = latest;
+}
+
+void
+xaccQuerySetEarliestTS (Query *q, Timespec earliest)
+{
+  if (!q) return;
+  q->changed = 1;
+  q->earliest = earliest;
+}
+
+void
+xaccQuerySetLatestTS (Query *q, Timespec latest)
+{
+  if (!q) return;
+  q->changed = 1;
+  q->latest = latest;
+}
+
+time_t
+xaccQueryGetEarliest (Query *q)
+{
+  assert(q != NULL);
+
+  return q->earliest.tv_sec;
+}
+
+time_t
+xaccQueryGetLatest (Query *q)
+{
+  assert(q != NULL);
+
+  return q->latest.tv_sec;
+}
+
+/* ================================================== */
+
+void
+xaccQueryShowEarliestDateFound (Query *q)
+{
+   if (!q) return;
+   q->changed = 1; 
+   q->earliest.tv_sec = LONG_LONG_MIN;
+   q->earliest.tv_nsec = 0; 
+}
+
+void
+xaccQueryShowLatestDateFound (Query *q)
+{
+   if (!q) return;
+   q->changed = 1; 
+   q->latest.tv_sec = LONG_LONG_MAX;
+   q->latest.tv_nsec = 0; 
+}
+
+/* ================================================== */
+
 /* ================================================== */
 /* Note that the sort order for a transaction that is
  * currently being edited is based on its old values, 
@@ -239,6 +326,10 @@ xaccQuerySetDateRange (Query *q, time_t early, time_t late)
   if ( !(ta) && !(tb) ) return 0;		\
 
 
+#define CSTANDARD {                             \
+  retval = xaccSplitDateOrder(sa, sb);          \
+  if (retval) return retval;                    \
+}
 
 #define CDATE {					\
   /* if dates differ, return */			\
@@ -264,19 +355,38 @@ xaccQuerySetDateRange (Query *q, time_t early, time_t late)
 
 #define CNUM {					\
   /* sort on transaction number */		\
+  unsigned long n1;                             \
+  unsigned long n2;                             \
   da = ta->num;					\
   db = tb->num;					\
-  if (da && db) {				\
-    retval = strcmp (da, db);			\
-    /* if strings differ, return */		\
-    if (retval) return retval;			\
-  } else					\
+  if (gnc_strisnum(da)) {                       \
+    if (!gnc_strisnum(db)) {                    \
+      return -1;                                \
+    }                                           \
+    sscanf(da, "%lu", &n1);                     \
+    sscanf(db, "%lu", &n2);                     \
+    if (n1 < n2) {                              \
+      return -1;                                \
+    }                                           \
+    if (n1 == n2) {                             \
+      return 0;                                 \
+    }                                           \
+    return +1;                                  \
+  }                                             \
+  if (gnc_strisnum(db)) {                       \
+    return +1;                                  \
+  }                                             \
   if (!da && db) {				\
     return -1;					\
-  } else					\
+  }     					\
   if (da && !db) {				\
     return +1;					\
   }						\
+  if (!da && !db) {                             \
+    return 0;                                   \
+  }                                             \
+  retval = strcmp (da, db);			\
+  return retval;                                \
 }
 
 #define CMEMO {					\
@@ -313,7 +423,7 @@ xaccQuerySetDateRange (Query *q, time_t early, time_t late)
   }						\
 }
 
-#define CAMOUNT {					\
+#define CAMOUNT {				\
   double fa, fb;				\
   fa = ((*sa)->damount) * ((*sa)->share_price); \
   fb = ((*sb)->damount) * ((*sb)->share_price); \
@@ -325,6 +435,8 @@ xaccQuerySetDateRange (Query *q, time_t early, time_t late)
   }						\
 }
 
+#define CNONE
+
 #define DECLARE(ONE,TWO,THREE)			\
 static int Sort_##ONE##_##TWO##_##THREE		\
    (Split **sa, Split **sb) 			\
@@ -377,6 +489,7 @@ sub recur {
 /* ================================================== */
 /* Define the sorting comparison functions */
 
+DECLARE (STANDARD, NONE, NONE)
 DECLARE (DESC, MEMO, AMOUNT)
 DECLARE (DESC, MEMO, NUM)
 DECLARE (DESC, MEMO, DATE)
@@ -453,6 +566,7 @@ xaccQuerySetSortOrder (Query *q, int arga, int argb, int argc)
    if (!q) return;
    q->changed = 1; 
 
+   DECIDE (STANDARD, NONE, NONE)
    DECIDE (DESC, MEMO, AMOUNT)
    DECIDE (DESC, MEMO, NUM)
    DECIDE (DESC, MEMO, DATE)
@@ -547,6 +661,8 @@ xaccQueryGetSplits (Query *q)
    Split *s, **slist;
    Account *acc;
 
+   ENTER ("xaccQueryGetSplits()\n");
+
    if (!q) return NULL;
 
    /* tmp hack alert */
@@ -570,12 +686,12 @@ xaccQueryGetSplits (Query *q)
          /* now go through the splits */
          j=0; s = acc->splits[0];
          while (s) {
-            if (s->parent->date_posted.tv_sec >= q->earliest.tv_sec) {
-               nsplits ++;
-            }
             if (s->parent->date_posted.tv_sec > q->latest.tv_sec) {
                break;
             }
+            if (s->parent->date_posted.tv_sec >= q->earliest.tv_sec) {
+               nsplits ++;
+            }
             j++; s = acc->splits[j];
          }
          i++; acc = q->acc_list[i];
@@ -593,12 +709,12 @@ xaccQueryGetSplits (Query *q)
          /* now go through the splits */
          j=0; s = acc->splits[0];
          while (s) {
-            if (s->parent->date_posted.tv_sec >= q->earliest.tv_sec) {
-               slist[k] = s; k++;
-            }
             if (s->parent->date_posted.tv_sec > q->latest.tv_sec) {
                break;
             }
+            if (s->parent->date_posted.tv_sec >= q->earliest.tv_sec) {
+               slist[k] = s; k++;
+            }
             j++; s = acc->splits[j];
          }
          i++; acc = q->acc_list[i];
@@ -606,6 +722,8 @@ xaccQueryGetSplits (Query *q)
    }
    slist[k] = NULL;
 
+   DEBUG ("xaccQueryGetSplits(): will sort %d splits\n", nsplits);
+
    /* sort them ... */
    SortSplits (q, slist);
 
@@ -654,6 +772,7 @@ xaccQueryGetSplits (Query *q)
       }
       i++; s=slist[i];
    }
+   LEAVE ("xaccQueryGetSplits(): returning  %d splits\n", nret);
    
    return q->split_list;
 }
