@@ -1,7 +1,7 @@
 /********************************************************************\
  * Transaction.c -- the transaction data structure                  *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997, 1998 Linas Vepstas                           *
+ * Copyright (C) 1997, 1998, 1999, 2000 Linas Vepstas               *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -42,7 +42,7 @@
 
 
 /* 
- * If the "force_double_entry" flag determines how 
+ * The "force_double_entry" flag determines how 
  * the splits in a transaction will be balanced. 
  *
  * The following values have significance:
@@ -69,6 +69,8 @@ int force_double_entry = 0;
  * knowledge of the internals of the Transaction in one file.       *
 \********************************************************************/
 
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_ENGINE;
 
 /********************************************************************\
  * xaccInitSplit
@@ -76,9 +78,8 @@ int force_double_entry = 0;
 \********************************************************************/
 
 void
-xaccInitSplit( Split * split )
-  {
-  
+xaccInitSplit(Split * split)
+{
   /* fill in some sane defaults */
   split->acc         = NULL;
   split->parent      = NULL;
@@ -99,20 +100,21 @@ xaccInitSplit( Split * split )
   split->share_balance             = 0.0;
   split->share_cleared_balance     = 0.0;
   split->share_reconciled_balance  = 0.0;
+  split->cost_basis                = 0.0;
 
   split->tickee = 0;
-  }
+}
 
 /********************************************************************\
 \********************************************************************/
 
 Split *
-xaccMallocSplit( void )
-  {
+xaccMallocSplit(void)
+{
   Split *split = (Split *)_malloc(sizeof(Split));
   xaccInitSplit (split);
   return split;
-  }
+}
 
 /********************************************************************\
 \********************************************************************/
@@ -157,7 +159,7 @@ xaccCloneSplit (Split *s)
 
 void
 xaccFreeSplit( Split *split )
-  {
+{
   if (!split) return;
 
   if (split->memo) free (split->memo);
@@ -183,9 +185,25 @@ xaccFreeSplit( Split *split )
 /********************************************************************\
 \********************************************************************/
 
-#define MARK_SPLIT(split) {			\
-   Account *acc = (Account *) ((split)->acc);	\
-   if (acc) acc->changed = 1;			\
+void
+xaccConfigSetForceDoubleEntry (int force) 
+{
+   force_double_entry = force;
+}
+
+int
+xaccConfigGetForceDoubleEntry (void) 
+{
+   return (force_double_entry);
+}
+
+/********************************************************************\
+\********************************************************************/
+
+#define MARK_SPLIT(split) {			       \
+   Account *acc = (Account *) ((split)->acc);	       \
+   if (acc) acc->changed |= ACC_INVALIDATE_ALL;	       \
+   if (acc) xaccAccountGroupMarkNotSaved(acc->parent); \
 }
 
 static void
@@ -287,6 +305,13 @@ double xaccSplitGetShareBalance (Split *s)
    return s->share_balance;
 }
 
+double xaccSplitGetCostBasis (Split *s) 
+{
+   if (!s) return 0.0;
+   xaccAccountRecomputeCostBasis (s->acc);
+   return s->cost_basis;
+}
+
 /********************************************************************\
  * xaccInitTransaction
  * Initialize a transaction structure
@@ -294,7 +319,7 @@ double xaccSplitGetShareBalance (Split *s)
 
 void
 xaccInitTransaction( Transaction * trans )
-  {
+{
   Split *split;
   
   /* fill in some sane defaults */
@@ -318,21 +343,21 @@ xaccInitTransaction( Transaction * trans )
   trans->date_posted.tv_sec  = 0;
   trans->date_posted.tv_nsec = 0;
 
-  trans->write_flag  = 0;
-  trans->open        = 0;
+  trans->marker = 0;
+  trans->open = 0;
   trans->orig = NULL;
-  }
+}
 
 /********************************************************************\
 \********************************************************************/
 
 Transaction *
 xaccMallocTransaction( void )
-  {
+{
   Transaction *trans = (Transaction *)_malloc(sizeof(Transaction));
   xaccInitTransaction (trans);
   return trans;
-  }
+}
 
 /********************************************************************\
 \********************************************************************/
@@ -382,11 +407,12 @@ xaccCloneTransaction (Transaction *t)
 
 void
 xaccFreeTransaction( Transaction *trans )
-  {
+{
   int i;
   Split *s;
 
   if (!trans) return;
+  ENTER ("xaccFreeTransaction(): addr=%p\n", trans);
 
   /* free up the destination splits */
   if (trans->splits) {
@@ -425,6 +451,7 @@ xaccFreeTransaction( Transaction *trans )
   }
 
   _free(trans);
+  LEAVE ("xaccFreeTransaction(): addr=%p\n", trans);
 }
 
 /********************************************************************\
@@ -435,7 +462,7 @@ xaccSplitSetBaseValue (Split *s, double value, char * base_currency)
 {
    if (!s) return;
 
-   /* ahh -- stupid users may not want or use the double entry 
+   /* Novice/casual users may not want or use the double entry 
     * features of this engine.  So, in particular, there
     * may be the occasional split without a parent account. 
     * Well, that's ok,  we'll just go with the flow. 
@@ -449,7 +476,7 @@ xaccSplitSetBaseValue (Split *s, double value, char * base_currency)
       }
    }
 
-   /* be more precise -- the value depends on the curency 
+   /* be more precise -- the value depends on the currency 
     * we want it expressed in.
     */
    if (!safe_strcmp(s->acc->currency, base_currency)) {
@@ -462,9 +489,9 @@ xaccSplitSetBaseValue (Split *s, double value, char * base_currency)
       s -> damount = (value / (s->share_price));   
    } else 
    {
-      printf ("Error: xaccSplitSetBaseValue(): "
-              " inappropriate base currency %s "
-              " given split currency=%s and security=%s\n",
+      PERR ("xaccSplitSetBaseValue(): "
+            " inappropriate base currency %s "
+            " given split currency=%s and security=%s\n",
               base_currency, s->acc->currency, s->acc->security);
       return;
    }
@@ -504,9 +531,9 @@ xaccSplitGetBaseValue (Split *s, char * base_currency)
       value = s->damount * s->share_price;   
    } else 
    {
-      printf ("Error: xaccSplitGetBaseValue(): "
-              " inappropriate base currency %s "
-              " given split currency=%s and security=%s\n",
+      PERR ("xaccSplitGetBaseValue(): "
+            " inappropriate base currency %s "
+            " given split currency=%s and security=%s\n",
               base_currency, s->acc->currency, s->acc->security);
       return 0.0;
    }
@@ -516,7 +543,6 @@ xaccSplitGetBaseValue (Split *s, char * base_currency)
 /********************************************************************\
 \********************************************************************/
 
-
 static double
 ComputeValue (Split **sarray, Split * skip_me, char * base_currency)
 {
@@ -553,8 +579,8 @@ ComputeValue (Split **sarray, Split * skip_me, char * base_currency)
             if (!safe_strcmp(s->acc->security, base_currency)) {
                value += s->damount;
             } else {
-               printf ("Internal Error: ComputeValue(): "
-                       " inconsistent currencies \n");
+               PERR ("Internal Error: ComputeValue(): "
+                     " inconsistent currencies \n");
                assert (0);
             }
          }
@@ -565,6 +591,136 @@ ComputeValue (Split **sarray, Split * skip_me, char * base_currency)
    return value;
 }
 
+double
+xaccTransGetImbalance (Transaction * trans)
+{
+  char * currency = xaccTransFindCommonCurrency (trans);
+  double imbal = ComputeValue (trans->splits, NULL, currency);
+  return imbal;
+}
+
+/********************************************************************\
+\********************************************************************/
+gncBoolean xaccIsCommonCurrency(char *currency_1, char *security_1,
+				char *currency_2, char *security_2)
+{
+  int c1c2, c1s2, s1c2, s1s2;
+
+  if ((currency_1 == NULL) || (currency_2 == NULL))
+    return GNC_F;
+
+  if ((security_1 != NULL) && (security_1[0] == 0x0))
+    security_1 = NULL;
+
+  if ((security_2 != NULL) && (security_2[0] == 0x0))
+    security_2 = NULL;
+
+  c1c2 = safe_strcmp(currency_1, currency_2);
+  c1s2 = safe_strcmp(currency_1, security_2);
+
+  if (security_1 != NULL)
+  {
+    s1c2 = safe_strcmp(security_1, currency_2);
+    s1s2 = safe_strcmp(security_1, security_2);
+  }
+  else /* no match */
+  {
+    s1c2 = 1;
+    s1s2 = 1;
+  }
+
+  return (c1c2 == 0) || (c1s2 == 0) || (s1c2 == 0) || (s1s2 == 0);
+}
+
+static char *
+FindCommonCurrency (Split **slist, char * ra, char * rb)
+{
+  Split *s;
+  int i = 0;
+
+  if (!slist) return NULL;
+
+  if (rb && (0x0==rb[0])) rb = 0x0;
+
+  i=0; s = slist[0];
+  while (s) {
+    char *sa, *sb;
+
+    /* Novice/casual users may not want or use the double entry 
+     * features of this engine.   Because of this, there
+     * may be the occasional split without a parent account. 
+     * Well, that's ok,  we'll just go with the flow. 
+     */
+    if (force_double_entry) {
+       assert (s->acc);
+    } else
+    if (NULL == s->acc) {
+       i++; s=slist[i]; continue;
+    }
+
+    sa = s->acc->currency;
+    sb = s->acc->security;
+    if (sb && (0x0==sb[0])) sb = 0x0;
+
+    if (ra && rb) {
+       int aa = safe_strcmp (ra,sa);
+       int ab = safe_strcmp (ra,sb);
+       int ba = safe_strcmp (rb,sa);
+       int bb = safe_strcmp (rb,sb);
+       if ( (!aa) && bb) rb = 0x0;
+       else
+       if ( (!ab) && ba) rb = 0x0;
+       else
+       if ( (!ba) && ab) ra = 0x0;
+       else
+       if ( (!bb) && aa) ra = 0x0;
+       else
+       if ( aa && bb && ab && ba ) { ra=0x0; rb=0x0; }
+
+       if (!ra) { ra=rb; rb=0x0; }
+    } 
+    else
+    if (ra && !rb) {
+       int aa = safe_strcmp (ra,sa);
+       int ab = safe_strcmp (ra,sb);
+       if ( aa && ab )  ra= 0x0;
+    }
+
+    if ((!ra) && (!rb)) return NULL;
+    i++; s = slist[i];
+  }
+
+  return (ra);
+}
+
+
+char *
+xaccTransFindCommonCurrency (Transaction *trans)
+{
+  char *ra, *rb, *com;
+
+  assert (trans->splits);
+  assert (trans->splits[0]);
+  assert (trans->splits[0]->acc);
+
+  ra = trans->splits[0]->acc->currency;
+  rb = trans->splits[0]->acc->security;
+
+  com = FindCommonCurrency (trans->splits, ra, rb);
+  return com;
+}
+
+char *
+xaccTransIsCommonCurrency (Transaction *trans, char * ra)
+{
+  char *com;
+  com = FindCommonCurrency (trans->splits, ra, NULL);
+  return com;
+}
+
+/********************************************************************\
+\********************************************************************/
+
 /* hack alert -- the algorithm used in this rebalance routine
  * is less than intuitive, and could use some write-up.  
  * Maybe it does indeed do the right thing, but that is
@@ -585,7 +741,6 @@ xaccSplitRebalance (Split *split)
   int i = 0;
   double value = 0.0;
   char *base_currency=0x0;
-  char *ra=0x0, *rb =0x0;
 
   trans = split->parent;
 
@@ -598,6 +753,7 @@ xaccSplitRebalance (Split *split)
 
   if (DEFER_REBALANCE & (trans->open)) return;
   if (split->acc) {
+    char *ra, *rb;
     if (ACC_DEFER_REBALANCE & (split->acc->open)) return;
     assert (trans->splits);
     assert (trans->splits[0]);
@@ -606,70 +762,28 @@ xaccSplitRebalance (Split *split)
      * and which one(s) all of the splits have in common.  */
     ra = split->acc->currency;
     rb = split->acc->security;
-    if (rb && (0x0==rb[0])) rb = 0x0;
-  
-    i=0; s = trans->splits[0];
-    while (s) {
-      char *sa, *sb;
-  
-      /* ahh -- stupid users may not want or use the double entry 
-       * features of this engine.  So, in particular, there
-       * may be the occasional split without a parent account. 
-       * Well, that's ok,  we'll just go with the flow. 
-       */
-      if (force_double_entry) {
-         assert (s->acc);
-      } else {
-         i++; s=trans->splits[i]; continue;
+    base_currency = FindCommonCurrency (trans->splits, ra, rb);
+
+    if (!base_currency) {
+      PERR ("Internal Error: SplitRebalance(): no common split currencies \n");
+      s = trans->splits[0];
+      while (s) {
+        if (s->acc) {
+          PERR ("\taccount=%s currency=%s security=%s\n",
+                s->acc->accountName, s->acc->currency, s->acc->security);
+        } else {
+          PERR ("\t*** No parent account *** \n");
+        }
+        i++; s = trans->splits[i];
       }
-  
-      sa = s->acc->currency;
-      sb = s->acc->security;
-      if (sb && (0x0==sb[0])) sb = 0x0;
-  
-      if (ra && rb) {
-         int aa = safe_strcmp (ra,sa);
-         int ab = safe_strcmp (ra,sb);
-         int ba = safe_strcmp (rb,sa);
-         int bb = safe_strcmp (rb,sb);
-         if ( (!aa) && bb) rb = 0x0;
-         else
-         if ( (!ab) && ba) rb = 0x0;
-         else
-         if ( (!ba) && ab) ra = 0x0;
-         else
-         if ( (!bb) && aa) ra = 0x0;
-         else
-         if ( aa && bb && ab && ba ) { ra=0x0; rb=0x0; }
-  
-         if (!ra) { ra=rb; rb=0x0; }
-      } 
-      else
-      if (ra && !rb) {
-         int aa = safe_strcmp (ra,sa);
-         int ab = safe_strcmp (ra,sb);
-         if ( aa && ab )  ra= 0x0;
-      }
-  
-    if ((!ra) && (!rb)) {
-        printf ("Internal Error: SplitRebalance(): "
-                " no common split currencies \n");
-        printf ("\tbase acc=%s cur=%s base_sec=%s\n"
-                "\tacc=%s scur=%s ssec=%s \n", 
-            split->acc->accountName, split->acc->currency, split->acc->security,
-            s->acc->accountName, s->acc->currency, s->acc->security );
-        assert (0);
-        return;
-      }
-      i++; s = trans->splits[i];
+      assert (0);
+      return;
     }
   } else {
     assert (trans->splits);
     assert (trans->splits[0]);
   }
 
-  base_currency = ra;
-
   if (split == trans->splits[0]) {
     /* The indicated split is the source split.
      * Pick a destination split (by default, 
@@ -701,7 +815,7 @@ xaccSplitRebalance (Split *split)
        if (force_double_entry) {
           if (! (DEQ (0.0, split->damount))) {
              value = split->share_price * split->damount;
-   
+
              /* malloc a new split, mirror it to the source split */
              s = xaccMallocSplit ();
              s->damount = -value;
@@ -709,7 +823,7 @@ xaccSplitRebalance (Split *split)
              s->memo = strdup (split->memo);
              free (s->action);
              s->action = strdup (split->action);
-   
+
              /* insert the new split into the transaction and 
               * the same account as the source split */
              MARK_SPLIT (s);
@@ -722,7 +836,7 @@ xaccSplitRebalance (Split *split)
 
     /* The indicated split is a destination split.
      * Compute grand total of all destination splits,
-     * and force the source split to blanace.
+     * and force the source split to balance.
      */
     s = trans->splits[0];
     value = ComputeValue (trans->splits, s, base_currency);
@@ -745,9 +859,9 @@ xaccSplitRebalance (Split *split)
 
 #define CHECK_OPEN(trans) {					\
    if (!trans->open) {						\
-      printf ("Error: transaction %p not open for editing\n", trans);	\
+      PERR ("transaction %p not open for editing\n", trans);	\
       /* assert (trans->open); */					\
-      printf ("%s:%d \n", __FILE__, __LINE__);			\
+      PERR ("\t%s:%d \n", __FILE__, __LINE__);			\
       /* return; */						\
    }								\
 }
@@ -755,9 +869,14 @@ xaccSplitRebalance (Split *split)
 void
 xaccTransBeginEdit (Transaction *trans, int defer)
 {
+   char open;
+
    assert (trans);
+   open = trans->open;
    trans->open = BEGIN_EDIT;
    if (defer) trans->open |= DEFER_REBALANCE;
+   if (open & BEGIN_EDIT) return;
+
    xaccOpenLog ();
    xaccTransWriteLog (trans, 'B');
 
@@ -775,6 +894,7 @@ xaccTransCommitEdit (Transaction *trans)
    Account *acc;
 
    if (!trans) return;
+   ENTER ("xaccTransCommitEdit(): trans addr=%p\n", trans);
    CHECK_OPEN (trans);
 
    /* At this point, we check to see if we have a valid transaction.
@@ -785,6 +905,7 @@ xaccTransCommitEdit (Transaction *trans)
    split = trans->splits[0];
    if (!split)
    {
+      PINFO ("xaccTransCommitEdit(): delete trans at addr=%p\n", trans);
       /* Make a log in the journal before destruction.  */
       xaccTransWriteLog (trans, 'D');
       xaccFreeTransaction (trans);
@@ -825,32 +946,22 @@ xaccTransCommitEdit (Transaction *trans)
    trans->open &= ~DEFER_REBALANCE;
    xaccTransRebalance (trans);
 
-   /* check to see if the date has changed.  We use the date as the sort key. */
-   if ((trans->orig->date_entered.tv_sec != trans->date_entered.tv_sec) ||
-       (trans->orig->date_posted.tv_sec  != trans->date_posted.tv_sec))
-   {
-
-      /* since the date has changed, we need to be careful to 
-       * make sure all associated splits are in proper order
-       * in thier accounts.  The easiest way of ensuring this
-       * is to remove and reinsert every split. The reinsertion
-       * process will place the split in the correct date-sorted
-       * order.
-       */
-      i=0;
-      split = trans->splits[i];
-      while (split) {
-         acc = split ->acc;
-         xaccCheckDateOrder(acc, trans->splits[i]);
-         i++;
-         split = trans->splits[i];
-      }
-   }
-
+   /* Make sure all associated splits are in proper order
+    * in their accounts. */
    i=0;
    split = trans->splits[i];
    while (split) {
       acc = split ->acc;
+      xaccCheckDateOrder(acc, trans->splits[i]);
+      i++;
+      split = trans->splits[i];
+   }
+
+   /* Recompute the account balances. */
+   i=0;
+   split = trans->splits[i];
+   while (split) {
+      acc = split->acc;
       xaccAccountRecomputeBalance (acc); 
       i++;
       split = trans->splits[i];
@@ -864,6 +975,7 @@ xaccTransCommitEdit (Transaction *trans)
    xaccFreeTransaction (trans->orig);
    trans->orig = NULL;
 
+   LEAVE ("xaccTransCommitEdit(): trans addr=%p\n", trans);
 }
 
 void
@@ -876,6 +988,7 @@ xaccTransRollbackEdit (Transaction *trans)
 
    if (!trans) return;
    CHECK_OPEN (trans);
+   ENTER ("xaccTransRollbackEdit(): trans addr=%p\n", trans);
 
    /* copy the original values back in. */
    orig = trans->orig;
@@ -971,8 +1084,16 @@ xaccTransRollbackEdit (Transaction *trans)
    xaccFreeTransaction (trans->orig);
    trans->orig = NULL;
    trans->open = 0;
+   LEAVE ("xaccTransRollbackEdit(): trans addr=%p\n", trans);
 }
 
+gncBoolean
+xaccTransIsOpen (Transaction *trans)
+{
+  if (trans == NULL) return GNC_F;
+
+  return ((trans->open & BEGIN_EDIT) != 0);
+}
 
 /********************************************************************\
 \********************************************************************/
@@ -1245,7 +1366,7 @@ xaccSplitOrder (Split **sa, Split **sb)
   SAFE_STRCMP (da, db);
 
   /* the reconciled flag ... */
-  diff = ((*sa)->reconciled) - ((*sb)->reconciled) ;
+  diff = ((*sa)->reconciled) - ((*sb)->reconciled);
   if (diff) return diff;
 
   /* if dates differ, return */
@@ -1259,6 +1380,53 @@ xaccSplitOrder (Split **sa, Split **sb)
   return 0;
 }
 
+int
+xaccSplitMatch (Split **sa, Split **sb)
+{
+  char *da, *db;
+  char diff;
+
+  if ( (*sa) && !(*sb) ) return -1;
+  if ( !(*sa) && (*sb) ) return +1;
+  if ( !(*sa) && !(*sb) ) return 0;
+
+  /* compare amounts use parenthesis paranoia for multiplication, pointers etc. */
+  if ( ((((*sa)->damount)*((*sa)->share_price))+EPS) < 
+        (((*sb)->damount)*((*sb)->share_price))) return -1;
+
+  if ( ((((*sa)->damount)*((*sa)->share_price))-EPS) > 
+        (((*sb)->damount)*((*sb)->share_price))) return +1;
+
+  if ((((*sa)->share_price)+EPS) < ((*sb)->share_price)) return -1;
+  if ((((*sa)->share_price)-EPS) > ((*sb)->share_price)) return +1;
+
+  /* otherwise, sort on memo strings */
+  da = (*sa)->memo;
+  db = (*sb)->memo;
+  SAFE_STRCMP (da, db);
+
+  /* otherwise, sort on action strings */
+  da = (*sa)->action;
+  db = (*sb)->action;
+  SAFE_STRCMP (da, db);
+
+  /* If the reconciled flags are different, don't compare the
+   * dates, since we want to match splits with different reconciled
+   * values. But if they do match, the dates must match as well. 
+   * Note that 
+   */
+  diff = ((*sa)->reconciled) - ((*sb)->reconciled);
+  if (!diff) {
+    DATE_CMP(sa,sb,date_reconciled);
+  }
+
+  /* otherwise, sort on docref string */
+  da = (*sa)->docref;
+  db = (*sb)->docref;
+  SAFE_STRCMP (da, db);
+
+  return 0;
+}
 
 int
 xaccTransOrder (Transaction **ta, Transaction **tb)
@@ -1326,8 +1494,8 @@ xaccTransMatch (Transaction **tap, Transaction **tbp)
      nb=0; 
      while ((sb=tb->splits[nb])) { 
         if (-1 < sb->tickee) {nb++; continue;}
-        retval = xaccSplitOrder (&sa, &sb);
-        if ((0 == retval) && (sa->acc = sb->acc)) {
+        retval = xaccSplitMatch (&sa, &sb);
+        if ((0 == retval) && (sa->acc == sb->acc)) {
            sb->tickee = na;
            sa->tickee = nb;
            break;
@@ -1375,6 +1543,8 @@ xaccTransSetDateSecs (Transaction *trans, time_t secs)
 {
    if (!trans) return;
    CHECK_OPEN (trans);
+   PINFO ("xaccTransSetDateSecs(): addr=%p set date to %lu %s \n",
+           trans, secs, ctime (&secs));
 
    trans->date_posted.tv_sec = secs;
    trans->date_posted.tv_nsec = 0;
@@ -1389,6 +1559,21 @@ xaccTransSetDateSecs (Transaction *trans, time_t secs)
 
 }
 
+void
+xaccTransSetDateSecsL (Transaction *trans, long long secs)
+{
+   if (!trans) return;
+   CHECK_OPEN (trans);
+   DEBUGCMD ({ 
+      time_t sicko = secs;
+      PINFO ("xaccTransSetDateSecsL(): addr=%p set date to %Lu %s \n",
+              trans, secs, ctime (&sicko));
+   })
+
+   trans->date_posted.tv_sec = secs;
+   trans->date_posted.tv_nsec = 0;
+}
+
 void
 xaccTransSetDateEnteredSecs (Transaction *trans, time_t secs)
 {
@@ -1400,17 +1585,22 @@ xaccTransSetDateEnteredSecs (Transaction *trans, time_t secs)
 }
 
 void
-xaccTransSetDateTS (Transaction *trans, struct timespec *ts)
+xaccTransSetDateTS (Transaction *trans, const Timespec *ts)
 {
    if (!trans) return;
    CHECK_OPEN (trans);
+   DEBUGCMD ({
+         time_t sicko = ts->tv_sec;
+         PINFO ("xaccTransSetDateTS(): addr=%p set date to %Lu %s \n",
+                trans, ts->tv_sec, ctime (&sicko));
+    })
 
    trans->date_posted.tv_sec = ts->tv_sec;
    trans->date_posted.tv_nsec = ts->tv_nsec;
 }
 
 void
-xaccTransSetDateEnteredTS (Transaction *trans, struct timespec *ts)
+xaccTransSetDateEnteredTS (Transaction *trans, const Timespec *ts)
 {
    if (!trans) return;
    CHECK_OPEN (trans);
@@ -1419,23 +1609,51 @@ xaccTransSetDateEnteredTS (Transaction *trans, struct timespec *ts)
    trans->date_entered.tv_nsec = ts->tv_nsec;
 }
 
+#define THIRTY_TWO_YEARS 0x3c30fc00LL
+
+Timespec
+gnc_dmy2timespec(int day, int month, int year) {
+  Timespec result;
+  struct tm date;
+  long long secs = 0;
+  long long era = 0;
+  
+  year -= 1900;
+  
+  /* make a crude attempt to deal with dates outside the 
+   * range of Dec 1901 to Jan 2038. Note the we screw up 
+   * centenial leap years here ... so hack alert --
+   */
+  if ((2 > year) || (136 < year)) 
+  {
+    era = year / 32;
+    year %= 32;
+    if (0 > year) { year += 32; era -= 1; } 
+  }
+  
+  date.tm_year = year;
+  date.tm_mon = month - 1;
+  date.tm_mday = day;
+  date.tm_hour = 11;
+  date.tm_min = 0;
+  date.tm_sec = 0;
+  
+  /* compute number of seconds */
+  secs = mktime (&date);
+  
+  secs += era * THIRTY_TWO_YEARS;
+  
+  result.tv_sec = secs;
+  result.tv_nsec = 0;
+  
+  return(result);
+}
+
+
 void
-xaccTransSetDate (Transaction *trans, int day, int mon, int year)
-{
-   struct tm date;
-   time_t secs;
-
-   date.tm_year = year - 1900;
-   date.tm_mon = mon - 1;
-   date.tm_mday = day;
-   date.tm_hour = 11;
-   date.tm_min = 0;
-   date.tm_sec = 0;
-
-   /* compute number of seconds */
-   secs = mktime (&date);
-
-   xaccTransSetDateSecs (trans, secs);
+xaccTransSetDate (Transaction *trans, int day, int mon, int year) {
+  Timespec ts = gnc_dmy2timespec(day, mon, year);
+  xaccTransSetDateTS (trans, &ts);
 }
 
 void
@@ -1449,6 +1667,10 @@ xaccTransSetDateToday (Transaction *trans)
    gettimeofday (&tv, NULL);
    trans->date_posted.tv_sec = tv.tv_sec;
    trans->date_posted.tv_nsec = 1000 * tv.tv_usec;
+
+   PINFO ("xaccTransSetDateToday(): addr=%p set date to %lu %s \n",
+         trans, tv.tv_sec, ctime ((time_t *)&tv.tv_sec));
+
 }
 
 
@@ -1581,6 +1803,13 @@ xaccTransGetDate (Transaction *trans)
    return (trans->date_posted.tv_sec);
 }
 
+long long 
+xaccTransGetDateL (Transaction *trans)
+{
+   if (!trans) return 0;
+   return (trans->date_posted.tv_sec);
+}
+
 void
 xaccTransGetDateTS (Transaction *trans, Timespec *ts)
 {
@@ -1665,7 +1894,7 @@ xaccSplitSetDateReconciledSecs (Split *split, time_t secs)
 }
 
 void
-xaccSplitSetDateReconciledTS (Split *split, struct timespec *ts)
+xaccSplitSetDateReconciledTS (Split *split, Timespec *ts)
 {
    if (!split) return;
    MARK_SPLIT (split);
@@ -1779,6 +2008,36 @@ xaccGetAccountByName (Transaction *trans, const char * name)
 /********************************************************************\
 \********************************************************************/
 
+Account *
+xaccGetAccountByFullName (Transaction *trans, const char * name,
+                          const char separator)
+{
+   Split *s;
+   Account *acc = NULL;
+   int i;
+
+   if (!trans) return NULL;
+   if (!name) return NULL;
+
+   /* walk through the splits, looking for one, any one, that has a parent account */
+   i = 0;
+   s = trans->splits[0];
+   while (s) {
+      acc = s->acc;
+      if (acc) break;
+      i++;
+      s = trans->splits[i];
+   }
+   
+   if (!acc) return 0x0;
+
+   acc = xaccGetPeerAccountFromFullName (acc, name, separator);
+   return acc;
+}
+
+/********************************************************************\
+\********************************************************************/
+
 Split *
 xaccGetOtherSplit (Split *split)
 {
@@ -1809,5 +2068,22 @@ xaccIsPeerSplit (Split *sa, Split *sb)
    return 0;
 }
 
+/********************************************************************\
+\********************************************************************/
+
+Split *
+IthSplit (Split **list, int i)
+{
+   if (!list || 0 > i) return NULL;
+   return list[i];
+}
+
+Transaction *
+IthTransaction (Transaction **list, int i)
+{
+   if (!list || 0 > i) return NULL;
+   return list[i];
+}
+
 /************************ END OF ************************************\
 \************************* FILE *************************************/
