@@ -1,7 +1,7 @@
 /********************************************************************\
  * Transaction.h -- defines transaction for xacc (X-Accountant)     *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997, 1998 Linas Vepstas                           *
+ * Copyright (C) 1997, 1998, 1999, 2000 Linas Vepstas               *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -27,6 +27,7 @@
 #define __XACC_TRANSACTION_H__
 
 #include "config.h"
+#include "gnc-common.h"
 
 #include <time.h>
 
@@ -44,7 +45,7 @@
  */
 
 /* A split transaction is one which shows up as a credit (or debit) in
- * one account, and peices of it show up as debits (or credits) in other
+ * one account, and pieces of it show up as debits (or credits) in other
  * accounts.  Thus, a single credit-card transaction might be split
  * between "dining", "tips" and "taxes" categories.
  */
@@ -54,21 +55,58 @@ typedef struct _account_group AccountGroup;
 typedef struct _split         Split;
 typedef struct _transaction   Transaction;
 
+/* struct timespec64 is just like timespec except that we use 
+ * a 64-bit signed int to store the seconds.  This should adequately
+ * cover dates in the distant future as well as the distant past, as long
+ * as they're not more than a couple dozen times the age of the universe.
+ * Note that both gcc and the IBM Toronto xlC compiler (aka CSet,
+ * VisualAge, etc) correctly handle long long as a 64 bit quantity,
+ * even on the 32-bit Intel x86 and PowerPC architectures.  I'm assuming
+ * that all the other modern compilers are clean on this issue too ...
+ */
+#ifndef SWIG   /* swig 1.1p5 can't hack the long long type */
+struct timespec64 {
+   long long int tv_sec;     
+   long int tv_nsec;
+};
+#endif /* SWIG */
+typedef struct timespec64 Timespec;
 
 /** PROTOTYPES ******************************************************/
+
 /*
- * The xaccMallocTransaction() will malloc memory and initilize it.
+ * The xaccConfigSetForceDoubleEntry() and xaccConfigGetForceDoubleEntry()
+ *    set and get the "force_double_entry" flag.  This flag determines how
+ *    the splits in a transaction will be balanced.
+ *
+ *    The following values have significance:
+ *    0 -- anything goes
+ *    1 -- The sum of all splits in a transaction will be
+ *         forced to be zero, even if this requires the
+ *         creation of additional splits.  Note that a split
+ *         whose value is zero (e.g. a stock price) can exist
+ *         by itself. Otherwise, all splits must come in at
+ *         least pairs.
+ *    2 -- splits without parents will be forced into a
+ *         lost & found account.  (Not implemented)
+ */
+
+void   xaccConfigSetForceDoubleEntry (int force);
+int    xaccConfigGetForceDoubleEntry (void);
+
+/*
+ * The xaccMallocTransaction() will malloc memory and initialize it.
  *    Once created, it is usually unsafe to merely "free" this memory;
  *    the xaccTransDestroy() method should be called.
  *
  * The xaccInitTransaction() method will initialize the indicated memory 
- *    area.  Handy for on-stack trasnactions.
+ *    area.  Handy for on-stack transactions.
  */ 
 Transaction * xaccMallocTransaction (void); 
 void          xaccInitTransaction (Transaction *);
 
 /* The xaccTransDestroy() method will remove all 
- *    of the splits from each of thier accounts, free the memory
+ *    of the splits from each of their accounts, free the memory
  *    associated with them.  This routine must be followed by either
  *    an xaccTransCommitEdit(), in which case the transaction 
  *    memory will be freed, or by xaccTransRollbackEdit(), in which 
@@ -78,16 +116,16 @@ void          xaccInitTransaction (Transaction *);
 void          xaccTransDestroy (Transaction *);
 
 /* The xaccTransBeginEdit() method must be called before any changes
- *    are made to a transaction or any of its componenet splits.  If 
+ *    are made to a transaction or any of its component splits.  If 
  *    this is not done, errors will result.  If the defer flag is set, 
  *    then the automated re-balancing of all splits in this transaction
- *    is defered until the xaccTransCommitEdit() call. This allows 
+ *    is deferred until the xaccTransCommitEdit() call. This allows 
  *    multiple splits to be edited, and prices fiddled with, and the whole
  *    system sent temporarily out of balance, up until the Commit
  *    call is made when double-entry is once again enforced.
  *
- * The xaccTransCommitEdit() method should be used toindicate that 
- *    all of the manipulations on teh transaction are complete, and
+ * The xaccTransCommitEdit() method should be used to indicate that 
+ *    all of the manipulations on the transaction are complete, and
  *    that these should be made permanent.  Note that this routine
  *    may result in the deletion of the transaction, if the transaction 
  *    is "empty" (has no splits, or * has a single split in it whose 
@@ -98,24 +136,32 @@ void          xaccTransDestroy (Transaction *);
  *    started.  This includes restoring any deleted splits, removing
  *    any added splits, and undoing the effects of xaccTransDestroy,
  *    as well as restoring prices, memo's descriptions, etc.
+ *
+ * The xaccTransIsOpen() method returns GNC_T if the transaction
+ *    is open for editing. Otherwise, it returns false.
  */
 void          xaccTransBeginEdit (Transaction *, int defer);
 void          xaccTransCommitEdit (Transaction *);
 void          xaccTransRollbackEdit (Transaction *);
 
+gncBoolean    xaccTransIsOpen (Transaction *trans);
+
+/* Convert a day, month, and year to a Timespec */
+Timespec      gnc_dmy2timespec(int day, int month, int year);
+
 /*
  * The xaccTransSetDateSecs() method will modify the posted date 
  *    of the transaction.  (Footnote: this shouldn't matter to a user,
  *    but anyone modifying the engine should understand that when
  *    xaccTransCommitEdit() is called, the date order of each of the 
- *    component splits will be checked, and will be resotred in 
+ *    component splits will be checked, and will be restored in 
  *    ascending date order.)
  *
  * The xaccTransSetDate() method does the same thing as 
  *    xaccTransSetDateSecs(), but takes a convenient day-month-year format.
  *
  * The xaccTransSetDateTS() method does the same thing as 
- *    xaccTransSetDateSecs(), but takes a struct timespec.
+ *    xaccTransSetDateSecs(), but takes a struct timespec64.
  *
  * The xaccTransSetDateToday() method does the same thing as 
  *    xaccTransSetDateSecs(), but sets the date to the current system
@@ -124,10 +170,10 @@ void          xaccTransRollbackEdit (Transaction *);
 void          xaccTransSetDate (Transaction *, int day, int mon, int year);
 void          xaccTransSetDateSecs (Transaction *, time_t);
 void          xaccTransSetDateToday (Transaction *);
-void          xaccTransSetDateTS (Transaction *, struct timespec *);
+void          xaccTransSetDateTS (Transaction *, const Timespec *);
 
 void          xaccTransSetDateEnteredSecs (Transaction *, time_t);
-void          xaccTransSetDateEnteredTS (Transaction *, struct timespec *);
+void          xaccTransSetDateEnteredTS (Transaction *, const Timespec *);
 
 
 /* set the Num and Description fields ... */
@@ -136,7 +182,7 @@ void          xaccTransSetDescription (Transaction *, const char *);
 
 /* The xaccTransSetMemo() and xaccTransSetAction() methods are
  * convenience routines to keep the memo and action fields
- * of two-split trasnactions in sync.  If the transaction has
+ * of two-split transactions in sync.  If the transaction has
  * precisely two splits, then these routines will set the memo 
  * and action of both splits.   Otherwise, they will set the
  * memo and action of the first split (source split) only.
@@ -148,15 +194,15 @@ void          xaccTransSetDocref (Transaction *, const char *);
 /*
  * The xaccTransAppendSplit() method will append the indicated 
  *    split to the collection of splits in this transaction.
- *    If the split is alredy a part of another transaction,
+ *    If the split is already a part of another transaction,
  *    it will be removed from that transaction first.
  */
 void          xaccTransAppendSplit (Transaction *, Split *);
 
 /* 
  * The xaccSplitDestroy() method will update its parent account and 
- *    transaction in a consistent maner, resulting in the complete 
- *    unlinking of the split, and the freeing of it's associated memory.
+ *    transaction in a consistent manner, resulting in the complete 
+ *    unlinking of the split, and the freeing of its associated memory.
  *    The goal of this routine is to perform the removal and destruction
  *    of the split in an atomic fashion, with no chance of accidentally
  *    leaving the accounting structure out-of-balance or otherwise
@@ -176,8 +222,8 @@ void          xaccSplitDestroy (Split *);
 /* The xaccTransGetSplit() method returns a pointer to each of the 
  *    splits in this transaction.  Valid values for i are zero to 
  *    (number_of__splits-1).  An invalid value of i will cause NULL to
- *    be returned.  A conenient way of cycling through all splits is
- *    to start at zero, and kep incrementing until a null value is returned.
+ *    be returned.  A convenient way of cycling through all splits is
+ *    to start at zero, and keep incrementing until a null value is returned.
  */
 Split *       xaccTransGetSplit (Transaction *trans, int i);
 
@@ -188,12 +234,66 @@ char *        xaccTransGetNum (Transaction *);
 char *        xaccTransGetDescription (Transaction *);
 char *        xaccTransGetDocref (Transaction *);
 time_t        xaccTransGetDate (Transaction *);
-void          xaccTransGetDateTS (Transaction *, struct timespec *);
-void          xaccTransGetDateEnteredTS (Transaction *, struct timespec *);
+#ifndef SWIG  /* swig chokes on long long */
+long long     xaccTransGetDateL (Transaction *);
+#endif
+void          xaccTransGetDateTS (Transaction *, Timespec *);
+void          xaccTransGetDateEnteredTS (Transaction *, Timespec *);
 
 /* return the number of splits */
 int           xaccTransCountSplits (Transaction *trans);
 
+
+/* xaccIsCommonCurrency returns true if the given currency/security
+ * pairs have a currency in common. It uses the same method as
+ * xaccTransFindCommonCurrency. This method is useful for determining
+ * whether two accounts can have transactions in common.
+ */
+gncBoolean xaccIsCommonCurrency(char *currency_1, char *security_1,
+				char *currency_2, char *security_2);
+
+/* The xaccTransFindCommonCurrency () method returns a string value 
+ *    indicating a currency denomination that all of the splits in this
+ *    transaction have in common.  This routine is useful in dealing
+ *    with currency trading accounts and/or with "stock boxes", where
+ *    securities of differing types are moved across accounts.
+ *    It returns NULL if the transaction is internally inconsistent.
+ *    (This should never ??? happen, as it would be an internal error).
+ *
+ *    If all of the splits share both a common security and a common currency,
+ *    then the string name for the currency is returned.
+ */
+char * xaccTransFindCommonCurrency (Transaction *trans);
+
+/* The xaccTransIsCommonCurrency () method compares the input string
+ *    to the currency/security denominations of all splits in the
+ *    transaction, and returns the input string if it is common with
+ *    all the splits, otherwise, it returns NULL.
+ *
+ *    Note that this routine is *not* merely a string compare on the
+ *    value returned by TransFindCommonCurrency().  This is because
+ *    all of the splits in a transaction may share *both* a common
+ *    currency and a common security.  If the desired match is the
+ *    security, a simple string match won't reveal this fact.
+ *
+ *    This routine is useful in dealing
+ *    with currency trading accounts and/or with "stock boxes", where
+ *    transaction have in common.  This routine is useful in dealing
+ *    securities of differing types are moved across accounts.
+ */
+char * xaccTransIsCommonCurrency (Transaction *trans, char * currency);
+
+/* The xaccTransGetImbalance() method returns the total value of the
+ *    transaction.  In a pure double-entry system, this imbalance
+ *    should be exactly zero, and if it is not, something is broken.
+ *    However, when double-entry semantics are not enforced, unbalanced
+ *    transactions can sneak in, and this routine can be used to find
+ *    out how much things are off by.  The value returned is denominated
+ *    in the currency that is returned by the xaccTransFindCommonCurrency()
+ *    method.
+ */
+double xaccTransGetImbalance (Transaction * trans);
+
 /* ------------- splits --------------- */
 Split       * xaccMallocSplit (void);
 void          xaccInitSplit   (Split *);    /* clears a split struct */
@@ -218,8 +318,8 @@ void          xaccSplitSetDocref (Split *, const char *);
  */
 void          xaccSplitSetReconcile (Split *, char);
 void          xaccSplitSetDateReconciledSecs (Split *, time_t);
-void          xaccSplitSetDateReconciledTS (Split *, struct timespec *);
-void          xaccSplitGetDateReconciledTS (Split *, struct timespec *);
+void          xaccSplitSetDateReconciledTS (Split *, Timespec *);
+void          xaccSplitGetDateReconciledTS (Split *, Timespec *);
 
 
 /* 
@@ -241,7 +341,7 @@ void          xaccSplitGetDateReconciledTS (Split *, struct timespec *);
  *     update the share price and the number of shares. This 
  *     is a utility routine that is equivalent to a xaccSplitSetSharePrice()
  *     followed by and xaccSplitSetAmount(), except that it incurs the
- *     processing overhead of balancing only once, instead of twise.
+ *     processing overhead of balancing only once, instead of twice.
  */
 
 void         xaccSplitSetSharePriceAndAmount (Split *, double price,
@@ -274,6 +374,7 @@ double xaccSplitGetBalance (Split *);
 double xaccSplitGetClearedBalance (Split *);
 double xaccSplitGetReconciledBalance (Split *);
 double xaccSplitGetShareBalance (Split *);
+double xaccSplitGetCostBasis (Split *);
 double xaccSplitGetBaseValue (Split *s, char *base_currency);
 
 
@@ -316,6 +417,16 @@ Account *     xaccSplitGetAccount (Split *);
  *    Finally, it returns zero if all of the above match.
  *    Note that it does *NOT* compare its parent transaction.
  *
+ * The xaccSplitMatch(sa,sb) method works like the xaccSplitOrder method
+ *    except for the reconciled flag and reconciled dates. If the two
+ *    splits have different reconciled flags, then neither the reconciled
+ *    flags nor the reconciled dates are used in the comparison. If the
+ *    reconciled flags are the same, the reconciled dates are compared
+ *    as in xaccSplitOrder. This method is useful for matching splits
+ *    which are "almost" the same, as would be generated by loading
+ *    two QIF files for two different accounts that have transactions
+ *    in common.
+ *
  * The xaccSplitDateOrder(sa,sb) method is useful for sorting.
  *    It is just like the xaccSplitOrder() routine, except that it first
  *    calls xaccTransOrder(), and then does split comparisons only if
@@ -324,13 +435,14 @@ Account *     xaccSplitGetAccount (Split *);
  * The xaccTransMatch() method is just like the xaccTransOrder method, 
  *    except that it also performs a comparison of each of its child splits,
  *    and returns a zero (match) condition only if they match as well.
- *    Note that split-matching includes matching thier parent accounts.
+ *    Note that split-matching includes matching their parent accounts.
  */
 
 int  xaccTransOrder     (Transaction **ta, Transaction **tb);
 int  xaccTransMatch     (Transaction **ta, Transaction **tb);
 int  xaccSplitOrder     (Split **sa, Split **sb);
 int  xaccSplitDateOrder (Split **sa, Split **sb);
+int  xaccSplitMatch     (Split **sa, Split **sb);
 
 /********************************************************************\
  * Miscellaneous utility routines.
@@ -347,8 +459,14 @@ int xaccCountSplits (Split **sarray);
  * The xaccGetAccountByName() is a convenience routine that 
  *    is essentially identical to xaccGetPeerAccountFromName(),
  *    except that it accepts the handy transaction as root.
+ *
+ * The xaccGetAccountByFullName routine is similar, but uses
+ *    full names using the given separator.
  */
 Account * xaccGetAccountByName (Transaction *, const char *);
+Account * xaccGetAccountByFullName (Transaction *trans,
+                                    const char *name,
+                                    const char separator);
 
 /* 
  * The xaccGetOtherSplit() is a convenience routine that returns
@@ -363,4 +481,12 @@ Split * xaccGetOtherSplit (Split *);
  */
 int xaccIsPeerSplit (Split *, Split *);
 
+/* The IthSplit() and IthTransaction() routines merely dereference
+ *    the lists supplied as arguments; i.e. they return list[i].
+ *    These routines are needed by the perl swig wrappers, which
+ *    are unable to dereference on their own.
+ */
+Transaction * IthTransaction (Transaction **tarray, int i);
+Split       * IthSplit       (Split **sarray,       int i);
+
 #endif /* __XACC_TRANSACTION_H__ */
