@@ -2,7 +2,7 @@
  * AccWindow.c -- window for creating new accounts for xacc         *
  *                (X-Accountant)                                    *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997 Linas Vepstas                                 *
+ * Copyright (C) 1997, 1998, 1999 Linas Vepstas                     *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -41,15 +41,28 @@
 #include "Account.h"
 #include "AccountMenu.h"
 #include "AccWindow.h"
+#include "BuildMenu.h"
 #include "Group.h"
 #include "top-level.h"
-#include "messages.h"
+#include "enriched-messages.h"
 #include "MainWindow.h"
+#include "PriceMenu.h"
 #include "RegWindow.h"
+#include "Refresh.h"
+#include "FileDialog.h"
 #include "TextBox.h"
 #include "util.h"
 #include "xtutil.h"
 
+/* this global parameter determines whether the user is allowed to make
+ * unsafe edits on the data file.  This includes changing the name of the 
+ * currency and/or the security -- since any chagnes here can seriously scrmble
+ * currency and security trading accounts.
+ * hack alert --  this should be a user-configurable parameter.
+ * see also unsafe_ops in the engine ... 
+ */
+int unsafe_edits = 1;
+
 /* this define sets how many digits should be used in the pre-guessed account
  * codes.  Default to three; -- hack alert -- this should be a user-configurable
  * paramter.
@@ -67,11 +80,15 @@ struct _accwindow {
   Widget currency;       /* Account denomination field              */
   Widget security;       /* Account stock/security field            */
   Widget sec_label;      /* Account stock/security label            */
+  Widget psource;        /* Account stock price source field        */
+  Widget psrc_label;     /* Account stock price sourc label         */
   Widget code;           /* Account code field                      */
 
-  AccountMenu *accMenu;
+  AccountMenu  *accMenu;
+  PriceMenu    *prcMenu;
 
-  Account *newacc;       /* tmp account for editing */
+  Account      *newacc;  /* tmp account for editing */
+  AccountGroup *topgrp;  /* top of account heirarchy */
 
 };
 
@@ -82,9 +99,11 @@ struct _editaccwindow {
   Widget  desc;          /* Account description text field          */
   Widget  currency;      /* Account denomination field              */
   Widget  security;      /* Account stock/security field            */
+  Widget  psource;       /* Account stock price source field        */
   Widget  code;          /* Account code field                      */
   
   Account *account;      /* The account to edit                     */
+  PriceMenu    *prcMenu;
 
 };
 
@@ -94,12 +113,12 @@ struct _editnoteswindow {
 };
 
 
-/** GLOBALS *********************************************************/
-extern Widget toplevel;
-
 static EditAccWindow   ** editAccList   = NULL;
 static EditNotesWindow ** editNotesList = NULL;
 
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_GUI;
+
 /** PROTOTYPES ******************************************************/
 static void closeAccWindow      ( Widget mw, XtPointer cd, XtPointer cb );
 static void closeEditAccWindow  ( Widget mw, XtPointer cd, XtPointer cb );
@@ -108,6 +127,7 @@ static void editNotesCB         ( Widget mw, XtPointer cd, XtPointer cb );
 static void createCB            ( Widget mw, XtPointer cd, XtPointer cb );
 static void finishEditCB        ( Widget mw, XtPointer cd, XtPointer cb );
 static void selectAccountCB     ( Widget mw, XtPointer cd, XtPointer cb );
+static void priceSourceCB       ( Widget mw, XtPointer cd, XtPointer cb );
 static void selectTypeCB        ( Widget mw, XtPointer cd, XtPointer cb );
 static void closeEditNotesWindow( Widget mw, XtPointer cd, XtPointer cb );
 
@@ -122,19 +142,21 @@ EditNotesWindow * editNotesWindow (Account *acc);
  * Return: none                                                     *
 \********************************************************************/
 AccWindow *
-accWindow( Widget parent )
+accWindow (AccountGroup *grp)
   {
   int i;
   Widget    dialog, form, frame, rc, widget, 
             label, buttonform, group_menu, topwid;
   AccWindow *accData;
-  AccountGroup *grp = topgroup;  /* hack alert -- should be passed as argument */
   char *tmp;
+  Widget parent = gnc_get_ui_data();
   
-  setBusyCursor( parent );
+  ENTER ("accWindow() \n");
+  gnc_set_busy_cursor( parent );
   
   accData = (AccWindow *)_malloc(sizeof(AccWindow));
 
+  accData->topgrp = grp;
   accData->newacc = xaccMallocAccount();
   
   /* force the size of the dialog so it is not resizable */
@@ -143,7 +165,7 @@ accWindow( Widget parent )
 				 XmNtitle,            SETUP_ACCT_STR,
 				 XmNdeleteResponse,   XmDESTROY,
                                  /*
-                                  * Let the window find it's own size,
+                                  * Let the window find its own size,
                                   * based on the font size.
 				  * XmNwidth,     350,
 				  * XmNminWidth,  350,
@@ -287,12 +309,14 @@ accWindow( Widget parent )
 			     XmNleftAttachment,  XmATTACH_POSITION,
 			     XmNleftPosition,    35,        /* 35% */
 			     NULL );
-  
+
+  topwid = accData->currency;
+
   accData->sec_label = 
     XtVaCreateManagedWidget( SECURITY_C_STR,
 			     xmLabelGadgetClass, form,
 			     XmNtopAttachment,   XmATTACH_WIDGET,
-			     XmNtopWidget,       accData->currency, 
+			     XmNtopWidget,       topwid, 
 			     XmNtopOffset,       10,
 			     XmNrightAttachment, XmATTACH_POSITION,
 			     XmNrightPosition,   35,        /* 35% */
@@ -304,7 +328,7 @@ accWindow( Widget parent )
 			     XmNmaxLength,       20,
 			     XmNcolumns,         15,
 			     XmNtopAttachment,   XmATTACH_WIDGET,
-			     XmNtopWidget,       accData->currency,
+			     XmNtopWidget,       topwid,
 			     XmNtopOffset,       10, 
 			     XmNleftAttachment,  XmATTACH_POSITION,
 			     XmNleftPosition,    35,        /* 35% */
@@ -338,6 +362,7 @@ accWindow( Widget parent )
   free (tmp);
   topwid = accData->code;
   
+  /* ---------------------------------------------------------- */
   label = 
     XtVaCreateManagedWidget( PARENT_ACC_C_STR,
 			     xmLabelGadgetClass, form,
@@ -354,7 +379,7 @@ accWindow( Widget parent )
   xaccAccountMenuAddCallback (accData->accMenu, selectAccountCB, (XtPointer) accData);
 
   XtVaSetValues( group_menu,
-                             XmNtopAttachment,  XmATTACH_WIDGET,
+                             XmNtopAttachment,   XmATTACH_WIDGET,
 			     XmNtopWidget,       topwid,
 			     XmNtopOffset,       10, 
 			     XmNleftAttachment,  XmATTACH_POSITION,
@@ -362,13 +387,40 @@ accWindow( Widget parent )
                              NULL );
   
   XtManageChild (group_menu); 
+  topwid = group_menu;
+
+  /* ---------------------------------------------------------- */
+  /* create another menu box to allow user to select a data source */
+  accData->psrc_label = 
+    XtVaCreateManagedWidget( QUOTE_SRC_C_STR,
+			     xmLabelGadgetClass, form,
+			     XmNtopAttachment,   XmATTACH_WIDGET,
+			     XmNtopWidget,       topwid, 
+			     XmNtopOffset,       10,
+			     XmNrightAttachment, XmATTACH_POSITION,
+			     XmNrightPosition,   35,        /* 35% */
+			     NULL );
+  
+  accData->prcMenu = xaccBuildPriceMenu (form, "");
+  accData->psource = xaccGetPriceMenuWidget (accData->prcMenu);
+  xaccPriceMenuAddCallback (accData->prcMenu, priceSourceCB, 
+                            (XtPointer) accData->newacc);
+
+  XtVaSetValues( accData->psource,
+                             XmNtopAttachment,   XmATTACH_WIDGET,
+			     XmNtopWidget,       topwid,
+			     XmNtopOffset,       10, 
+			     XmNleftAttachment,  XmATTACH_POSITION,
+			     XmNleftPosition,    35,        /* 35% */
+                             NULL );
+  
+  XtManageChild (accData->psource); 
+  topwid = accData->psource;
 
   /******************************************************************\
    * The buttons at the bottom...                                   *
   \******************************************************************/
 
-  topwid = group_menu;
-  
   buttonform = XtVaCreateWidget( "form", 
 				 xmFormWidgetClass,   form,
 				 XmNfractionBase,     5,
@@ -405,6 +457,7 @@ accWindow( Widget parent )
 				    XmNrightAttachment,    XmATTACH_POSITION,
 				    XmNrightPosition,      3,  
 				    XmNshowAsDefault,      True,
+                                    XmNnavigationType,     XmTAB_GROUP, 
 				    NULL );
   
   /* We need to do something to clean up memory too! */
@@ -421,6 +474,7 @@ accWindow( Widget parent )
 				    XmNrightAttachment,    XmATTACH_POSITION,
 				    XmNrightPosition,      4,  
 				    XmNshowAsDefault,      True,
+                                    XmNnavigationType,     XmTAB_GROUP, 
 				    NULL );
   
   XtAddCallback( widget, XmNactivateCallback, 
@@ -436,8 +490,10 @@ accWindow( Widget parent )
   XtVaSetValues (accData->type_widgets[BANK],
                              XmNset,             True,
                              NULL);
-  XtSetSensitive (accData->security,  False);
+  XtSetSensitive (accData->security,   False);
   XtSetSensitive (accData->sec_label,  False);
+  XtSetSensitive (accData->psource,    False);
+  XtSetSensitive (accData->psrc_label, False);
 
   XtManageChild(buttonform);
   
@@ -446,8 +502,9 @@ accWindow( Widget parent )
   
   XtPopup( dialog, XtGrabNone );
   
-  unsetBusyCursor( parent );
+  gnc_unset_busy_cursor( parent );
 
+  LEAVE ("accWindow() \n");
   return accData;
   }
 
@@ -466,12 +523,15 @@ static void
 closeAccWindow( Widget mw, XtPointer cd, XtPointer cb )
   {
   AccWindow *accData = (AccWindow *)cd;
+  ENTER("closeAccWindow\n");
 
   if(accData->newacc) xaccFreeAccount (accData->newacc);  
 
   xaccFreeAccountMenu (accData->accMenu);
+  xaccFreePriceMenu (accData->prcMenu);
+  accData->topgrp = NULL;
   _free(accData);
-  DEBUG("close AccWindow");
+  LEAVE("closeAccWindow\n");
   }
 
 /********************************************************************\
@@ -483,33 +543,25 @@ closeAccWindow( Widget mw, XtPointer cd, XtPointer cb )
  * Return: none                                                     *
 \********************************************************************/
 EditAccWindow *
-editAccWindow( Widget parent, Account *acc )
+editAccWindow( Account *acc )
 {
 
   Widget dialog, form, widget, label, buttonform, lasttext;
   EditAccWindow *editAccData;
   char * str;
   int acctype;
+  Widget parent = gnc_get_ui_data();
+  ENTER("editAccWindow\n");
   
   FETCH_FROM_LIST (EditAccWindow, editAccList, acc, account, editAccData);
 
-  setBusyCursor( parent );
+  gnc_set_busy_cursor( parent );
   
   /* force the size of the dialog so it is not resizable */
   dialog = XtVaCreatePopupShell( "dialog", 
 				 xmDialogShellWidgetClass, parent,
 				 XmNtitle,            EDIT_ACCT_STR,
 				 XmNdeleteResponse,   XmDESTROY,
-                                 /*
-                                  * Let the window find it's own size, 
-                                  * based on the size of the font.
-				  * XmNwidth,     350,
-				  * XmNminWidth,  350,
-				  * XmNmaxWidth,  350,
-				  * XmNheight,    150,
-				  * XmNminHeight, 150,
-				  * XmNmaxHeight, 150,
-                                  */
                                  XmNresizable, FALSE,
                                  XmNallowShellResize, FALSE,
                                  XmNtransient, FALSE,  /* allow window to be repositioned */
@@ -600,12 +652,19 @@ editAccWindow( Widget parent, Account *acc )
 			     XmNleftPosition,    35,        /* 35% */
 			     NULL );
   
+  if (unsafe_edits) {
+     /* normally, user is not allowed to change this value. */
+     XtSetSensitive (editAccData->currency,  True);
+     XtSetSensitive (label,  True);
+  } else
   if (0x0 != str[0]) {
+    /* allow user to change this value only if not yet set */
     XtSetSensitive (editAccData->currency,  False);
     XtSetSensitive (label,  False);
   }
   lasttext = editAccData->currency;
 
+  /* ------------------------------------------------------------ */
   /* the security field should only be displayed for stock trading 
    * accounts, and currency trading accounts 
    */
@@ -616,7 +675,7 @@ editAccWindow( Widget parent, Account *acc )
       XtVaCreateManagedWidget( SECURITY_C_STR,
 			     xmLabelGadgetClass, form,
 			     XmNtopAttachment,   XmATTACH_WIDGET,
-			     XmNtopWidget,       editAccData->currency, 
+			     XmNtopWidget,       lasttext, 
 			     XmNtopOffset,       10,
 			     XmNrightAttachment, XmATTACH_POSITION,
 			     XmNrightPosition,   35,        /* 35% */
@@ -631,19 +690,64 @@ editAccWindow( Widget parent, Account *acc )
 			     XmNcolumns,         15,
 			     XmNvalue,           str,
 			     XmNtopAttachment,   XmATTACH_WIDGET,
-			     XmNtopWidget,       editAccData->currency,
+			     XmNtopWidget,       lasttext,
 			     XmNtopOffset,       10, 
 			     XmNleftAttachment,  XmATTACH_POSITION,
 			     XmNleftPosition,    35,        /* 35% */
 			     NULL );
   
+    if (unsafe_edits) {
+       /* normally, user is not allowed to change this value. */
+       XtSetSensitive (editAccData->security,  True);
+       XtSetSensitive (label,  True);
+    } else
     if (0x0 != str[0]) {
+      /* allow user to change this value only if not yet set */
       XtSetSensitive (editAccData->security,  False);
       XtSetSensitive (label,  False);
     }
+
+
     lasttext = editAccData->security;
+    /* ------------------------------------------------------------ */
+    label = 
+      XtVaCreateManagedWidget( QUOTE_SRC_C_STR,
+			     xmLabelGadgetClass, form,
+			     XmNtopAttachment,   XmATTACH_WIDGET,
+			     XmNtopWidget,       lasttext, 
+			     XmNtopOffset,       10,
+			     XmNrightAttachment, XmATTACH_POSITION,
+			     XmNrightPosition,   35,        /* 35% */
+			     NULL );
+  
+    editAccData->prcMenu = xaccBuildPriceMenu (form, "");
+    editAccData->psource = xaccGetPriceMenuWidget (editAccData->prcMenu);
+    xaccPriceMenuAddCallback (editAccData->prcMenu, priceSourceCB, 
+                            (XtPointer) editAccData->account);
+
+    {
+    /* make sure the the menu is showing the correct choice */
+    AccInfo *accinfo = xaccAccountGetAccInfo (editAccData->account);
+    InvAcct *invacct = xaccCastToInvAcct (accinfo);
+    str = "";
+    if (invacct) str = xaccInvAcctGetPriceSrc (invacct);
+    xaccPriceMenuSetValue (editAccData->prcMenu, str);
+    }
+    XtVaSetValues( editAccData->psource,
+                             XmNtopAttachment,   XmATTACH_WIDGET,
+			     XmNtopWidget,       lasttext,
+			     XmNtopOffset,       10, 
+			     XmNleftAttachment,  XmATTACH_POSITION,
+			     XmNleftPosition,    35,        /* 35% */
+                             NULL );
+  
+    XtManageChild (editAccData->psource); 
+    lasttext = editAccData->psource;
+
   } else {
     editAccData->security = 0;
+    editAccData->psource = 0;
+    editAccData->prcMenu = NULL;
   }
 
   label = 
@@ -747,8 +851,9 @@ editAccWindow( Widget parent, Account *acc )
   
   XtPopup( dialog, XtGrabNone );
   
-  unsetBusyCursor( parent );
+  gnc_unset_busy_cursor( parent );
 
+  LEAVE("editAccWindow\n");
   return editAccData;
 }
 
@@ -782,9 +887,11 @@ closeEditAccWindow( Widget mw, XtPointer cd, XtPointer cb )
   EditAccWindow *editAccData = (EditAccWindow *)cd;
   Account *acc = editAccData->account;
 
+  ENTER("closeEditAccWindow\n");
+  xaccFreePriceMenu (editAccData->prcMenu);
   REMOVE_FROM_LIST (EditAccWindow,editAccList,acc,account); 
   free(editAccData);
-  DEBUG("close EditAccWindow");
+  LEAVE("closeEditAccWindow\n");
   }
 
 /********************************************************************\
@@ -846,11 +953,12 @@ createCB( Widget mw, XtPointer cd, XtPointer cb )
 
   String name, tmp;
   
+  ENTER ("createCB for newacct\n");
   name = XmTextGetString(accData->name);
 
   /* The account has to have a name! */
   if( strcmp( name, "" ) == 0 ) {
-    errorBox (toplevel, ACC_NO_NAME_MSG);
+    gnc_error_dialog (ACC_NO_NAME_MSG);
     XtFree (name);
     return;
   }
@@ -909,18 +1017,21 @@ createCB( Widget mw, XtPointer cd, XtPointer cb )
   if (parent_acc) {
     xaccInsertSubAccount (parent_acc, acc);
   } else {
-    xaccGroupInsertAccount( topgroup, acc );
+    AccountGroup * topmostgroup = xaccGetAccountMenuTop (accData->accMenu);
+    xaccGroupInsertAccount( topmostgroup, acc );
   }
   xaccAccountCommitEdit (acc);
   
   /* make sure the accountlist is updated to reflect the new account */
-  refreshMainWindow();
+  gnc_refresh_main_window();
+  gnc_group_ui_refresh(gncGetCurrentGroup());
 
   /* open up the account window for the user */
-  regWindowSimple ( toplevel, acc );
+  regWindowSimple (acc);
 
   /* if we got to here, tear down the dialog window */
   XtDestroyWidget (accData->dialog);
+  LEAVE ("createCB for newacct\n");
   }
 
 /********************************************************************\
@@ -939,8 +1050,9 @@ finishEditCB( Widget mw, XtPointer cd, XtPointer cb )
   EditAccWindow *editAccData = (EditAccWindow *)cd;
   Account *acc = editAccData->account;
   String name, tmp;
-  char * oldval;
+  char * oldcurr;
   
+  ENTER ("finishEditC\n");
   xaccAccountBeginEdit (acc, 0);
 
   /* The account has to have a name! */
@@ -959,27 +1071,47 @@ finishEditCB( Widget mw, XtPointer cd, XtPointer cb )
   xaccAccountSetCode (acc, tmp);
   XtFree(tmp);
 
-  oldval = xaccAccountGetCurrency (acc);
-  if (!oldval) oldval = "";
-  if (0x0 == oldval[0]) {
-    tmp = XmTextGetString (editAccData->currency);
-    xaccAccountSetCurrency (acc, tmp);
-    XtFree(tmp);
+  oldcurr = xaccAccountGetCurrency (acc);
+  if (!oldcurr) oldcurr = "";
+  if ((0x0 == oldcurr[0]) || unsafe_edits) {
+    char * newcurr = XmTextGetString (editAccData->currency);
+    if (0x0 == oldcurr[0]) {
+       xaccAccountSetCurrency (acc, newcurr);
+    } else 
+    if (strcmp (oldcurr, newcurr)) {
+       char buff[1000];
+       sprintf (buff, EDIT_CURRENCY_MSG, oldcurr, newcurr);
+       if (gnc_verify_dialog (buff, GNC_T)) {
+          xaccAccountSetCurrency (acc, newcurr);
+       }
+    }
+    XtFree(newcurr);
   }
 
   /* if this field wasn't displayed, then don't try to deal with it */
   if (editAccData->security) {
-     oldval = xaccAccountGetSecurity (acc);
-     if (!oldval) oldval = "";
-     if (0x0 == oldval[0]) {
-       tmp = XmTextGetString (editAccData->security);
-       xaccAccountSetSecurity (acc, tmp);
-       XtFree(tmp);
+     char * oldsec = xaccAccountGetSecurity (acc);
+     if (!oldsec) oldsec = "";
+     if ((0x0 == oldsec[0]) || unsafe_edits) {
+       char * newsec = XmTextGetString (editAccData->security);
+       if (0x0 == oldsec[0]) {
+          xaccAccountSetSecurity (acc, newsec);
+       } else 
+       if (strcmp (oldsec, newsec)) {
+          char buff[1000];
+          sprintf (buff, EDIT_SECURITY_MSG, oldsec, newsec);
+          if (gnc_verify_dialog (buff, GNC_T)) {
+             xaccAccountSetSecurity (acc, newsec);
+          }
+       }
+       XtFree(newsec);
      }
   }  
   xaccAccountCommitEdit (acc);
   
-  refreshMainWindow();
+  gnc_refresh_main_window();
+  gnc_group_ui_refresh(gncGetCurrentGroup());
+  LEAVE ("finishEditC\n");
   }
 
 /********************************************************************\
@@ -1004,8 +1136,10 @@ selectTypeCB( Widget mw, XtPointer cd, XtPointer cb )
        case STOCK:
        case MUTUAL:
        case CURRENCY:
-          XtSetSensitive (menu->security, True);
-          XtSetSensitive (menu->sec_label, True);
+          XtSetSensitive (menu->security,   True);
+          XtSetSensitive (menu->sec_label,  True);
+          XtSetSensitive (menu->psource,    True);
+          XtSetSensitive (menu->psrc_label, True);
           break;
 
        case BANK:
@@ -1021,8 +1155,28 @@ selectTypeCB( Widget mw, XtPointer cd, XtPointer cb )
        case EXPENSE:
        case EQUITY:
        default:
-          XtSetSensitive (menu->security, False);
-          XtSetSensitive (menu->sec_label, False);
+          XtSetSensitive (menu->security,   False);
+          XtSetSensitive (menu->sec_label,  False);
+          XtSetSensitive (menu->psource,    False);
+          XtSetSensitive (menu->psrc_label, False);
+  }
+}
+
+/********************************************************************\
+ * priceSourceCB -- set the source for stock & mutual fund pricing info   * 
+ *                                                                  * 
+\********************************************************************/
+static void
+priceSourceCB( Widget mw, XtPointer cd, XtPointer cb )
+{
+  char * src = (char *) cb;
+  Account *acc = (Account *) cd;
+  AccInfo *accinfo = xaccAccountGetAccInfo (acc);
+  InvAcct *invacct;
+
+  invacct = xaccCastToInvAcct (accinfo);
+  if (invacct) {
+    xaccInvAcctSetPriceSrc (invacct, src);
   }
 }
 
@@ -1227,8 +1381,8 @@ selectAccountCB( Widget mw, XtPointer cd, XtPointer cb )
     }
   } else {
     char * tmp;
-    /* gues an account code, and fill it in */
-    tmp = xaccGroupGetNextFreeCode (topgroup, XACC_ACC_CODE_DIGITS);
+    /* guess an account code, and fill it in */
+    tmp = xaccGroupGetNextFreeCode (menu->topgrp, XACC_ACC_CODE_DIGITS);
     XtVaSetValues (menu->code, XmNvalue, tmp, NULL);
     free (tmp);
 
@@ -1260,24 +1414,19 @@ EditNotesWindow *
 editNotesWindow (Account *acc)
 {
   EditNotesWindow *enw;
-  char * notes;
+  char *notes;
 
   FETCH_FROM_LIST (EditNotesWindow, editNotesList, acc, account, enw);
   
   notes = xaccAccountGetNotes (acc);
-  enw->tb = textBox( toplevel, NOTES_STR, 
+  enw->tb = textBox( gnc_get_ui_data(), NOTES_STR, 
                      &notes,
                      closeEditNotesWindow, enw);
-  xaccAccountBeginEdit (acc, 1);
-  xaccAccountSetNotes (acc, notes);
-  xaccAccountCommitEdit (acc);
-  XtFree (notes);
-
   return enw;
 }
 
 /********************************************************************\
- * don't delete any structures; te close callack will do this       *
+ * don't delete any structures; the close callack will do this       *
 \********************************************************************/
 
 void 
@@ -1299,6 +1448,19 @@ closeEditNotesWindow( Widget mw, XtPointer cd, XtPointer cb )
 {
   EditNotesWindow *enw = (EditNotesWindow *) cd;
   Account * acc = enw->account;
+  char * newnotes = (char *) cb;
+  char *notes;
+
+  /* don't bother updating the database if the value of the field
+   * literally did not change.  Note that this callback is called 
+   * for commit and cancel buttons ... 
+   */
+  notes = xaccAccountGetNotes (acc);
+  if (safe_strcmp (notes, newnotes)) {
+    xaccAccountBeginEdit (acc, 1);
+    xaccAccountSetNotes (acc, newnotes);
+    xaccAccountCommitEdit (acc);
+  }
 
   REMOVE_FROM_LIST (EditNotesWindow,editNotesList,acc,account) 
 
