@@ -1,7 +1,7 @@
 /********************************************************************\
  * HelpWindow.c -- a help window for hypertext help.                *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1998 Linas Vepstas                                 *
+ * Copyright (C) 1998,1999 Linas Vepstas                            *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -44,23 +44,21 @@
 #  include <X11/xpm.h>
 #endif
 
-#if USE_HTMLW
-#include <HTML.h>
-#endif 
-
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
 #include <XmHTML/XmHTML.h>
 #endif 
 
 #include "File.h"
 #include "HelpWindow.h"
 #include "top-level.h"
+#include "ui-callbacks.h"
 #include "messages.h"
 #include "Sheet.h"
 #include "util.h"
 #include "xtutil.h"
 
 extern Widget toplevel;
+static short module = MOD_HTML; 
 
 /********************************************************************\
  *     HTML History functions                                       * 
@@ -80,6 +78,7 @@ typedef struct _HTMLHistory {
 static int
 historyInsert( HTMLHistory **history, const char *htmlfile )
   {
+  ENTER ("historyInsert()\n");
   if( (*history) != NULL )
     {
     HTMLHistory *temp;
@@ -109,6 +108,7 @@ historyInsert( HTMLHistory **history, const char *htmlfile )
     (*history)->next = temp;
     (*history) = temp;
     
+    LEAVE ("historyInsert(): have history\n");
     return FALSE;
     }
   else
@@ -125,6 +125,7 @@ historyInsert( HTMLHistory **history, const char *htmlfile )
     (*history)->next = NULL;
     
     /* ...so return TRUE */
+    LEAVE ("historyInsert(): new history \n");
     return TRUE;
     }
   }
@@ -205,6 +206,9 @@ historyClear( HTMLHistory **history )
 struct _HTMLWindow {
    Widget htmlwidget;
    Widget data_target_frame;
+   Widget menu_target_frame;
+   short  installed_data_frame_form_cb;
+   short  installed_menu_frame_form_cb;
    HTMLHistory *history;
 };
 
@@ -220,29 +224,15 @@ static void   htmlWindow( Widget parent, HTMLWindow **hwinp,
                           const char * const text);
 static void   closeHtmlWin( Widget mw, XtPointer cd, XtPointer cb );
 static void   frameCB( Widget mw, XtPointer cd, XtPointer cb );
+static void   formCB( Widget mw, XtPointer cd, XtPointer cb );
 static void   htmlBackCB( Widget mw, XtPointer cd, XtPointer cb );
 static void   htmlFwdCB( Widget mw, XtPointer cd, XtPointer cb );
 static void   htmlAnchorCB( Widget mw, XtPointer cd, XtPointer cb );
 
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
 static char * xaccJumpToLabel (Widget mw, const char * jumpfile, char * text);
 static XmImageInfo * htmlReadImageProc (Widget w, String file);
-#endif /* USE_XMHTML */
-
-#if USE_HTMLW
-#if HAVE_XPM
-struct _image_info {
-  int width;
-  int height;
-  int num_colors;
-  Pixmap pixmap;
-};
-
-typedef struct _image_info ImageInfo;
-  
-static ImageInfo *htmlResolveImage( Widget wm, char *file, int nl );
-#endif /* HAVE_XPM */
-#endif /* USE_HTMLW */
+#endif /* WANT_XMHTML */
 
 /********************************************************************\
  * reportWindow                                                     * 
@@ -260,6 +250,9 @@ reportWindow( Widget parent, const char *title, const char *file)
     reportwindow = (HTMLWindow *) malloc (sizeof (HTMLWindow));
     reportwindow->htmlwidget = 0;
     reportwindow->data_target_frame = 0;
+    reportwindow->menu_target_frame = 0;
+    reportwindow->installed_data_frame_form_cb = 0;
+    reportwindow->installed_menu_frame_form_cb = 0;
     reportwindow->history = NULL;
   } 
  
@@ -282,6 +275,9 @@ helpWindow( Widget parent, const char *title, const char *htmlfile )
     helpwindow = (HTMLWindow *) malloc (sizeof (HTMLWindow));
     helpwindow->htmlwidget = 0;
     helpwindow->data_target_frame = 0;
+    helpwindow->menu_target_frame = 0;
+    helpwindow->installed_data_frame_form_cb = 0;
+    helpwindow->installed_menu_frame_form_cb = 0;
     helpwindow->history = NULL;
   } 
  
@@ -313,7 +309,8 @@ htmlWindow( Widget parent,
   HTMLWindow *hw = *hwinp;
   char * text=0x0;
   
-  setBusyCursor( parent );
+  gnc_set_busy_cursor( parent );
+  ENTER ("htmlWindow()\n");
   
   historyInsert (&(hw->history), htmlfile); 
   if (htmltext) text = strdup (htmltext);
@@ -330,8 +327,8 @@ htmlWindow( Widget parent,
 				   XmNdialogStyle,  XmDIALOG_APPLICATION_MODAL,
 				   XmNtitle,        title,
 				   XmNdeleteResponse, XmDESTROY,
-				   XmNwidth,        500,
-				   XmNheight,       400,
+				   XmNwidth,        600,
+				   XmNheight,       500,
                                    XmNtransient,    FALSE,  /* allow window to be repositioned */
 
 				   XmNminWidth,     500,
@@ -359,15 +356,8 @@ htmlWindow( Widget parent,
     
     hw->htmlwidget =
       XtVaCreateManagedWidget( "html",
-#if USE_HTMLW
-			       htmlWidgetClass,         controlform,
-#if HAVE_XPM
-			       WbNresolveImageFunction, htmlResolveImage,
-#endif
-			       WbNdelayImageLoads,      False,
-#endif 
 
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
                                xmHTMLWidgetClass,       controlform,
                                XmNanchorButtons,        False, 
                                XmNimageProc,            htmlReadImageProc,
@@ -376,16 +366,14 @@ htmlWindow( Widget parent,
 			       XmNbottomAttachment,     XmATTACH_FORM,
 			       XmNleftAttachment,       XmATTACH_FORM,
 			       XmNrightAttachment,      XmATTACH_FORM,
-			       XmNwidth,                500,
-			       XmNheight,               400,
+			       XmNwidth,                600,
+			       XmNheight,               500,
 			       NULL );
     
-#if USE_HTMLW
-    XtAddCallback( hw->htmlwidget, WbNanchorCallback, htmlAnchorCB, (XtPointer) hw);
-#endif 
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
     XtAddCallback( hw->htmlwidget, XmNactivateCallback, htmlAnchorCB, (XtPointer) hw);
     XtAddCallback( hw->htmlwidget, XmNframeCallback,    frameCB, (XtPointer) hw);
+    XtAddCallback( hw->htmlwidget, XmNformCallback,     formCB, (XtPointer) hw);
 #endif
     
     hw->data_target_frame = hw->htmlwidget;
@@ -444,7 +432,7 @@ htmlWindow( Widget parent,
     
     XtManageChild( hw->htmlwidget );
     
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
     /* HTML Widget seems happier if it get text before its fully managed. */
     text = xaccJumpToLabel( hw->htmlwidget, htmlfile, text);
     hw->history->text = text;
@@ -465,13 +453,14 @@ htmlWindow( Widget parent,
     }
   else 
     {
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
     text = xaccJumpToLabel( hw->htmlwidget, htmlfile, text);
     hw->history->text = text;
 #endif
     }
   
-  unsetBusyCursor( parent );
+  gnc_unset_busy_cursor( parent );
+  LEAVE ("htmlWindow()\n");
   }
 
 /********************************************************************\
@@ -487,17 +476,19 @@ htmlWindow( Widget parent,
  * draw to ... and we could look up that name in our list.
  */
 #define DATA_TARGET "report-target"
+#define MENU_TARGET "menu-target"
 
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
 static void
 frameCB( Widget mw, XtPointer cd, XtPointer cb )
 {
   XmHTMLFrameCallbackStruct *fcs = (XmHTMLFrameCallbackStruct *) cb;
   HTMLWindow *hw = (HTMLWindow *) cd;
+  ENTER ("frameCB()\n");
   
   if (XmCR_HTML_FRAMECREATE == fcs->reason) { 
      fcs->doit = True;
-  }
+  } else
   if (XmCR_HTML_FRAME == fcs->reason) { 
      XtVaSetValues (fcs->html, XmNimageProc, htmlReadImageProc,  NULL);
      XtAddCallback (fcs->html, XmNactivateCallback, htmlAnchorCB, cd);
@@ -506,8 +497,44 @@ frameCB( Widget mw, XtPointer cd, XtPointer cb )
      /* keep track of the frame where we will be placing the reports. */
      if (!strcmp (fcs->name, DATA_TARGET)) {
         hw->data_target_frame = fcs->html;
+        if (0 == hw->installed_data_frame_form_cb) {
+           hw->installed_data_frame_form_cb = 1;
+           XtAddCallback( hw->data_target_frame, 
+               XmNformCallback, formCB, (XtPointer) hw);
+        }
+     } else 
+     if (!strcmp (fcs->name, MENU_TARGET)) {
+        hw->menu_target_frame = fcs->html;
+        if (0 == hw->installed_menu_frame_form_cb) {
+           hw->installed_menu_frame_form_cb = 1;
+           XtAddCallback( hw->menu_target_frame, 
+               XmNformCallback, formCB, (XtPointer) hw);
+        }
      }
   }
+  LEAVE ("frameCB()\n");
+}
+
+/********************************************************************\
+\********************************************************************/
+static void
+formCB( Widget mw, XtPointer cd, XtPointer cb )
+{
+  int i,n;
+  XmHTMLFormCallbackStruct *fcs = (XmHTMLFormCallbackStruct *) cb;
+  // HTMLWindow *hw = (HTMLWindow *) cd;
+  
+   PINFO ("formCB(): action=%s\n", fcs->action);
+   PINFO ("formCB(): encoding=%s\n", fcs->enctype);
+   PINFO ("formCB(): num keys=%d \n", fcs->ncomponents);
+
+   DEBUGCMD ({
+      n = fcs->ncomponents;
+      for (i=0; i<n; i++) {
+         DEBUG ("formCB(): %s=%s\n", 
+             fcs->components[i].name, fcs->components[i].value);
+      }
+   });
 }
 #endif
 
@@ -526,7 +553,7 @@ htmlBackCB( Widget mw, XtPointer cd, XtPointer cb )
   HTMLWindow *hw = (HTMLWindow *) cd;
   char *file = historyBack(&(hw->history));
   char *text = hw->history->text;
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
   text = xaccJumpToLabel( hw->htmlwidget, file, text);
   hw->history->text = text;
 #endif
@@ -547,7 +574,7 @@ htmlFwdCB( Widget mw, XtPointer cd, XtPointer cb )
   HTMLWindow *hw = (HTMLWindow *) cd;
   char *file = historyFwd(&(hw->history));
   char *text = hw->history->text;
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
   text = xaccJumpToLabel( hw->htmlwidget, file, text);
   hw->history->text = text;
 #endif
@@ -585,19 +612,8 @@ static void
 htmlAnchorCB( Widget mw, XtPointer cd, XtPointer cb )
   {
   HTMLWindow *hw = (HTMLWindow *) cd;
-#if USE_HTMLW
-  WbAnchorCallbackData *acbs = (WbAnchorCallbackData *)cb;
-  fprintf(stderr,"%d %s\n",acbs->element_id, acbs->text);
 
-  
-  if( historyInsert(&(hw->history),acbs->href) )
-    {ERROR();}     /* CB shouldn't be called if there is not history!!! */
-  else {
-    XtVaSetValues( mw, WbNtext, gncReadFile(acbs->href), NULL );
-    }
-#endif
-
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
    XmHTMLAnchorCallbackStruct *acbs = (XmHTMLAnchorCallbackStruct *) cb;
 
    if(acbs->reason != XmCR_ACTIVATE) return;
@@ -620,17 +636,17 @@ htmlAnchorCB( Widget mw, XtPointer cd, XtPointer cb )
 
       /*  other types are unsupported, but it would be fun if they were ... */
       case ANCHOR_FTP:
-         fprintf(stderr, "Error: this help window doesn't support ftp: %s\n", acbs->href);
+         PERR ("this help window doesn't support ftp: %s\n", acbs->href);
          break;
       case ANCHOR_HTTP:
-         fprintf(stderr, "Error: this help window doesn't support http: %s\n", acbs->href);
+         PERR ("this help window doesn't support http: %s\n", acbs->href);
          break;
       case ANCHOR_MAILTO:
-         fprintf(stderr, "Error: this help window doesn't support email: %s\n", acbs->href);
+         PERR("this help window doesn't support email: %s\n", acbs->href);
          break;
       case ANCHOR_UNKNOWN:
       default:
-         fprintf(stderr, "Error: don't know this type of url: %s\n", acbs->href);
+         PERR("don't know this type of url: %s\n", acbs->href);
          break;
    }
 
@@ -641,7 +657,7 @@ htmlAnchorCB( Widget mw, XtPointer cd, XtPointer cb )
  *     utility functions...                                         * 
 \********************************************************************/
 
-#if USE_XMHTML
+#if HAVE_LIBXMHTML
 static char * 
 xaccJumpToLabel (Widget mw, const char * jumpfile, char * text)
 {
@@ -676,7 +692,6 @@ xaccJumpToLabel (Widget mw, const char * jumpfile, char * text)
 
    return text;
 }
-#endif
 
 /********************************************************************\
  *     HTML functions...                                            * 
@@ -688,7 +703,6 @@ xaccJumpToLabel (Widget mw, const char * jumpfile, char * text)
  * Return: none                                                     * 
  * Global: helpPath - the path to the help files                    * 
 \********************************************************************/
-#if USE_XMHTML
 static XmImageInfo *
 htmlReadImageProc (Widget w, String file) 
   {
@@ -704,7 +718,7 @@ htmlReadImageProc (Widget w, String file)
   free(filename);
   return retval;
 }
-#endif /* USE_XMHTML */
+#endif /* WANT_XMHTML */
 
 #if USE_HTMLW
 #if HAVE_XPM
@@ -718,6 +732,15 @@ htmlReadImageProc (Widget w, String file)
  * Global: helpPath - the path to the help files                    * 
 \********************************************************************/
 
+struct _image_info {
+  int width;
+  int height;
+  int num_colors;
+  Pixmap pixmap;
+};
+
+typedef struct _image_info ImageInfo;
+
 static ImageInfo *
 htmlResolveImage( Widget mw, char *file, int nl )
   {
