@@ -39,7 +39,7 @@
 #include "date.h"
 #include "Group.h"
 #include "MainWindow.h"
-#include "messages.h"
+#include "enriched-messages.h"
 #include "MultiLedger.h"
 #include "RecnWindow.h"
 #include "util.h"
@@ -71,6 +71,9 @@ extern XtAppContext app;
 
 static RecnWindow **recnList = NULL;
 
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_GUI;
+
 /********************************************************************/
 
 /********************************************************************\
@@ -238,6 +241,11 @@ static void
 startRecnCancelCB( Widget wm, XtPointer cd, XtPointer cb )
   { *(int *)cd = 0; }
 
+static void
+help_helper( Widget mw, XtPointer cd, XtPointer cb ) {
+  gnc_show_help((gncHelpTypes) cd);
+}
+
 /********************************************************************\
  * startRecnWindow                                                  *
  *   opens up the window to prompt the user to enter the ending     *
@@ -267,7 +275,7 @@ startRecnWindow( Widget parent, Account *acc, double *diff )
   short shrs = 0;
   int acc_type;
 
-  setBusyCursor( parent );
+  gnc_set_busy_cursor( parent );
   
   /* Get the previous ending balance.  Use the published
    * account interface for this, since the ending balance
@@ -406,7 +414,7 @@ startRecnWindow( Widget parent, Account *acc, double *diff )
                                     NULL );
   
   XtAddCallback( widget, XmNactivateCallback,
-                 helpMenubarCB, (XtPointer)HMB_RECNWIN );
+                 help_helper, (XtPointer)HMB_RECNWIN );
 
   /* Fix action area of the pane to its current size, and not let it
    *  resize. */
@@ -420,7 +428,7 @@ startRecnWindow( Widget parent, Account *acc, double *diff )
   XtManageChild( pane );
   XtPopup( dialog, XtGrabNone );
   
-  unsetBusyCursor( parent );
+  gnc_unset_busy_cursor( parent );
   
   /* while the user hasn't pushed "Ok", simulate XtMainLoop. */
   while( (done == -1) || XtAppPending(app) )
@@ -464,7 +472,7 @@ recnWindow( Widget parent, Account *acc )
     
   FETCH_FROM_LIST (RecnWindow, recnList, acc, acc, recnData);
 
-  setBusyCursor(parent);
+  gnc_set_busy_cursor(parent);
   recnData->ddiff = ddiff;
   
   sprintf( title, "%s: %s", xaccAccountGetName (acc), RECONCILE_STR);
@@ -738,7 +746,7 @@ recnWindow( Widget parent, Account *acc )
                                     NULL );
   
   XtAddCallback( widget, XmNactivateCallback,
-                 helpMenubarCB, (XtPointer)HMB_RECNWIN );
+                 help_helper, (XtPointer)HMB_RECNWIN );
   
   /* Fix button area of the pane to its current size, and not let 
    * it resize. */
@@ -758,7 +766,7 @@ recnWindow( Widget parent, Account *acc )
   /* and then refresh the total/difference balance fields: */
   recnRecalculateBalance(recnData);
   
-  unsetBusyCursor(parent);
+  gnc_unset_busy_cursor(parent);
 
   return recnData;
   }
