@@ -43,34 +43,39 @@
 #include "BuildMenu.h"
 #include "datecell.h"
 #include "FileBox.h"
+#include "FileDialog.h"
 #include "Group.h"
 #include "LedgerUtils.h"
 #include "MainWindow.h"
-#include "top-level.h"
-#include "messages.h"
+#include "enriched-messages.h"
 #include "MultiLedger.h"
 #include "RecnWindow.h"
 #include "RegWindow.h"
+#include "Scrub.h"
 #include "SplitLedger.h"
 #include "table-html.h"
+#include "top-level.h"
 #include "Transaction.h"
 #include "util.h"
+#include "XferWindow.h"
+#include "top-level.h"
 #include "xtutil.h"
 
 /* The names of X11 resources for the different types of register
  * windows.  Used to set color resources, etc.
- * Must match the enums in register.h */
+ * Must match the enums in splitreg.h */
 String accRes[] ={
+  "none",
   "regbank",
   "regcash",
   "regasset",
   "regcredit",
   "regliability",
-  "regliability"
   "regincome",
   "regexpense",
   "regequity",
   "regstock",
+  "regcurrency",
   "ledgeneral",
   "ledincome",
   "ledportfolio"
@@ -97,16 +102,20 @@ struct _RegWindow {
 
 
 /** GLOBALS *********************************************************/
-extern Widget  toplevel;
+
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_GUI;
 
 /** PROTOTYPES ******************************************************/
-RegWindow * regWindowLedger( Widget parent, xaccLedgerDisplay *);
+static RegWindow * regWindowLedger(xaccLedgerDisplay *);
 static void regRefresh (xaccLedgerDisplay *ledger);
 static void regDestroy (xaccLedgerDisplay *ledger);
 
 static void closeRegWindow( Widget mw, XtPointer cd, XtPointer cb );
-static void startRecnCB( Widget mw, XtPointer cd, XtPointer cb );
-static void startAdjBCB( Widget mw, XtPointer cd, XtPointer cb );
+static void startRecnCB   ( Widget mw, XtPointer cd, XtPointer cb );
+static void startAdjBCB   ( Widget mw, XtPointer cd, XtPointer cb );
+static void scrubCB       ( Widget mw, XtPointer cd, XtPointer cb );
+static void transferCB    ( Widget mw, XtPointer cd, XtPointer cb );
 
 static void recordCB( Widget mw, XtPointer cd, XtPointer cb );
 static void deleteCB( Widget mw, XtPointer cd, XtPointer cb );
@@ -138,16 +147,16 @@ static void dateChangeCB( Widget mw, XtPointer cd, XtPointer cb );
  * Return: regData - the register window instance                   *
 \********************************************************************/
 RegWindow *
-regWindowSimple( Widget parent, Account *acc )
-  {
-  xaccLedgerDisplay * ledger;
-  RegWindow *regData;
-
-  ledger = xaccLedgerDisplaySimple (acc);
-  regData = regWindowLedger (parent, ledger);
+regWindowSimple(Account *acc)
+{
+  RegWindow *regData = NULL;
+  xaccLedgerDisplay * ledger = xaccLedgerDisplaySimple (acc);
 
+  if(ledger) {
+    regData = regWindowLedger(ledger);
+  }  
   return regData;
-  }
+}
 
 /********************************************************************\
  * regWindowAccGroup                                                *
@@ -158,16 +167,20 @@ regWindowSimple( Widget parent, Account *acc )
  * Return: regData - the register window instance                   *
 \********************************************************************/
 RegWindow *
-regWindowAccGroup( Widget parent, Account *acc )
-  {
-  xaccLedgerDisplay * ledger;
-  RegWindow *regData;
-
-  ledger = xaccLedgerDisplayAccGroup (acc);
-  regData = regWindowLedger (parent, ledger);
-
-  return regData;
+regWindowAccGroup(Account *acc)
+{
+  RegWindow *regData = NULL;
+  xaccLedgerDisplay * ledger = xaccLedgerDisplayAccGroup (acc);
+  if(ledger) {
+    regData = regWindowLedger(ledger);
   }
+  return regData;
+}
+
+static void
+help_helper( Widget mw, XtPointer cd, XtPointer cb ) {
+  gnc_show_help((gncHelpTypes) cd);
+}
 
 /********************************************************************\
  * regWindowLedger                                                  *
@@ -181,10 +194,12 @@ regWindowAccGroup( Widget parent, Account *acc )
  * Return: regData  - the register window instance                  *
 \********************************************************************/
 RegWindow *
-regWindowLedger (Widget parent, xaccLedgerDisplay *ledger)
+regWindowLedger (xaccLedgerDisplay *ledger)
   {
+  Widget parent = gnc_get_ui_data();  
   RegWindow   *regData = NULL;
   Widget menubar, pane, buttonform, frame, reg, widget;
+  Widget helpmenu;
   Dimension buttonform_height;
   int    position=0;
   char *windowname;
@@ -212,15 +227,17 @@ regWindowLedger (Widget parent, xaccLedgerDisplay *ledger)
   
   MenuItem activityMenu[] = {
     { TRANSFER_E_STR,       &xmPushButtonWidgetClass, 'T', NULL, NULL, True,
-      accountMenubarCB, (XtPointer)AMB_TRNS,   (MenuItem *)NULL, 0 },
+      transferCB,        NULL,                 (MenuItem *)NULL, 0 },
     { "",                   &xmSeparatorWidgetClass,    0, NULL, NULL, True,
       NULL,              NULL,                 (MenuItem *)NULL, 0 },
     { RECONCILE_E_STR,      &xmPushButtonWidgetClass, 'C', NULL, NULL, True,
       startRecnCB,       NULL,                 (MenuItem *)NULL, 0 },
     { ADJ_BALN_E_STR,       &xmPushButtonWidgetClass, 'A', NULL, NULL, True,
       startAdjBCB,       NULL,                 (MenuItem *)NULL, 0 },
-    { "Print...",         &xmPushButtonWidgetClass,   'R', NULL, NULL, True,
+    { PRINT_E_STR,          &xmPushButtonWidgetClass, 'R', NULL, NULL, True,
       NULL,              (XtPointer)0,         (MenuItem *)NULL, 0 },
+    { SCRUB_STR,            &xmPushButtonWidgetClass, 'S', NULL, NULL, True,
+      scrubCB,           (XtPointer)0,         (MenuItem *)NULL, 0 },
 /*
     { REPORT_E_STR,         &xmPushButtonWidgetClass, 'R', NULL, NULL, False,
       NULL,              (XtPointer)0,         (MenuItem *)NULL, 0 },
@@ -240,13 +257,13 @@ regWindowLedger (Widget parent, xaccLedgerDisplay *ledger)
   
   MenuItem helpMenu[] = {
     { ABOUT_E_STR,          &xmPushButtonWidgetClass, 'A', NULL, NULL, True,
-      helpMenubarCB, (XtPointer)HMB_ABOUT, (MenuItem *)NULL, 0 },
+      help_helper, (XtPointer)HMB_ABOUT, (MenuItem *)NULL, 0 },
     { HELP_E_STR,           &xmPushButtonWidgetClass, 'H', NULL, NULL, True,
-      helpMenubarCB, (XtPointer)HMB_REGWIN,(MenuItem *)NULL, 0 },
+      help_helper, (XtPointer)HMB_REGWIN,(MenuItem *)NULL, 0 },
     { "",                   &xmSeparatorWidgetClass,    0, NULL, NULL, True,
       NULL,          NULL,                 (MenuItem *)NULL, 0 },
     { LICENSE_E_STR,        &xmPushButtonWidgetClass, 'L', NULL, NULL, True,
-      helpMenubarCB, (XtPointer)HMB_LIC,   (MenuItem *)NULL, 0 },
+      help_helper, (XtPointer)HMB_LIC,   (MenuItem *)NULL, 0 },
     { NULL,                 NULL,                      0,  NULL, NULL, False,
       NULL,          (XtPointer)0,         (MenuItem *)NULL, 0 },
   };
@@ -290,7 +307,7 @@ regWindowLedger (Widget parent, xaccLedgerDisplay *ledger)
     windowname = "General Ledger";
   }
 
-  setBusyCursor( parent );
+  gnc_set_busy_cursor( parent );
 
   regData->dialog =
     XtVaCreatePopupShell( "dialog", 
@@ -298,7 +315,7 @@ regWindowLedger (Widget parent, xaccLedgerDisplay *ledger)
                           XmNdeleteResponse,   XmDESTROY,
                           XmNtitle,            windowname,
                           /*
-                           * Let the window find it's own size, 
+                           * Let the window find its own size, 
                            * based on the size of the fonts.
                            * XmNwidth,            395,
                            * XmNheight,           400,
@@ -346,8 +363,9 @@ regWindowLedger (Widget parent, xaccLedgerDisplay *ledger)
 
   activityMenu[2].callback_data=(XtPointer)regData;
   activityMenu[3].callback_data=(XtPointer)regData;
-  activityMenu[6].callback_data=(XtPointer)regData;
-  activityMenu[8].callback_data=(XtPointer)(regData->dialog);  /* destroy callback */
+  activityMenu[5].callback_data=(XtPointer)regData;
+  activityMenu[7].callback_data=(XtPointer)regData;
+  activityMenu[9].callback_data=(XtPointer)(regData->dialog);  /* destroy callback */
 
   activityMenu[4].subitems = reportMenu;
 
@@ -361,8 +379,11 @@ regWindowLedger (Widget parent, xaccLedgerDisplay *ledger)
   
   BuildMenu( menubar, XmMENU_PULLDOWN, ACTIVITIES_STR, 'A',
              False, 0, activityMenu );
-  BuildMenu( menubar, XmMENU_PULLDOWN, HELP_STR,       'H', 
+  helpmenu = BuildMenu( menubar, XmMENU_PULLDOWN, HELP_STR,       'H', 
              False, 0, helpMenu );
+  XtVaSetValues( menubar,
+		 XmNmenuHelpWidget,	helpmenu,
+		 NULL );
   
   XtManageChild( menubar );
   
@@ -384,22 +405,6 @@ regWindowLedger (Widget parent, xaccLedgerDisplay *ledger)
   xaccCreateCursor (ledger->ledger->table,  ledger->ledger->double_cursor);
   xaccCreateCursor (ledger->ledger->table,  ledger->ledger->trans_cursor);
   xaccCreateCursor (ledger->ledger->table,  ledger->ledger->split_cursor);
-    
-  /* complete GUI initialization */
-  {
-    AccountGroup *grp;
-    Account * base_acc;
-    grp = xaccGetAccountRoot (ledger->leader);
-    base_acc = ledger->leader;
-
-    if (!grp) {
-      grp = xaccGetAccountRoot (ledger->displayed_accounts[0]);
-      base_acc = ledger->displayed_accounts[0];
-    }
-    xaccLoadXferCell (ledger->ledger->xfrmCell, grp, base_acc);
-    xaccLoadXferCell (ledger->ledger->mxfrmCell, grp, base_acc);
-    /* xaccLoadXferCell (ledger->ledger->xtoCell, grp);  */
-  }
 
   XtManageChild (reg);
   XtManageChild (frame);
@@ -649,7 +654,7 @@ regWindowLedger (Widget parent, xaccLedgerDisplay *ledger)
   
   XtPopup( regData->dialog, XtGrabNone );
   
-  unsetBusyCursor( parent );
+  gnc_unset_busy_cursor( parent );
   
   return regData;
 }
@@ -664,12 +669,29 @@ regRefresh (xaccLedgerDisplay *ledger)
   RegWindow *regData = (RegWindow *) (ledger->gui_hook);
   xaccLedgerDisplay *ld = regData->ledger;
   char buf [BUFSIZE];
+  double amount;
   char * amt;
+  int typo;
 
-  amt = xaccPrintAmount (ledger->balance, PRTSYM);
+  xaccSRLoadXferCells (ledger->ledger, ledger->leader);
+
+  typo = ledger->ledger->type & REG_TYPE_MASK;
+
+  amount = ledger->balance;
+  if ((INCOME_REGISTER == typo) ||
+      (EXPENSE_REGISTER == typo))
+    amount = -amount;
+
+  amt = xaccPrintAmount (amount, PRTSYM | PRTSEP);
   strcpy (buf, amt);
   strcat (buf, "\n");
-  amt = xaccPrintAmount (ledger->clearedBalance, PRTSYM);
+
+  amount = ledger->clearedBalance;
+  if ((INCOME_REGISTER == typo) ||
+      (EXPENSE_REGISTER == typo))
+    amount = -amount;
+
+  amt = xaccPrintAmount (amount, PRTSYM | PRTSEP);
   strcat (buf, amt);
   XmTextSetString( regData->balance, buf );
 
@@ -699,7 +721,11 @@ regDestroy (xaccLedgerDisplay *ledger)
 {
    RegWindow *regData = (RegWindow *) (ledger->gui_hook);
 
-   if (regData) XtDestroyWidget(regData->dialog);
+   if (regData) {
+      XtDestroyWidget(regData->dialog);
+      assert (ledger == regData->ledger);
+      regData->ledger = NULL;
+   }
 }
 
 /********************************************************************\
@@ -716,11 +742,12 @@ static void
 closeRegWindow( Widget mw, XtPointer cd, XtPointer cb )
 {
   RegWindow *regData = (RegWindow *)cd;
+  ENTER("closeRegWindow\n");
 
   xaccLedgerDisplayClose (regData->ledger);
 
   free(regData);
-  DEBUG("closed RegWindow\n");
+  LEAVE("closeRegWindow\n");
 }
 
 /********************************************************************\
@@ -749,7 +776,7 @@ startAdjBCB( Widget mw, XtPointer cd, XtPointer cb )
     if (1 != ledger->numAcc) return;
     acc = ledger->displayed_accounts[0];
   }
-  adjBWindow( toplevel, acc );
+  adjBWindow(acc);
 }
 
 /********************************************************************\
@@ -778,7 +805,62 @@ startRecnCB( Widget mw, XtPointer cd, XtPointer cb )
     if (1 != ledger->numAcc) return;
     acc = ledger->displayed_accounts[0];
   }
-  recnWindow( toplevel, acc );
+  recnWindow( gnc_get_ui_data(), acc );
+}
+
+/********************************************************************\
+ * scrubCB -- clean up hanging double entries                       *
+ *                                                                  *
+ * Args:   mw - the widget that called us                           *
+ *         cd - regData - the data struct for this register         *
+ *         cb -                                                     *
+ * Return: none                                                     *
+\********************************************************************/
+static void 
+scrubCB( Widget mw, XtPointer cd, XtPointer cb )
+{
+  RegWindow *regData = (RegWindow *)cd;
+  xaccLedgerDisplay *ledger = regData->ledger;
+  Account *acc;
+  
+  /* Must have number of accounts be one.  If not one,
+   * then this callback should never have been called,
+   * since the menu entry is supposed to be greyed out.
+   */
+  if (ledger->leader) {
+    acc = ledger->leader;
+  } else {
+    if (1 != ledger->numAcc) return;
+    acc = ledger->displayed_accounts[0];
+  }
+
+  PINFO ("Will scrub %s \n", xaccAccountGetName (acc));
+  xaccAccountTreeScrubOrphans (acc);
+  xaccAccountScrubImbalance (acc);
+
+  /* redraw this window, and any other windows displaying this account.
+   * hack alert -- potentially lots of accounts are affected, and
+   * they should get redrawn as well.  Flip side, its possible
+   * that nothing changed, so we can skip the redraw.  Either way,
+   * this refresh code is not quite right, but will have to do for now. 
+   */
+  xaccAccountDisplayRefresh (acc);
+  gnc_refresh_main_window ();
+}
+
+/********************************************************************\
+ * transferCB -- transfer from one account to another               *
+ *                                                                  *
+ * Args:   mw - the widget that called us                           *
+ *         cd - regData - the data struct for this register         *
+ *         cb -                                                     *
+ * Return: none                                                     *
+\********************************************************************/
+static void 
+transferCB( Widget mw, XtPointer cd, XtPointer cb )
+{
+  AccountGroup *grp = gncGetCurrentGroup(); 
+  xferWindow(gnc_get_ui_data(), grp);
 }
 
 /********************************************************************\
@@ -795,21 +877,14 @@ recordCB( Widget mw, XtPointer cd, XtPointer cb )
   RegWindow *regData = (RegWindow *)cd;
   xaccLedgerDisplay *ld = regData->ledger;
   SplitRegister *ledger = ld->ledger;
-  Transaction *trans=NULL;
-  
-  xaccSRSaveRegEntry (ledger);
 
-  /* Get the currently open transaction, commit it ..
-   * and then repaint everything.  
-   */
-  trans = (Transaction *) (ledger->user_huck);
-  if (trans) {
-     xaccTransCommitEdit (trans);
-     ledger->user_huck = NULL;
-  }
+  ENTER ("RegWindow.c: recordCB\n");
+  
+  xaccSRSaveRegEntry (ledger, NULL);
   xaccSRRedrawRegEntry (ledger);
-  xaccTransDisplayRefresh (trans);
-  refreshMainWindow ();
+  gnc_refresh_main_window ();
+
+  LEAVE ("RegWindow.c: recordCB\n");
 }
 
 /********************************************************************\
@@ -825,38 +900,22 @@ static void
 deleteCB( Widget mw, XtPointer cd, XtPointer cb )
 {
   RegWindow *regData = (RegWindow *)cd;
-  Split * split;
+  Split *split;
   Transaction *trans;
   char buf[BUFSIZE];
-  int i, num_splits;
-  Account **affected_accounts;
   
   /* get the current split based on cursor position */
   split = xaccSRGetCurrentSplit (regData->ledger->ledger);
-  if (NULL == split ) return;
+  if (NULL == split) return;
 
-  /* ask for user confirmation before performing 
-   * permanent damage */
+  /* Ask for user confirmation before performing permanent damage */
   trans = xaccSplitGetParent (split);
-  sprintf (buf, TRANS_DEL_MSG, xaccSplitGetMemo (split), xaccTransGetDescription (trans));
-  if (!verifyBox (toplevel, buf)) return;
+  sprintf (buf, TRANS_DEL_MSG, xaccSplitGetMemo (split),
+           xaccTransGetDescription (trans));
 
-  /* make a copy of all of the accounts that will be  
-   * affected by this deletion, so that we can update
-   * thier register windows after the deletion.
-   */
-  num_splits = xaccTransCountSplits (trans);
-  affected_accounts = (Account **) malloc ((num_splits+1) * sizeof (Account *));
-  for (i=0; i<num_splits; i++) {
-    split = xaccTransGetSplit (trans, i);
-    affected_accounts[i] = xaccSplitGetAccount (split);
-  }
-  affected_accounts[num_splits] = NULL;
+  if (!gnc_verify_dialog (buf, GNC_F)) return;
 
-  xaccSplitDestroy (split);
-  xaccAccListDisplayRefresh (affected_accounts);
-  free (affected_accounts);
-  refreshMainWindow ();
+  xaccSRDeleteCurrentSplit(regData->ledger->ledger);
 }
 
 /********************************************************************\
@@ -872,59 +931,15 @@ static void
 cancelCB( Widget mw, XtPointer cd, XtPointer cb )
 {
    RegWindow *regData = (RegWindow *)cd;
-   xaccLedgerDisplay *ld = regData->ledger;
-   SplitRegister *ledger = ld->ledger;
-   Transaction *trans=NULL;
-   Split * split;
-   int i, num_splits=0, more_splits=0;
-   Account **affected_accounts=NULL;
 
 #ifdef CANCEL_ONLY_ONE_SPLIT_NOT_ENTIRE_TRANS
-  /* when cancelling edits, reload the cursor from the transaction */
-  split = xaccSRGetCurrentSplit (ledger);
-  xaccSRLoadRegEntry (ledger, split);
-  xaccRefreshTableGUI (ledger->table);
+  xaccSRCancelCursorSplitChanges(regData->ledger->ledger);
 #endif /* CANCEL_ONLY_ONE_SPLIT_NOT_ENTIRE_TRANS */
 
 #define CANCEL_THE_WHOLE_ENCHILADA
 #ifdef CANCEL_THE_WHOLE_ENCHILADA
-  /* Get the currently open transaction, rollback the edits on it,
-   * and then repaint everything.  To repaint everything, 
-   * make a note of all of the accounts that will be  
-   * affected by this rollback.  Geez, there must be some easier
-   * way of doing redraw notification ... 
-   */
-  trans = (Transaction *) (ledger->user_huck);
-  if (trans) {
-     num_splits = xaccTransCountSplits (trans);
-     affected_accounts = (Account **) malloc ((num_splits+1) * sizeof (Account *));
-     for (i=0; i<num_splits; i++) {
-       split = xaccTransGetSplit (trans, i);
-       affected_accounts[i] = xaccSplitGetAccount (split);
-     }
-     affected_accounts[num_splits] = NULL;
-   
-     /* rollback ... */
-     xaccTransRollbackEdit (trans);
-     
-     /* and do some more redraw, for the new set of accounts .. */
-     more_splits = xaccTransCountSplits (trans);
-     affected_accounts = (Account **) realloc (affected_accounts, 
-                              (more_splits+num_splits+1) * sizeof (Account *));
-     for (i=0; i<more_splits; i++) {
-       split = xaccTransGetSplit (trans, i);
-       affected_accounts[i+num_splits] = xaccSplitGetAccount (split);
-     }
-     affected_accounts[num_splits+more_splits] = NULL;
-   
-     xaccAccListDisplayRefresh (affected_accounts);
-     free (affected_accounts);
-   
-     ledger->user_huck = NULL;
-  }
+  xaccSRCancelCursorTransChanges (regData->ledger->ledger);
 #endif /* CANCEL_THE_WHOLE_ENCHILADA */
-
-  refreshMainWindow ();
 }
 
 /********************************************************************\
@@ -934,7 +949,7 @@ static void
 reportCB( Widget mw, XtPointer cd, XtPointer cb )
 {
   RegWindow *regData = (RegWindow *)cd;
-  char *outfile = fileBox( toplevel, OPEN_STR, "*.html" );  
+  char *outfile = fileBox( OPEN_STR, "*.html" );  
 
   if (!outfile) return;
   xaccTablePrintHTML (regData->ledger->ledger->table, outfile);
@@ -1124,8 +1139,19 @@ dateChangeCB( Widget mw, XtPointer cd, XtPointer cb)
   strcat (newval, &oldval[((tvcbs->endPos)-adj_len)]);
 
   /* let the cell handler convert it into a valid date */
-  retval = (char *) 
-     ((*handler->cell.modify_verify) (&(handler->cell), oldval, change, newval));
+  {
+    /* These are unuses at present. */
+    int cursor_position = 0;
+    int start_sel;
+    int end_sel;
+
+    retval = (char *) 
+      ((*handler->cell.modify_verify) (&(handler->cell),
+                                       oldval, change, newval,
+                                       &cursor_position,
+                                       &start_sel, &end_sel));
+  }
+  
   retvaltoo = (char *) ((*handler->cell.leave_cell) (&(handler->cell), retval));
   if (retval != newval) free (retval);
   tvcbs->doit = False;
@@ -1142,10 +1168,12 @@ dateChangeCB( Widget mw, XtPointer cd, XtPointer cb)
   free (retvaltoo);
 
   /* OK, now use these new values to control the displayed splits */
+  /* hack alert -- this breaks for dates earlier than Dec 1901 or 
+   * later than Jan 2038 */
   early_time = mktime (&(regData->early_date_handler->date));
   late_time  = mktime (&(regData-> late_date_handler->date));
 
-  xaccQuerySetDateRange (ld->query, early_time, late_time);
+  xaccQuerySetDateRangeL (ld->query, early_time, late_time);
 
   /* don't let max-number of splits to display to interfere with the date 
    * fiddling; so unclamp it (ok, clamp at 1000 which should be acceptable!?!)
