@@ -2,7 +2,7 @@
  * XferWindow.c -- the transfer window for xacc (X-Accountant)      *
  *                     (for transferring between accounts)          *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997 Linas Vepstas                                 *
+ * Copyright (C) 1997, 1998 Linas Vepstas                           *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -67,10 +67,11 @@ typedef struct _xferwindow
   int from;                        /* the chosen to/from accounts   */
   int numMenuData;
   MenuData **menuData;
+  AccountGroup *topgrp;
 } XferWindow;
 
-/** GLOBALS *********************************************************/
-extern Widget  toplevel;
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_GUI;
 
 /** PROTOTYPES ******************************************************/
 void closeXferWindow( Widget mw, XtPointer cd, XtPointer cb );
@@ -83,10 +84,9 @@ void xferCB( Widget mw, XtPointer cd, XtPointer cb );
  *                                                                  * 
  * Args:   parent   - the parent of the window to be created        * 
  * Return: none                                                     *
- * Global: topgroup - the accounts, and stuff...                    *
 \********************************************************************/
 void 
-xferWindow( Widget parent )
+xferWindow (Widget parent, AccountGroup *grp)
   {
   char       buf[MAX_DATE_LENGTH];
   Widget     dialog, form, widget, label, buttonform;
@@ -94,22 +94,22 @@ xferWindow( Widget parent )
   XferWindow *xferData;
   int        position,i;
   int        initial = 0;
-  AccountGroup *grp = topgroup;  /* hack alert -- should be pased as arg */
   int        nacc;
   
   nacc = xaccGroupGetNumAccounts (grp);
   if (1 >= nacc) {
-    errorBox (toplevel, XFER_NSF_MSG);
+    gnc_error_dialog (XFER_NSF_MSG);
     return;
   }
 
-  setBusyCursor( parent );
+  gnc_set_busy_cursor( parent );
   
   xferData = (XferWindow *)_malloc(sizeof(XferWindow));
   xferData->to   = initial;
   xferData->from = initial;
   xferData->menuData    = NULL;
   xferData->numMenuData = 0;
+  xferData->topgrp = grp;
   
   /* force the size of the dialog so it is not resizable */
   dialog = XtVaCreatePopupShell( "dialog", 
@@ -390,7 +390,7 @@ xferWindow( Widget parent )
   
   XtPopup( dialog, XtGrabNone );
   
-  unsetBusyCursor( parent );
+  gnc_unset_busy_cursor( parent );
   }
 
 /********************************************************************\
@@ -445,7 +445,6 @@ menuCB( Widget mw, XtPointer cd, XtPointer cb )
  *              the XfewWindow                                      * 
  *         cb -                                                     * 
  * Return: none                                                     * 
- * Global: topgroup        - the topgroup of accounts               *
 \********************************************************************/
 void
 xferCB( Widget mw, XtPointer cd, XtPointer cb )
@@ -456,11 +455,11 @@ xferCB( Widget mw, XtPointer cd, XtPointer cb )
   Account *acc;
   String  str;
   float   val=0.0;
-  AccountGroup *grp = topgroup; /* hack alert -- should pass as arg */
+  AccountGroup *grp = xferData->topgrp;
 
   /* silently reject transfers into-out-of the same account */
   if (xferData->from == xferData->to) {
-    errorBox (toplevel, XFER_DIFF_MSG);
+    gnc_error_dialog (XFER_DIFF_MSG);
     return;
   }
 
@@ -477,7 +476,7 @@ xferCB( Widget mw, XtPointer cd, XtPointer cb )
   /* TransSetMemo will set the memo for both splits */
   xaccTransSetMemo (trans, XmTextGetString(xferData->memo));
   
-  dest_split = xaccTransGetSplit (trans, 1);
+  dest_split = xaccTransGetSplit (trans, 0);
   str = XmTextGetString(xferData->amount);
   sscanf( str, "%f", &val );  /* sscanf must take float not double arg */
   xaccSplitSetShareAmount (dest_split, -val);
@@ -491,15 +490,18 @@ xferCB( Widget mw, XtPointer cd, XtPointer cb )
   recnRefresh(acc);
 
   acc = xaccGroupGetAccount(grp, xferData->to);
-  src_split = xaccTransGetSplit (trans, 0);
+  src_split = xaccMallocSplit ();
+  xaccSplitSetShareAmount (src_split, val);
+  xaccTransAppendSplit (trans, src_split); 
+
   xaccAccountBeginEdit (acc, 0);
   xaccAccountInsertSplit (acc, src_split);
-  xaccTransCommitEdit (trans);
   xaccAccountCommitEdit (acc);
+  xaccTransCommitEdit (trans);
   xaccAccountDisplayRefresh (acc);
   recnRefresh(acc);
 
-  refreshMainWindow();
+  gnc_refresh_main_window();
 
   /* now close xfer window */
   XtDestroyWidget(xferData->dialog);
