@@ -3,13 +3,14 @@
 
 #include <qmenubar.h>
 #include <qlist.h>
+#include <qlistview.h>
 #include <qpopmenu.h>
 
 #include <kfm.h>
 #include <kapp.h>
 #include <kurl.h>
 #include <klocale.h>
-#include <ktablistbox.h>
+#include <ktopwidget.h>
 
 #define ID_FILE_OPEN 1
 #define ID_FILE_SAVE 3
@@ -34,6 +35,7 @@
 #define ID_HELP_LICENSE 103
 
 extern "C" {
+#include "Group.h"
 //  #include <Account.h>
 //  #include <FileIO.h>
 //  #include <Data.h>
@@ -46,19 +48,14 @@ public:
 
   AccWidget();
   ~AccWidget();
-  
-private:
-    KMenuBar *menu;
-    KTabListBox *acctList; 
-    
+
+public slots:
+
+  void refresh();
+ 
 protected:
     void initMenu();
     
-    void doFileOpen ();
-    void doFileSave ();
-    void doFileSaveAs ();
-    void doFileNew ();
-    void doFileImport ();
     void doEditAcct (); 
     void doNewAcct ();
     void doOpenAcct ();
@@ -70,6 +67,13 @@ protected:
 protected slots:
     void selectAccount (int idx);
     void menuCallback (int item);
+
+private:
+    void insertGroup(AccountGroup *grp, QListViewItem *parent = 0);
+    
+    KMenuBar   *menu;
+    KStatusBar *statusbar;
+    QListView  *acctList; 
 };
 
 #endif
