@@ -1,19 +1,3 @@
-/*
- * FILE:
- * splitreg.c
- *
- * FUNCTION:
- * Implements the register object.
- * Specifies the physical layout of the register cells.
- * See the header file for additional documentation.
- *
- * hack alert -- most of the code in this file should be 
- * replaced by a guile/scheme based config file.
- *
- * HISTORY:
- * Copyright (c) 1998 Linas Vepstas
- */
-
 /********************************************************************\
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -30,6 +14,22 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 \********************************************************************/
 
+/*
+ * FILE:
+ * splitreg.c
+ *
+ * FUNCTION:
+ * Implements the register object.
+ * Specifies the physical layout of the register cells.
+ * See the header file for additional documentation.
+ *
+ * hack alert -- most of the code in this file should be 
+ * replaced by a guile/scheme based config file.
+ *
+ * HISTORY:
+ * Copyright (c) 1998, 1999, 2000 Linas Vepstas
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -38,6 +38,59 @@
 #include "splitreg.h"
 #include "table-allgui.h"
 #include "textcell.h"
+#include "util.h"
+
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_REGISTER;
+
+typedef struct _CellBuffer CellBuffer;
+struct _CellBuffer
+{
+  char * value;
+  unsigned int changed;
+};
+
+struct _SplitRegisterBuffer
+{
+  CellBuffer dateCell;
+  CellBuffer numCell;
+  CellBuffer descCell;
+  CellBuffer recnCell;
+  CellBuffer shrsCell;
+  CellBuffer balanceCell;
+  CellBuffer actionCell;
+  CellBuffer xfrmCell;
+  CellBuffer mxfrmCell;
+  CellBuffer xtoCell;
+  CellBuffer memoCell;
+  CellBuffer creditCell;
+  CellBuffer debitCell;
+  CellBuffer priceCell;
+  CellBuffer valueCell;
+  CellBuffer ncreditCell;
+  CellBuffer ndebitCell;
+};
+
+static SplitRegisterColors reg_colors = {
+  0xffdddd, /* pale red, single cursor active */
+  0xccccff, /* pale blue, single cursor passive */
+  0xccccff, /* pale blue, single cursor passive 2 */
+
+  0xffdddd, /* pale red, double cursor active */
+  0xccccff, /* pale blue, double cursor passive */
+  0xffffff, /* white, double cursor passive 2 */
+
+  GNC_F,    /* double mode alternate by physical row */
+
+  0xffdddd, /* pale red, trans cursor active */
+  0xccccff, /* pale blue, trans cursor passive */
+
+  0xffffdd, /* pale yellow, split cursor active */
+  0xffffff, /* white, split cursor passive */
+
+  0xffffff  /* white, header color */
+};
+
 
 /* utility defines for cell configuration data */
 #define DATE_CELL      0
@@ -46,7 +99,6 @@
 #define RECN_CELL      3   
 #define SHRS_CELL      4
 #define BALN_CELL      5
-
 #define ACTN_CELL      6
 #define XFRM_CELL      7
 #define XTO_CELL       8
@@ -83,6 +135,24 @@
 #define BALN_CELL_WIDTH    12
 
 
+#define DATE_CELL_ALIGN    ALIGN_RIGHT
+#define NUM_CELL_ALIGN     ALIGN_LEFT
+#define ACTN_CELL_ALIGN    ALIGN_LEFT
+#define XFRM_CELL_ALIGN    ALIGN_RIGHT
+#define MXFRM_CELL_ALIGN   ALIGN_RIGHT
+#define XTO_CELL_ALIGN     ALIGN_RIGHT
+#define DESC_CELL_ALIGN    ALIGN_LEFT
+#define MEMO_CELL_ALIGN    ALIGN_LEFT
+#define RECN_CELL_ALIGN    ALIGN_CENTER
+#define DEBT_CELL_ALIGN    ALIGN_RIGHT
+#define CRED_CELL_ALIGN    ALIGN_RIGHT
+#define NDEBT_CELL_ALIGN   ALIGN_RIGHT
+#define NCRED_CELL_ALIGN   ALIGN_RIGHT
+#define PRIC_CELL_ALIGN    ALIGN_RIGHT
+#define VALU_CELL_ALIGN    ALIGN_RIGHT
+#define SHRS_CELL_ALIGN    ALIGN_RIGHT
+#define BALN_CELL_ALIGN    ALIGN_RIGHT
+
 /* ============================================== */
 
 #define LABEL(NAME,label)					\
@@ -104,20 +174,19 @@ configLabels (SplitRegister *reg)
 
    LABEL (DATE,  DATE_STR);
    LABEL (NUM,   NUM_STR);
-   LABEL (ACTN,  NUM_STR);
+   LABEL (DESC,  DESC_STR);
+   LABEL (RECN,  "R");
+   LABEL (SHRS,  TOT_SHRS_STR);
+   LABEL (BALN,  BALN_STR);
+   LABEL (ACTN,  ACTION_STR);
    LABEL (XFRM,  XFRM_STR);
    LABEL (MXFRM, XFRM_STR);
    LABEL (XTO,   XFTO_STR);
-   LABEL (DESC,  DESC_STR);
-   LABEL (MEMO,  DESC_STR);
-   LABEL (RECN,  "R");
-   LABEL (DEBT,  DEBIT_STR);
+   LABEL (MEMO,  MEMO_STR);
    LABEL (CRED,  CREDIT_STR);
-
+   LABEL (DEBT,  DEBIT_STR);
    LABEL (PRIC,  PRICE_STR);
    LABEL (VALU,  VALUE_STR);
-   LABEL (SHRS,  TOT_SHRS_STR);
-   LABEL (BALN,  BALN_STR);
 
 
    /* setup custom labels for the debit/credit columns */
@@ -196,6 +265,9 @@ configAction (SplitRegister *reg)
          xaccAddComboCellMenuItem ( reg->actionCell, ONLINE_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, ACH_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, WIRE_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, CREDIT_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, DIRECTDEBIT_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, TRANSFER_STR);
          break;
       case CASH_REGISTER:
          xaccAddComboCellMenuItem ( reg->actionCell, BUY_STR);
@@ -204,6 +276,7 @@ configAction (SplitRegister *reg)
       case ASSET_REGISTER:
          xaccAddComboCellMenuItem ( reg->actionCell, BUY_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, SELL_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, FEE_STR);
          break;
       case CREDIT_REGISTER:
          xaccAddComboCellMenuItem ( reg->actionCell, BUY_STR);
@@ -240,10 +313,12 @@ configAction (SplitRegister *reg)
          xaccAddComboCellMenuItem ( reg->actionCell, BUY_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, SELL_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, PRICE_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, FEE_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, DIV_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, INT_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, LTCG_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, STCG_STR);
+         xaccAddComboCellMenuItem ( reg->actionCell, INCOME_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, DIST_STR);
          xaccAddComboCellMenuItem ( reg->actionCell, SPLIT_STR);
          break;
@@ -263,17 +338,19 @@ configAction (SplitRegister *reg)
 								\
    if ((0<=row) && (0<=col)) {					\
       curs->cells [row][col] = (handler);			\
+      curs->cell_types[row][col] = NAME##_CELL;                 \
       header->widths[col] = NAME##_CELL_WIDTH;			\
+      header->alignments[col] = NAME##_CELL_ALIGN;		\
+      curs->widths[col] = NAME##_CELL_WIDTH;			\
+      curs->alignments[col] = NAME##_CELL_ALIGN;		\
       if (hcell) {						\
-         if (1 == reg->num_header_rows) {			\
-            header->cells[0][col] = hcell;			\
-         } else {						\
+         if (row < reg->num_header_rows) {			\
             header->cells[row][col] = hcell;			\
          }							\
       }								\
    }								\
 }
-   
+
 /* BASIC & FANCY macros initialize cells in the register */
 
 #define BASIC(NAME,CN,col,row) {			\
@@ -320,44 +397,46 @@ configLayout (SplitRegister *reg)
       case INCOME_LEDGER:     /* hack alert do xto cell too */
       case GENERAL_LEDGER:    /* hack alert do xto cell too */
       {
+         curs = reg->double_cursor;
+         FANCY (DATE,   date,     0,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (MXFRM,  mxfrm,    3,  0);
+         BASIC (RECN,   recn,     4,  0);
+         FANCY (DEBT,   debit,    5,  0);
+         FANCY (CRED,   credit,   6,  0);
+         FANCY (BALN,   balance,  7,  0);
+
+         FANCY (ACTN,   action,   1,  1);
+         FANCY (MEMO,   memo,     2,  1);
+
+         curs = reg->trans_cursor;
+         FANCY (DATE,   date,     0,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         BASIC (RECN,   recn,     4,  0);
+         FANCY (DEBT,   debit,    5,  0);
+         FANCY (CRED,   credit,   6,  0);
+         FANCY (BALN,   balance,  7,  0);
+
+         curs = reg->split_cursor;
+         FANCY (ACTN,   action,   1,  0);
+         FANCY (MEMO,   memo,     2,  0);
+         FANCY (XFRM,   xfrm,     3,  0);
+         FANCY (NDEBT,  ndebit,   5,  0);
+         FANCY (NCRED,  ncredit,  6,  0);
 
          /* basic common 8 column config */
          curs = reg->single_cursor;
          FANCY (DATE,   date,     0,  0);
-         BASIC (NUM,    num,      1,  0);
-         FANCY (MXFRM,  mxfrm,    2,  0);
-         FANCY (DESC,   desc,     3,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (MXFRM,  mxfrm,    3,  0);
          BASIC (RECN,   recn,     4,  0);
          FANCY (DEBT,   debit,    5,  0);
          FANCY (CRED,   credit,   6,  0);
          FANCY (BALN,   balance,  7,  0);
 
-         curs = reg->double_cursor;
-         FANCY (DATE,   date,     0,  0);
-         BASIC (NUM,    num,      1,  0);
-         FANCY (DESC,   desc,     3,  0);
-         BASIC (RECN,   recn,     4,  0);
-         FANCY (DEBT,   debit,    5,  0);
-         FANCY (CRED,   credit,   6,  0);
-         FANCY (BALN,   balance,  7,  0);
-  
-         FANCY (ACTN,   action,   1,  1);
-         FANCY (MXFRM,  mxfrm,    2,  1);
-         BASIC (MEMO,   memo,     3,  1);
-
-         curs = reg->trans_cursor;
-         FANCY (DATE,   date,     0,  0);
-         BASIC (NUM,    num,      1,  0);
-         FANCY (DESC,   desc,     3,  0);
-         BASIC (RECN,   recn,     4,  0);
-         FANCY (BALN,   balance,  7,  0);
-     
-         curs = reg->split_cursor;
-         FANCY (ACTN,   action,   1,  0);
-         FANCY (XFRM,   xfrm,     2,  0);
-         BASIC (MEMO,   memo,     3,  0);
-         FANCY (NDEBT,  ndebit,   5,  0);
-         FANCY (NCRED,  ncredit,  6,  0);
          break;
       }
 
@@ -366,25 +445,12 @@ configLayout (SplitRegister *reg)
       case PORTFOLIO:
       case CURRENCY_REGISTER:
       {
-         /* 11 column config */
-         curs = reg->single_cursor;
-         FANCY (DATE,   date,     0,  0);
-         BASIC (NUM,    num,      1,  0);
-         FANCY (MXFRM,  mxfrm,    2,  0);
-         FANCY (DESC,   desc,     3,  0);
-         BASIC (RECN,   recn,     4,  0);
-         FANCY (DEBT,   debit,    5,  0);
-         FANCY (CRED,   credit,   6,  0);
-         FANCY (PRIC,   price,    7,  0);
-         FANCY (VALU,   value,    8,  0);
-         FANCY (SHRS,   shrs,     9,  0);
-         FANCY (BALN,   balance,  10, 0);
-
          /* prep the second row of the double style */
          curs = reg->double_cursor;
          FANCY (DATE,   date,     0,  0);
-         BASIC (NUM,    num,      1,  0);
-         FANCY (DESC,   desc,     3,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (MXFRM,  mxfrm,    3,  0);
          BASIC (RECN,   recn,     4,  0);
          FANCY (DEBT,   debit,    5,  0);
          FANCY (CRED,   credit,   6,  0);
@@ -394,14 +460,13 @@ configLayout (SplitRegister *reg)
          FANCY (BALN,   balance,  10, 0);
 
          FANCY (ACTN,   action,   1,  1);
-         FANCY (MXFRM,  mxfrm,    2,  1);
-         BASIC (MEMO,   memo,     3,  1);
+         FANCY (MEMO,   memo,     2,  1);
 
          /* only the transaction cursor gets used */
          curs = reg->trans_cursor;
          FANCY (DATE,   date,     0,  0);
-         BASIC (NUM,    num,      1,  0);
-         FANCY (DESC,   desc,     3,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
          BASIC (RECN,   recn,     4,  0);
          FANCY (DEBT,   debit,    5,  0);
          FANCY (CRED,   credit,   6,  0);
@@ -412,15 +477,30 @@ configLayout (SplitRegister *reg)
     
          curs = reg->split_cursor;
          FANCY (ACTN,   action,   1,  0);
-         FANCY (XFRM,   xfrm,     2,  0);
-         BASIC (MEMO,   memo,     3,  0);
+         FANCY (MEMO,   memo,     2,  0);
+         FANCY (XFRM,   xfrm,     3,  0);
          FANCY (NDEBT,  ndebit,   5,  0);
          FANCY (NCRED,  ncredit,  6,  0);
+
+         /* 11 column config */
+         curs = reg->single_cursor;
+         FANCY (DATE,   date,     0,  0);
+         FANCY (NUM,    num,      1,  0);
+         FANCY (DESC,   desc,     2,  0);
+         FANCY (MXFRM,  mxfrm,    3,  0);
+         BASIC (RECN,   recn,     4,  0);
+         FANCY (DEBT,   debit,    5,  0);
+         FANCY (CRED,   credit,   6,  0);
+         FANCY (PRIC,   price,    7,  0);
+         FANCY (VALU,   value,    8,  0);
+         FANCY (SHRS,   shrs,     9,  0);
+         FANCY (BALN,   balance,  10, 0);
+
          break;
       }
       /* --------------------------------------------------------- */
       default:
-         printf ("Internal Error: configLayout(): "
+         PERR ("configLayout(): "
            "unknown register type %d \n", type);
          break;
    }
@@ -433,15 +513,20 @@ configLayout (SplitRegister *reg)
 /* hack alert -- if show_tamount or show_samount is set then don't traverse there */
 /* hack alert -- fix show_txfrm also ... */
 
+
+/* Right Traversals */
+
 #define FIRST_RIGHT(r,c) {				\
    prev_r = r; prev_c = c;				\
 }
 
-#define NEXT_RIGHT(r,c) {				\
+
+#define NEXT_RIGHT(r,c) {			        \
    xaccNextRight (curs, prev_r, prev_c, (r), (c));	\
    prev_r = r; prev_c = c;				\
 }
 
+
 #define TRAVERSE_NON_NULL_CELLS() {			\
    i = prev_r;						\
    for (j=prev_c+1; j<curs->numCols; j++) {		\
@@ -464,6 +549,7 @@ configLayout (SplitRegister *reg)
    }							\
 }
 
+
 #define FIRST_NON_NULL(r,c) {				\
    i = r;						\
    for (j=c; j<curs->numCols; j++) {			\
@@ -477,6 +563,7 @@ configLayout (SplitRegister *reg)
    }							\
 }
 
+
 #define NEXT_NON_NULL(r,c) {				\
    i = r;						\
    for (j=c+1; j<curs->numCols; j++) {			\
@@ -490,6 +577,11 @@ configLayout (SplitRegister *reg)
    }							\
 }
 
+#define EXIT_RIGHT() {                                  \
+  curs->right_exit_r = prev_r;     curs->right_exit_c = prev_c;  \
+}
+
+
 #define NEXT_SPLIT() {					\
    i = 0;						\
    for (j=0; j<reg->split_cursor->numCols; j++) {	\
@@ -503,6 +595,83 @@ configLayout (SplitRegister *reg)
    }							\
 }
 
+
+/* Left Traversals */
+
+#define LAST_LEFT(r,c) {				\
+   prev_r = r; prev_c = c;				\
+}
+
+#define NEXT_LEFT(r,c) {				\
+   xaccNextLeft (curs, prev_r, prev_c, (r), (c));	\
+   prev_r = r; prev_c = c;				\
+}
+
+#define TRAVERSE_NON_NULL_CELLS_LEFT() {		\
+   i = prev_r;						\
+   for (j=prev_c -1; j>=0; j--) {		        \
+      if ((reg->nullCell != curs->cells[i][j]) &&	\
+          (reg->recnCell != curs->cells[i][j]) &&	\
+          (XACC_CELL_ALLOW_INPUT & curs->cells[i][j]->input_output)) \
+      {	                                                \
+         NEXT_LEFT  (i, j);				\
+      }							\
+   }							\
+   for (i=prev_r-1; i>=0; i--) {	                \
+      for (j=curs->numCols-1; j>=0; j--) {		\
+         if ((reg->nullCell != curs->cells[i][j]) &&	\
+             (reg->recnCell != curs->cells[i][j]) &&	\
+             (XACC_CELL_ALLOW_INPUT & curs->cells[i][j]->input_output)) \
+         {					        \
+            NEXT_LEFT  (i, j);				\
+         }						\
+      }							\
+   }							\
+}
+
+#define LAST_NON_NULL(r,c) {				\
+   i = r;						\
+   for (j=c; j>=0; j--) {			        \
+      if ((reg->nullCell != curs->cells[i][j]) &&	\
+          (reg->recnCell != curs->cells[i][j]) &&	\
+          (XACC_CELL_ALLOW_INPUT & curs->cells[i][j]->input_output)) \
+      {							\
+         LAST_LEFT  (i, j);				\
+         break;						\
+      }							\
+   }							\
+}
+
+#define PREVIOUS_NON_NULL(r,c) {			\
+   i = r;						\
+   for (j=c-1; j>=0; j--) {			        \
+      if ((reg->nullCell != curs->cells[i][j]) &&	\
+          (reg->recnCell != curs->cells[i][j]) &&	\
+          (XACC_CELL_ALLOW_INPUT & curs->cells[i][j]->input_output)) \
+      {							\
+         NEXT_LEFT  (i, j);				\
+         break;						\
+      }							\
+   }							\
+}
+
+#define EXIT_LEFT() {                                   \
+  curs->left_exit_r = prev_r;  curs->left_exit_c = prev_c;      \
+}
+
+#define PREVIOUS_SPLIT() {					\
+   i = reg->split_cursor->numRows-1;				\
+   for (j=reg->split_cursor->numCols-1; j>=0; j--) {	        \
+      if ((reg->nullCell != reg->split_cursor->cells[i][j]) &&	\
+          (reg->recnCell != reg->split_cursor->cells[i][j]) &&	\
+          (XACC_CELL_ALLOW_INPUT & reg->split_cursor->cells[i][j]->input_output)) \
+      {							\
+         NEXT_LEFT  (i-1, j);				\
+         break;						\
+      }							\
+   }							\
+}
+
 static void
 configTraverse (SplitRegister *reg)
 {
@@ -516,33 +685,68 @@ configTraverse (SplitRegister *reg)
    FIRST_NON_NULL (0, 0);
    first_r = prev_r; first_c = prev_c;
    TRAVERSE_NON_NULL_CELLS ();
+   /* set the exit row, col */
+   EXIT_RIGHT ();
    /* wrap back to start of row after hitting the commit button */
-   NEXT_RIGHT (-1-first_r, -1-first_c);
+   NEXT_RIGHT (first_r, first_c);
+
+   /* left traverses */
+   LAST_NON_NULL (curs->numRows-1, curs->numCols - 1);
+   first_r = prev_r;  first_c = prev_c;
+   TRAVERSE_NON_NULL_CELLS_LEFT();
+   EXIT_LEFT ();
+   NEXT_LEFT (first_r, first_c);
 
    curs = reg->double_cursor;
    /* lead in with the date cell, return to the date cell */
    FIRST_NON_NULL (0, 0);
    first_r = prev_r; first_c = prev_c;
    TRAVERSE_NON_NULL_CELLS ();
+   /* set the exit row,col */
+   EXIT_RIGHT ();
    /* for double-line, hop back one row */
-   NEXT_RIGHT (-1-first_r, -1-first_c);
+   NEXT_RIGHT (first_r, first_c);
+
+   /* left traverses */
+   LAST_NON_NULL (curs->numRows-1, curs->numCols - 1);
+   first_r = prev_r;  first_c = prev_c;
+   TRAVERSE_NON_NULL_CELLS_LEFT ();
+   EXIT_LEFT ();
+   NEXT_LEFT (first_r, first_c);
 
    curs = reg->trans_cursor;
    FIRST_NON_NULL (0,0);
    TRAVERSE_NON_NULL_CELLS ();
+   /* set the exit row, col */
+   EXIT_RIGHT ();
    /* hop to start of next row (the split cursor) */
    NEXT_SPLIT();
 
+   /* left_traverses */
+   LAST_NON_NULL (curs->numRows-1, curs->numCols - 1);
+   TRAVERSE_NON_NULL_CELLS_LEFT ();
+   EXIT_LEFT ();
+   PREVIOUS_SPLIT ();
+
    curs = reg->split_cursor;
    FIRST_NON_NULL (0,0);
    TRAVERSE_NON_NULL_CELLS ();
+   /* set the exit row, col */
+   EXIT_RIGHT ();
    /* hop to start of next row (the split cursor) */
    NEXT_SPLIT();
+
+   /* left_traverses */
+   LAST_NON_NULL (curs->numRows-1, curs->numCols - 1);
+   TRAVERSE_NON_NULL_CELLS_LEFT ();
+   EXIT_LEFT ();
+   PREVIOUS_SPLIT ();
 }
 
 /* ============================================== */
 
-SplitRegister * xaccMallocSplitRegister (int type)
+SplitRegister *
+xaccMallocSplitRegister (int type)
 {
    SplitRegister * reg;
    reg = (SplitRegister *) malloc (sizeof (SplitRegister));
@@ -552,39 +756,83 @@ SplitRegister * xaccMallocSplitRegister (int type)
 
 /* ============================================== */
 
+void
+xaccSetSplitRegisterColors (SplitRegisterColors reg_colors_new)
+{
+  reg_colors = reg_colors_new;
+}
+
+/* ============================================== */
+
+static void
+configTable(SplitRegister *reg)
+{
+  int style = reg->type & REG_STYLE_MASK;
+
+  if ((reg == NULL) || (reg->table == NULL))
+    return;
+
+  switch (style) {
+    case REG_SINGLE_LINE:
+    case REG_SINGLE_DYNAMIC:
+      reg->table->alternate_bg_colors = GNC_T;
+      break;
+    case REG_DOUBLE_LINE:
+    case REG_DOUBLE_DYNAMIC:
+      reg->table->alternate_bg_colors = reg_colors.double_alternate_virt;
+      break;
+    default:
+      reg->table->alternate_bg_colors = GNC_F;
+      break;
+  }
+}
+
+/* ============================================== */
+
+void
+xaccSplitRegisterConfigColors (SplitRegister *reg)
+{
+   reg->single_cursor->active_bg_color =
+     reg_colors.single_cursor_active_bg_color;
+   reg->single_cursor->passive_bg_color =
+     reg_colors.single_cursor_passive_bg_color;
+   reg->single_cursor->passive_bg_color2 =
+     reg_colors.single_cursor_passive_bg_color2;
+
+   reg->double_cursor->active_bg_color =
+     reg_colors.double_cursor_active_bg_color;
+   reg->double_cursor->passive_bg_color =
+     reg_colors.double_cursor_passive_bg_color;
+   reg->double_cursor->passive_bg_color2 =
+     reg_colors.double_cursor_passive_bg_color2;
+
+   reg->trans_cursor->active_bg_color =
+     reg_colors.trans_cursor_active_bg_color;
+   reg->trans_cursor->passive_bg_color =
+     reg_colors.trans_cursor_passive_bg_color;
+   reg->trans_cursor->passive_bg_color2 =
+     reg_colors.trans_cursor_passive_bg_color;
+
+   reg->split_cursor->active_bg_color =
+     reg_colors.split_cursor_active_bg_color;
+   reg->split_cursor->passive_bg_color =
+     reg_colors.split_cursor_passive_bg_color;
+   reg->split_cursor->passive_bg_color2 =
+     reg_colors.split_cursor_passive_bg_color;
+
+   reg->header->active_bg_color = reg_colors.header_bg_color;
+   reg->header->passive_bg_color = reg_colors.header_bg_color;
+   reg->header->passive_bg_color2 = reg_colors.header_bg_color;
+
+   configTable(reg);
+}
+
+/* ============================================== */
+
 static void
 configCursors (SplitRegister *reg)
 {
-   /* --------------------------- */
-   /* set the color of the cells in the cursors */
-   /* hack alert -- the actual color should depend on the 
-    * type of register. */
-/* they used top be the following:
-  "*regbank.oddRowBackground:      #aaccff",
-  "*regcash.oddRowBackground:      #ccffcc",
-  "*regasset.oddRowBackground:     #aaffcc",
-  "*regcredit.oddRowBackground:    #ffffaa",
-  "*regliability.oddRowBackground: #ffcccc",
-  "*ledportfolio.oddRowBackground: #ccffff",
-  "*regmutual.oddRowBackground:    #ccffff",
-  "*regincome.oddRowBackground:    #aaccff",
-  "*regexpense.oddRowBackground:   #ffcccc",
-  "*regequity.oddRowBackground:    #ffffaa",
-  "*ledgportfolio.evenRowBackground:grey",
-  "*regmutual.evenRowBackground:   grey",
-*/
-   reg->single_cursor->active_bg_color = 0xffdddd; /* pale red */
-   reg->single_cursor->passive_bg_color = 0xccccff; /* pale blue */
-
-   reg->double_cursor->active_bg_color = 0xffdddd; /* pale red */
-   reg->double_cursor->passive_bg_color = 0xccccff; /* pale blue */
-
-   reg->trans_cursor->active_bg_color = 0xffdddd; /* pale red */
-   reg->trans_cursor->passive_bg_color = 0xccccff; /* pale blue */
-
-   reg->split_cursor->active_bg_color = 0xffffdd; /* pale yellow */
-   reg->split_cursor->passive_bg_color = 0xffffff; /* white */
-
+  xaccSplitRegisterConfigColors(reg);
 }
 
 /* ============================================== */
@@ -650,9 +898,8 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
    CellBlock *header;
    int phys_r, phys_c;
 
-   reg->user_hook = NULL;
-   reg->user_hack = NULL;
-   reg->user_huck = NULL;
+   reg->table = NULL;
+   reg->user_data = NULL;
    reg->destroy = NULL;
    reg->type = type;
 
@@ -679,7 +926,7 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
    HDR (VALU);
    HDR (SHRS);
    HDR (BALN);
-   
+
    HDR (NCRED);
    HDR (NDEBT);
 
@@ -688,7 +935,7 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
 
    NEW (null,    Basic);
    NEW (date,    Date);
-   NEW (num,     Text);
+   NEW (num,     Num);
    NEW (desc,    QuickFill);
    NEW (recn,    Recn);
    NEW (shrs,    Price);
@@ -698,7 +945,7 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
    NEW (mxfrm,   Combo);
    NEW (xto,     Combo);
    NEW (action,  Combo);
-   NEW (memo,    Text);
+   NEW (memo,    QuickFill);
    NEW (credit,  Price);
    NEW (debit,   Price);
    NEW (price,   Price);
@@ -729,9 +976,24 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
     * cells handles this for us.
     */
 
-   reg -> nullCell -> input_output = XACC_CELL_ALLOW_NONE;
+   reg->nullCell->input_output = XACC_CELL_ALLOW_NONE;
    xaccSetBasicCellValue (reg->nullCell, "");
 
+   /* The num cell is the transaction number */
+   xaccSetBasicCellBlankHelp (&reg->numCell->cell, NUM_CELL_HELP);
+
+   /* the xfer cells */
+   xaccSetBasicCellBlankHelp (&reg->mxfrmCell->cell, XFER_CELL_HELP);
+   xaccSetBasicCellBlankHelp (&reg->xfrmCell->cell, XFER_CELL_HELP);
+   xaccComboCellSetIgnoreString (reg->mxfrmCell, SPLIT_STR);
+   xaccComboCellSetIgnoreHelp (reg->mxfrmCell, TOOLTIP_MULTI_SPLIT);
+
+   /* the memo cell */
+   xaccSetBasicCellBlankHelp (&reg->memoCell->cell, MEMO_CELL_HELP);
+
+   /* the desc cell */
+   xaccSetBasicCellBlankHelp (&reg->descCell->cell, DESC_CELL_HELP);
+
    /* balance cell does not accept input; its display only.  */
    /* however, we *do* want it to shadow the true cell contents when 
     * the cursor is repositioned.  Othewise, it will just display 
@@ -740,27 +1002,41 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
    reg->balanceCell->cell.input_output = XACC_CELL_ALLOW_SHADOW;
    reg->shrsCell->cell.input_output = XACC_CELL_ALLOW_SHADOW;
 
-   /* the debit/credit/value cells show blank if value is 0.00 */
-   reg->debitCell->blank_zero = 1;
-   reg->creditCell->blank_zero = 1;
-   reg->valueCell->blank_zero = 1;
-   reg->ndebitCell->blank_zero = 1;
-   reg->ncreditCell->blank_zero = 1;
-
-   /* ok, now make sure the initail value of 0.0 is blanked.
-    * if this is not done, then various oddball situations 
-    * will show the non-blanked values. 
+   /* The reconcile cell should only be entered with the pointer,
+    * and only then when the user clicks directly on the cell.
     */
+   reg->recnCell->input_output |= XACC_CELL_ALLOW_EXACT_ONLY;
+
+   /* Initialize price cells */
    xaccSetPriceCellValue (reg->debitCell, 0.0);
    xaccSetPriceCellValue (reg->creditCell, 0.0);
    xaccSetPriceCellValue (reg->valueCell, 0.0);
    xaccSetPriceCellValue (reg->ndebitCell, 0.0);
    xaccSetPriceCellValue (reg->ncreditCell, 0.0);
 
-   /* use three decimal places to print share-related info.
-    * The format is a printf-style format for a double.  */
-   xaccSetPriceCellFormat (reg->shrsCell, "%.3f");
-   xaccSetPriceCellFormat (reg->priceCell, "%.3f");
+   /* Initialize shares cell */
+   xaccSetPriceCellSharesValue (reg->shrsCell, GNC_T);
+
+   /* The action cell should accept strings not in the list */
+   xaccComboCellSetStrict (reg->actionCell, GNC_F);
+   xaccSetBasicCellBlankHelp (&reg->actionCell->cell, ACTION_CELL_HELP);
+
+   /* number format for share quantities in stock ledgers */
+   switch (type & REG_TYPE_MASK) {
+      case STOCK_REGISTER:
+      case PORTFOLIO:
+      case CURRENCY_REGISTER:
+         xaccSetPriceCellSharesValue (reg->debitCell, GNC_T);
+         xaccSetPriceCellSharesValue (reg->creditCell, GNC_T);
+         xaccSetPriceCellSharesValue (reg->ndebitCell, GNC_T);
+         xaccSetPriceCellSharesValue (reg->ncreditCell, GNC_T);
+
+         xaccSetBasicCellBlankHelp (&reg->priceCell->cell, PRICE_CELL_HELP);
+         xaccSetBasicCellBlankHelp (&reg->valueCell->cell, VALUE_CELL_HELP);
+         break;
+      default:
+         break;
+   }
 
    /* -------------------------------- */   
    /* define how traversal works. This must be done *after* the balance, etc.
@@ -774,6 +1050,7 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
 
    /* -------------------------------- */   
    phys_r = header->numRows;
+   reg->cursor_phys_col = 0;
    reg->cursor_phys_row = phys_r;  /* cursor on first line past header */
    reg->cursor_virt_row = 1;
 
@@ -802,6 +1079,8 @@ xaccInitSplitRegister (SplitRegister *reg, int type)
    xaccMoveCursor (table, header->numRows, 0);
 
    reg->table = table;
+
+   configTable(reg);
 }
 
 /* ============================================== */
@@ -813,12 +1092,14 @@ xaccConfigSplitRegister (SplitRegister *reg, int newtype)
 
    reg->type = newtype;
 
-   /* Make sure that any GU elemnts associated with this reconfig 
+   /* Make sure that any GUI elements associated with this reconfig 
     * are properly initialized.  */
    xaccCreateCursor (reg->table, reg->single_cursor);
    xaccCreateCursor (reg->table, reg->double_cursor);
    xaccCreateCursor (reg->table, reg->trans_cursor);
    xaccCreateCursor (reg->table, reg->split_cursor);
+
+   configTable(reg);
 }
 
 /* ============================================== */
@@ -831,9 +1112,7 @@ xaccDestroySplitRegister (SplitRegister *reg)
       (*(reg->destroy)) (reg);
    }
    reg->destroy = NULL;
-   reg->user_hook = NULL;
-   reg->user_hack = NULL;
-   reg->user_huck = NULL;
+   reg->user_data = NULL;
 
    xaccDestroyTable (reg->table);
    reg->table = NULL;
@@ -850,7 +1129,7 @@ xaccDestroySplitRegister (SplitRegister *reg)
    reg->split_cursor = NULL;
 
    xaccDestroyDateCell      (reg->dateCell);
-   xaccDestroyBasicCell     (reg->numCell);
+   xaccDestroyNumCell       (reg->numCell);
    xaccDestroyQuickFillCell (reg->descCell);
    xaccDestroyBasicCell     (reg->recnCell);
    xaccDestroyPriceCell     (reg->shrsCell);
@@ -860,7 +1139,7 @@ xaccDestroySplitRegister (SplitRegister *reg)
    xaccDestroyComboCell     (reg->xfrmCell);
    xaccDestroyComboCell     (reg->mxfrmCell);
    xaccDestroyComboCell     (reg->xtoCell);
-   xaccDestroyBasicCell     (reg->memoCell);
+   xaccDestroyQuickFillCell (reg->memoCell);
    xaccDestroyPriceCell     (reg->creditCell);
    xaccDestroyPriceCell     (reg->debitCell);
    xaccDestroyPriceCell     (reg->priceCell);
@@ -903,7 +1182,7 @@ xaccSplitRegisterGetChangeFlag (SplitRegister *reg)
 
    /* be careful to use bitwise ands and ors to assemble bit flag */
    changed |= MOD_DATE  & reg->dateCell->cell.changed;
-   changed |= MOD_NUM   & reg->numCell->changed;
+   changed |= MOD_NUM   & reg->numCell->cell.changed;
    changed |= MOD_DESC  & reg->descCell->cell.changed;
    changed |= MOD_RECN  & reg->recnCell->changed;
 
@@ -911,7 +1190,7 @@ xaccSplitRegisterGetChangeFlag (SplitRegister *reg)
    changed |= MOD_XFRM  & reg->xfrmCell->cell.changed;
    changed |= MOD_MXFRM & reg->mxfrmCell->cell.changed;
    changed |= MOD_XTO   & reg->xtoCell->cell.changed; 
-   changed |= MOD_MEMO  & reg->memoCell->changed;
+   changed |= MOD_MEMO  & reg->memoCell->cell.changed;
    changed |= MOD_AMNT  & reg->creditCell->cell.changed;
    changed |= MOD_AMNT  & reg->debitCell->cell.changed;
    changed |= MOD_PRIC  & reg->priceCell->cell.changed;
@@ -923,4 +1202,200 @@ xaccSplitRegisterGetChangeFlag (SplitRegister *reg)
    return changed;
 }
 
+/* ============================================== */
+
+void
+xaccSplitRegisterClearChangeFlag (SplitRegister *reg)
+{
+   reg->dateCell->cell.changed = 0;
+   reg->numCell->cell.changed = 0;
+   reg->descCell->cell.changed = 0;
+   reg->recnCell->changed = 0;
+
+   reg->actionCell->cell.changed = 0;
+   reg->xfrmCell->cell.changed = 0;
+   reg->mxfrmCell->cell.changed = 0;
+   reg->xtoCell->cell.changed = 0;
+   reg->memoCell->cell.changed = 0;
+   reg->creditCell->cell.changed = 0;
+   reg->debitCell->cell.changed = 0;
+   reg->priceCell->cell.changed = 0;
+   reg->valueCell->cell.changed = 0;
+
+   reg->ncreditCell->cell.changed = 0;
+   reg->ndebitCell->cell.changed = 0;
+}
+
+/* ============================================== */
+
+CursorType
+xaccSplitRegisterGetCursorType (SplitRegister *reg)
+{
+  Table *table;
+  CellBlock *cursor;
+
+  assert(reg);
+
+  table = reg->table;
+  if (table == NULL)
+    return CURSOR_NONE;
+
+  cursor = table->current_cursor;
+  if (cursor == NULL)
+    return CURSOR_NONE;
+
+  if ((cursor == reg->single_cursor) ||
+      (cursor == reg->double_cursor) ||
+      (cursor == reg->trans_cursor))
+    return CURSOR_TRANS;
+
+  if (cursor == reg->split_cursor)
+    return CURSOR_SPLIT;
+
+  return CURSOR_NONE;
+}
+
+/* ============================================== */
+
+SplitRegisterBuffer *
+xaccMallocSplitRegisterBuffer ()
+{
+  SplitRegisterBuffer *srb;
+
+  srb = calloc(1, sizeof(SplitRegisterBuffer));
+
+  assert(srb != NULL);
+
+  return srb;
+}
+
+/* ============================================== */
+
+static void
+destroyCellBuffer(CellBuffer *cb)
+{
+  if (cb == NULL)
+    return;
+
+  if (cb->value != NULL)
+    free(cb->value);
+
+  cb->value = NULL;
+}
+
+void
+xaccDestroySplitRegisterBuffer (SplitRegisterBuffer *srb)
+{
+  if (srb == NULL)
+    return;
+
+  destroyCellBuffer(&srb->dateCell);
+  destroyCellBuffer(&srb->numCell);
+  destroyCellBuffer(&srb->descCell);
+  destroyCellBuffer(&srb->recnCell);
+  destroyCellBuffer(&srb->shrsCell);
+  destroyCellBuffer(&srb->balanceCell);
+  destroyCellBuffer(&srb->actionCell);
+  destroyCellBuffer(&srb->xfrmCell);
+  destroyCellBuffer(&srb->mxfrmCell);
+  destroyCellBuffer(&srb->xtoCell);
+  destroyCellBuffer(&srb->memoCell);
+  destroyCellBuffer(&srb->creditCell);
+  destroyCellBuffer(&srb->debitCell);
+  destroyCellBuffer(&srb->priceCell);
+  destroyCellBuffer(&srb->valueCell);
+  destroyCellBuffer(&srb->ncreditCell);
+  destroyCellBuffer(&srb->ndebitCell);
+
+  free(srb);
+}
+
+/* ============================================== */
+
+static void
+saveCell(BasicCell *bcell, CellBuffer *cb)
+{
+  if ((bcell == NULL) || (cb == NULL))
+    return;
+
+  if (cb->value != NULL)
+    free(cb->value);
+
+  cb->value = bcell->value;
+
+  if (cb->value != NULL)
+  {
+    cb->value = strdup(cb->value);
+    assert(cb->value != NULL);
+  }
+
+  cb->changed = bcell->changed;
+}
+
+void
+xaccSplitRegisterSaveCursor(SplitRegister *sr, SplitRegisterBuffer *srb)
+{
+  if ((sr == NULL) || (srb == NULL))
+    return;
+
+  saveCell(&sr->dateCell->cell, &srb->dateCell);
+  saveCell(&sr->numCell->cell, &srb->numCell);
+  saveCell(&sr->descCell->cell, &srb->descCell);
+  saveCell(sr->recnCell, &srb->recnCell);
+  saveCell(&sr->shrsCell->cell, &srb->shrsCell);
+  saveCell(&sr->balanceCell->cell, &srb->balanceCell);
+  saveCell(&sr->actionCell->cell, &srb->actionCell);
+  saveCell(&sr->xfrmCell->cell, &srb->xfrmCell);
+  saveCell(&sr->mxfrmCell->cell, &srb->mxfrmCell);
+  saveCell(&sr->xtoCell->cell, &srb->xtoCell);
+  saveCell(&sr->memoCell->cell, &srb->memoCell);
+  saveCell(&sr->creditCell->cell, &srb->creditCell);
+  saveCell(&sr->debitCell->cell, &srb->debitCell);
+  saveCell(&sr->priceCell->cell, &srb->priceCell);
+  saveCell(&sr->valueCell->cell, &srb->valueCell);
+  saveCell(&sr->ncreditCell->cell, &srb->ncreditCell);
+  saveCell(&sr->ndebitCell->cell, &srb->ndebitCell);
+}
+
+/* ============================================== */
+
+static void
+restoreCellChanged(BasicCell *bcell, CellBuffer *cb)
+{
+  if ((bcell == NULL) || (cb == NULL))
+    return;
+
+  if (cb->changed)
+  {
+    xaccSetBasicCellValue(bcell, cb->value);
+    bcell->changed = cb->changed;
+  }
+}
+
+void
+xaccSplitRegisterRestoreCursorChanged(SplitRegister *sr,
+                                      SplitRegisterBuffer *srb)
+{
+  if ((sr == NULL) || (srb == NULL))
+    return;
+
+  restoreCellChanged(&sr->dateCell->cell, &srb->dateCell);
+  restoreCellChanged(&sr->numCell->cell, &srb->numCell);
+  restoreCellChanged(&sr->descCell->cell, &srb->descCell);
+  restoreCellChanged(sr->recnCell, &srb->recnCell);
+  restoreCellChanged(&sr->shrsCell->cell, &srb->shrsCell);
+  restoreCellChanged(&sr->balanceCell->cell, &srb->balanceCell);
+  restoreCellChanged(&sr->actionCell->cell, &srb->actionCell);
+  restoreCellChanged(&sr->xfrmCell->cell, &srb->xfrmCell);
+  restoreCellChanged(&sr->mxfrmCell->cell, &srb->mxfrmCell);
+  restoreCellChanged(&sr->xtoCell->cell, &srb->xtoCell);
+  restoreCellChanged(&sr->memoCell->cell, &srb->memoCell);
+  restoreCellChanged(&sr->creditCell->cell, &srb->creditCell);
+  restoreCellChanged(&sr->debitCell->cell, &srb->debitCell);
+  restoreCellChanged(&sr->priceCell->cell, &srb->priceCell);
+  restoreCellChanged(&sr->valueCell->cell, &srb->valueCell);
+  restoreCellChanged(&sr->ncreditCell->cell, &srb->ncreditCell);
+  restoreCellChanged(&sr->ndebitCell->cell, &srb->ndebitCell);
+}
+
 /* ============ END OF FILE ===================== */
