@@ -8,7 +8,7 @@
  * ported to GTK, etc. 
  *
  * HISTORY:
- * Copyright (c) 1998 Linas Vepstas
+ * Copyright (c) 1998,1999 Linas Vepstas
  */
 
 /********************************************************************\
@@ -37,12 +37,12 @@
 #include "cellblock.h"
 #include "table-allgui.h"
 #include "table-motif.h"
+#include "util.h"
 
 static void enterCB (Widget mw, XtPointer cd, XtPointer cb);
 static void leaveCB (Widget mw, XtPointer cd, XtPointer cb);
 static void modifyCB (Widget mw, XtPointer cd, XtPointer cb);
 static void traverseCB (Widget mw, XtPointer cd, XtPointer cb);
-static void doRefreshCursorGUI (Table * table, CellBlock *curs, int row, int col);
 
 /* The XrmQuarks are used to figure out the direction of
  * traversal from cell to cell */
@@ -50,6 +50,8 @@ static void doRefreshCursorGUI (Table * table, CellBlock *curs, int row, int col
 static XrmQuark QPointer, QLeft, QRight, QUp, QDown;
 static Boolean haveQuarks = False;
 
+/* This static indicates the debugging module that this .o belongs to.  */
+static short module = MOD_REGISTER;
 
 /* ==================================================== */
 
@@ -59,26 +61,6 @@ xaccNextTabGroup (Table *table, Widget w)
    table->next_tab_group = w;
 }
 
-/* ==================================================== */
-
-static void 
-wrapVerifyCursorPosition (Table *table, int row, int col)
-{
-   CellBlock *save_curs = table->current_cursor;
-   int save_phys_row = table->current_cursor_phys_row;
-   int save_phys_col = table->current_cursor_phys_col;
-
-   /* VerifyCursor will do all sorts of gui-independent machinations */
-   xaccVerifyCursorPosition (table, row, col);
-
-   if ((save_phys_row != table->current_cursor_phys_row) ||
-       (save_phys_col != table->current_cursor_phys_col))
-   {
-      /* make sure *both* the old and the new cursor rows get redrawn */
-      xaccRefreshCursorGUI (table);  
-      doRefreshCursorGUI (table, save_curs, save_phys_row, save_phys_col);
-   }
-}
 
 /* ==================================================== */
 /* this routine calls the individual cell callbacks */
@@ -89,8 +71,6 @@ cellCB (Widget mw, XtPointer cd, XtPointer cb)
    Table *table;
    XbaeMatrixDefaultActionCallbackStruct *cbs;
    int row, col;
-   int rel_row, rel_col;
-   CellBlock *arr, *header;
    int invalid = 0;
 
    table = (Table *) cd;
@@ -99,66 +79,27 @@ cellCB (Widget mw, XtPointer cd, XtPointer cb)
    row = cbs->row;
    col = cbs->column;
 
-   /* If we are entering this cell, make sure that we've
-    * moved the cursor, and that any subsidiary GUI elements
-    * properly positioned.  Do this *before* we examine the 
-    * value of the "current cursor".  Note that this check
-    * could suck up massive cpu, as potentially it may trigger
-    * a redraw of the entire register.
+   /* If we are entering this cell, make sure that we've moved the
+    * cursor, and that any subsidiary GUI elements are properly
+    * positioned.  Do this *before* we examine the value of the
+    * "current cursor".  Note that this check could suck up massive
+    * cpu, as potentially it may trigger a redraw of the entire
+    * register.
     *
-    * Hmmm ... actually, theoretically, this is not neeeded.
-    * Before we get to here, we *should* have gone through
-    * a traverse callback the determine which cell to enter,
-    * and then, after the leavecell, the cursor should
-    * have been automagically repositioned.  This, the
-    * call below should always no-op out.  However, if/when
-    * things go berzerk, the call below does at least a partial
-    * butt-save for us, so lets leave it in for now.
-    */
+    * Hmmm ... actually, theoretically, this is not neeeded.  Before
+    * we get to here, we *should* have gone through a traverse
+    * callback the determine which cell to enter, and then, after the
+    * leavecell, the cursor should have been automagically
+    * repositioned.  Thus, the call below should always no-op out.
+    * However, if/when things go berzerk, the call below does at least
+    * a partial butt-save for us, so lets leave it in for now.  */
+
    if (XbaeEnterCellReason == cbs->reason) 
    {
-      wrapVerifyCursorPosition (table, row, col);
+     wrapVerifyCursorPosition (table, row, col);
    }
 
-   /* can't edit outside of the physical space */
-   invalid = (0 > row) || (0 > col) ;
-   invalid = invalid || (row >= table->num_phys_rows);
-   invalid = invalid || (col >= table->num_phys_cols);
-
-   /* header rows cannot be modified */
-   /* hack alert -- assumes that header is first cell */
-   /* if 0,0 is not a headr  row, then trouble ... */
-   header = table->handlers[0][0];
-   invalid = invalid || (row < header->numRows);
-
-   /* compute the cell location */
-   rel_row = table->locators[row][col]->phys_row_offset;
-   rel_col = table->locators[row][col]->phys_col_offset;
-
-  /* verify that cursor offsets are valid.  This may occur if
-   * the app that is using the table has a paritally initialized
-   * cursor. (probably due to a prograing error, but maybe they
-   * meant to do this). */
-   invalid = invalid || (0 > rel_row);
-   invalid = invalid || (0 > rel_col);
-
-   /* check for a cell handler, but only if cell adress is valid */
-   arr = table->current_cursor;
-   if (arr && !invalid) {
-      if (! (arr->cells[rel_row][rel_col])) {
-         invalid = TRUE;
-      } else {
-         /* if cell is marked as output-only,
-          * then don't call callbacks */
-         if (0 == (XACC_CELL_ALLOW_INPUT & ((arr->cells[rel_row][rel_col])->input_output))) 
-         {
-            invalid = TRUE;
-         }
-      }
-
-   } else {
-      invalid = TRUE;
-   }
+   invalid = ! gnc_register_cell_valid(table, row, col, GNC_T);
 
    /* oops the callback failed for some reason ... 
     * reject the enter/edit/leave  and return */
@@ -202,14 +143,7 @@ cellCB (Widget mw, XtPointer cd, XtPointer cb)
          break;
       }
       case XbaeLeaveCellReason: {
-         /* Do leaveCB first, so that the cell handler can do whatever
-          * it needs to do to commit values into its cell handler. 
-          * Then do the verify, which in general asumes that cell handlers
-          * are up to date (i.e. the leave has been processed.)
-          */
          leaveCB (mw, cd, cb);
-         wrapVerifyCursorPosition (table, table->reverify_phys_row,
-                                          table->reverify_phys_col);
          break;
       }
       default:
@@ -220,102 +154,61 @@ cellCB (Widget mw, XtPointer cd, XtPointer cb)
 /* ==================================================== */
 /* this callback assumes that basic error checking has already
  * been performed. */
-
+  
 static void
-enterCB (Widget mw, XtPointer cd, XtPointer cb)
-{
-   Table *table;
-   CellBlock *arr;
-   XbaeMatrixEnterCellCallbackStruct *cbs;
-   int row, col;
-   int rel_row, rel_col;
-   const char * (*enter) (BasicCell *, const char *);
+enterCB (Widget mw, XtPointer cd, XtPointer cb) {
 
-   table = (Table *) cd;
-   arr = table->current_cursor;
-   cbs = (XbaeMatrixEnterCellCallbackStruct *) cb;
+  Table *table = (Table *) cd;
+   XbaeMatrixEnterCellCallbackStruct *cbs =
+     (XbaeMatrixEnterCellCallbackStruct *) cb;
+   const int row = cbs->row;
+   const int col = cbs->column;
+   const char *new_text;
+   int cursor_pos, start, end; /* currently not used */
 
-   /* compute the cell location */
-   row = cbs->row;
-   col = cbs->column;
-   rel_row = table->locators[row][col]->phys_row_offset;
-   rel_col = table->locators[row][col]->phys_col_offset;
-
-printf ("enter %d %d (relrow=%d relcol=%d) cell=%p\n", row, col, rel_row, rel_col,
-arr->cells[rel_row][rel_col]);
-
-   /* since we are here, there must be a cell handler.
-    * therefore, we accept entry into the cell by default, 
-    */
-   cbs->doit = True;
-   cbs->map = True;
-
-   /* OK, if there is a callback for this cell, call it */
-   enter = arr->cells[rel_row][rel_col]->enter_cell;
-   if (enter) {
-      const char *val;
-      char *retval;
-
-      val = table->entries[row][col];
-      retval = (char *) enter (arr->cells[rel_row][rel_col], val);
-      if (NULL == retval) retval = (char *) val;
-      if (val != retval) {
-         if (table->entries[row][col]) free (table->entries[row][col]);
-         table->entries[row][col] = retval;
-         (arr->cells[rel_row][rel_col])->changed = 0xffffffff;
-         XbaeMatrixSetCell (mw, row, col, retval);
-         XbaeMatrixRefreshCell (mw, row, col);
-
-         /* don't map a text widget */
-         cbs->map = False;
-         cbs->doit = False;
-      }
+   ENTER ("enterCB()\n");
+   new_text = gnc_table_enter_update(table, row, col,
+                                     &cursor_pos, &start, &end);
+   
+   if(new_text) {
+      DEBUG ("enterCB new text = %s\n", new_text);
+      XbaeMatrixSetCell (mw, row, col, (char *)new_text);
+      XbaeMatrixRefreshCell (mw, row, col);
+      
+      /* don't map a text widget */
+      cbs->map = False;
+      cbs->doit = False;
+   } else {
+     cbs->doit = True;
+     cbs->map = True;
    }
-
-   /* record this position as the cell that will be
-    * traversed out of if a traverse even happens */
-   table->prev_phys_traverse_row = row;
-   table->prev_phys_traverse_col = col;
+   LEAVE ("enterCB()\n");
 }
-
+   
 /* ==================================================== */
 /* this routine calls the individual cell callbacks */
 
 static void
 modifyCB (Widget mw, XtPointer cd, XtPointer cb)
 {
-   Table *table;
-   CellBlock *arr;
-   XbaeMatrixModifyVerifyCallbackStruct *cbs;
-   int row, col;
-   int rel_row, rel_col;
-   const char * (*mv) (BasicCell *, 
-                       const char *, 
-                       const char *, 
-                       const char *);
-   const char *oldval, *change;
-   char *newval;
+   Table *table = (Table *) cd;
+   XbaeMatrixModifyVerifyCallbackStruct *cbs =
+     (XbaeMatrixModifyVerifyCallbackStruct *) cb;
+
+   const int row = cbs->row;
+   const int col = cbs->column;
+   const char *oldval = cbs->prev_text;
+   const char *change = cbs->verify->text->ptr;
+   int len = 1;
    const char *retval;
-   int len;
-
-   table = (Table *) cd;
-   arr = table->current_cursor;
-   cbs = (XbaeMatrixModifyVerifyCallbackStruct *) cb;
-
-   /* compute the cell location */
-   row = cbs->row;
-   col = cbs->column;
-   rel_row = table->locators[row][col]->phys_row_offset;
-   rel_col = table->locators[row][col]->phys_col_offset;
+   char *newval;
 
+   ENTER("modifyCB()\n");
+   
    /* accept edits by default, unless the cell handler rejects them */
    cbs->verify->doit = True;
 
-   oldval = cbs->prev_text;
-   change = cbs->verify->text->ptr;
-
    /* first, compute the newval string */
-   len = 1;
    if (oldval) len += strlen (oldval);
    if (change) len += strlen (change);
    newval = (char *) malloc (len);
@@ -326,112 +219,57 @@ modifyCB (Widget mw, XtPointer cd, XtPointer cb)
    if (change) strcat (newval, change);
    strcat (newval, &oldval[(cbs->verify->endPos)]);
 
-   /* OK, if there is a callback for this cell, call it */
-   mv = arr->cells[rel_row][rel_col]->modify_verify;
-   if (mv) {
-      retval = (*mv) (arr->cells[rel_row][rel_col], oldval, change, newval);
+   {
+     /* These are unused at present. */
+     int cursor_position = 0;
+     int start_sel = 0;
+     int end_sel = 0;
 
-      /* if the callback returned a non-null value, allow the edit */
-      if (retval) {
-
-         /* update data. bounds check done earlier */
-         free (table->entries[row][col]);
-         table->entries[row][col] = (char *) retval;
-         (arr->cells[rel_row][rel_col])->changed = 0xffffffff;
-
-         /* if the callback modified the display string,
-          * update the display cell as well */
-         if (retval != newval) {
-            XbaeMatrixSetCell (mw, row, col, (char *) retval);
-            XbaeMatrixRefreshCell (mw, row, col);
-            XbaeMatrixSetCursorPosition (mw, (cbs->verify->endPos) +1);
-
-            /* the default update has already been overridden,
-             * so don't allow Xbae to update */
-            cbs->verify->doit = False;
-
-            /* avoid wasting memory */
-            free (newval);
-         }
-      } else {
-         /* NULL return value means the edit was rejected */
-         cbs->verify->doit = False;
-
-         /* avoid wasting memory */
-         free(newval);
-      }
-   } else {
-      /* update data. bounds check done earlier */
-      free (table->entries[row][col]);
-      table->entries[row][col] = newval;
-      (arr->cells[rel_row][rel_col])->changed = 0xffffffff;
+     retval = gnc_table_modify_update(table, row, col, oldval, change, newval,
+                                      &cursor_position, &start_sel, &end_sel);
    }
+
+   if (retval && (retval != newval)) {
+     DEBUG ("modifyCB() new text: %s\n", retval);
+     XbaeMatrixSetCell (mw, row, col, (char *) retval);
+     XbaeMatrixRefreshCell (mw, row, col);
+     XbaeMatrixSetCursorPosition (mw, (cbs->verify->endPos) + 1);
+     
+     /* the default update has already been overridden,
+      * so don't allow Xbae to update */
+     cbs->verify->doit = False;
+
+     free (newval);
+   } else if(!retval) {
+     cbs->verify->doit = False;
+     free(newval);
+   }
+   LEAVE("modifyCB()\n");
 }
 
 /* ==================================================== */
-
+   
 static void
 leaveCB (Widget mw, XtPointer cd, XtPointer cb)
 {
-   Table *table;
-   CellBlock *arr;
-   XbaeMatrixLeaveCellCallbackStruct *cbs;
-   int row, col;
-   int rel_row, rel_col;
-   const char * (*leave) (BasicCell *, const char *);
-   char * newval;
+  Table *table = (Table *) cd;
+   XbaeMatrixLeaveCellCallbackStruct *cbs =
+     (XbaeMatrixLeaveCellCallbackStruct *) cb;
+   const int row = cbs->row;
+   const int col = cbs->column;
+   const char * new_text;
+   ENTER("leaveCB()\n");
 
-   table = (Table *) cd;
-   arr = table->current_cursor;
-   cbs = (XbaeMatrixLeaveCellCallbackStruct *) cb;
+   new_text = gnc_table_leave_update(table, row, col, cbs->value);
 
-   /* compute the cell location */
-   row = cbs->row;
-   col = cbs->column;
-   rel_row = table->locators[row][col]->phys_row_offset;
-   rel_col = table->locators[row][col]->phys_col_offset;
-
-printf ("leave %d %d \n", row, col);
+   if(new_text) {
+     DEBUG("leaveCB() new text=%s\n", new_text);
+     cbs->value = strdup (new_text);
+   }
 
    /* by default, accept whatever the final proposed edit is */
    cbs->doit = True;
-
-   /* OK, if there is a callback for this cell, call it */
-   leave = arr->cells[rel_row][rel_col]->leave_cell;
-   if (leave) {
-      const char *val, *retval;
-
-      val = cbs->value;
-      retval = leave (arr->cells[rel_row][rel_col], val);
-
-      newval = (char *) retval;
-      if (NULL == retval) newval = strdup (val);
-      if (val == retval) newval = strdup (val);
-
-      /* if the leave() routine declared a new string, lets use it */
-      if ( retval && (retval != val)) {
-         cbs->value = strdup (retval);
-      }
-
-   } else {
-      newval = strdup (cbs->value);
-   }
-
-   /* save whatever was returned; but lets check for  
-    * changes to avoid roiling the cells too much */
-   if (table->entries[row][col]) {
-      if (strcmp (table->entries[row][col], newval)) {
-         free (table->entries[row][col]);
-         table->entries[row][col] = newval;
-         (arr->cells[rel_row][rel_col])->changed = 0xffffffff;
-      } else {
-         /* leave() allocated memory, which we will not be using ... */
-         free (newval);
-      }
-   } else {
-      table->entries[row][col] = newval;
-      (arr->cells[rel_row][rel_col])->changed = 0xffffffff;
-   }
+   LEAVE("leaveCB()\n");
 }
 
 
@@ -440,126 +278,67 @@ printf ("leave %d %d \n", row, col);
 static void
 traverseCB (Widget mw, XtPointer cd, XtPointer cb)
 {
-   Table *table;
-   CellBlock *arr;
-   XbaeMatrixTraverseCellCallbackStruct *cbs;
-   int row, col;
-   int rel_row, rel_col;
+  Table *table = (Table *) cd;
+  XbaeMatrixTraverseCellCallbackStruct *cbs =
+    (XbaeMatrixTraverseCellCallbackStruct *) cb;
+  int row = cbs->row;
+  int col = cbs->column;
+  
+  gncTableTraversalDir dir;
 
-   table = (Table *) cd;
-   arr = table->current_cursor;
-   cbs = (XbaeMatrixTraverseCellCallbackStruct *) cb;
-
-   row = cbs->row;
-   col = cbs->column;
-
-   /* If the quark is zero, then it is likely that we are
-    * here because we traversed out of a cell that had a 
-    * ComboBox in it.  The ComboCell is clever enough to put
-    * us back into the register after tabing out of it.
-    * However, its not (cannot be) clever enough to pretend
-    * that it was a tab group in the register.  Thus,
-    * we will emulate that we left a tab group in the register
-    * to get here.  To put it more simply, we just set the 
-    * row and column to that of the ComboCell, which we had
-    * previously recorded, and continue on as if nothing 
-    * happened.  
-    *
-    * BTW -- note that we are emulating a normal, right-moving tab. 
-    * Backwards tabs are broken. 
-    *
-    * Note that the quark may also be zero if we are entering 
-    * the register for the first time after a table resize.
-    */
-   if (NULLQUARK == cbs->qparam) { 
-      if ((0==row) && (0==col)) {
-        if ((0 <= table->prev_phys_traverse_row) && 
-            (0 <= table->prev_phys_traverse_col)) {
-          cbs->qparam = QRight;
-          row = table->prev_phys_traverse_row;
-          col = table->prev_phys_traverse_col;
-          if (0 > row) row = 0;
-          if (0 > col) col = 0;
-        }
+  ENTER ("traverseCB()\n");
+  if(cbs->qparam == QRight) dir = GNC_TABLE_TRAVERSE_RIGHT;
+  else if(cbs->qparam == QLeft) dir = GNC_TABLE_TRAVERSE_LEFT;
+  else if(cbs->qparam == QUp) dir = GNC_TABLE_TRAVERSE_UP;
+  else if(cbs->qparam == QDown) dir = GNC_TABLE_TRAVERSE_DOWN;
+  else if(cbs->qparam == QPointer) dir = GNC_TABLE_TRAVERSE_POINTER;
+  else if(cbs->qparam == NULLQUARK) {
+    /* If the qparm is NULL, then it is likely that we are here
+       because we traversed out of a cell that had a ComboBox in it.
+       The ComboCell is clever enough to put us back into the register
+       after tabing out of it.  However, its not (cannot be) clever
+       enough to pretend that it was a tab group in the register.
+       Thus, we will emulate that we left a tab group in the register
+       to get here.  To put it more simply, we just set the row and
+       column to that of the ComboCell, which we had previously
+       recorded, and continue on as if nothing happened.
+       
+       BTW -- note that we are emulating a normal, right-moving tab.
+       Backwards tabs are broken.
+       
+       Note that the quark may also be zero if we are entering the
+       register for the first time after a table resize.  */
+      
+    dir = GNC_TABLE_TRAVERSE_RIGHT;
+      
+    if ((0==row) && (0==col)) {
+      if ((0 <= table->prev_phys_traverse_row) && 
+          (0 <= table->prev_phys_traverse_col)) {
+        row = table->prev_phys_traverse_row;
+        col = table->prev_phys_traverse_col;
+        if (0 > row) row = 0;
+        if (0 > col) col = 0;
       }
-   }
-
-   /* process right-moving traversals */
-   if (QRight == cbs->qparam) {
-      int next_row, next_col;
-
-      /* Don't do a thing unless we verify that the row and column
-       * are in bounds. Ordinarily, they are always in bounds, except 
-       * in an unusual, arguably buggy situation: If the table has 
-       * been recently resized smaller, then the Xbae code might report
-       * a traverse out of a cell that was in the larger array, but not
-       * in the smaller array.  This is probably an Xbae bug. It 
-       * will core dump array access.
-       */
-      if ((row >= table->num_phys_rows) || 
-          (col >= table->num_phys_cols)) {
-
-assert (0);
-         table->prev_phys_traverse_row = cbs->next_row;
-         table->prev_phys_traverse_col = cbs->next_column;
-         return;
+    }
+  } else {
+    PERR("traverseCB(): Invalid traversal direction. (%s)\n",
+         __FUNCTION__);
+    return;
+  }
+  
+  {
+    int exit_register = gnc_table_traverse_update(table, row, col,
+                                                  dir,
+                                                  &(cbs->next_row),
+                                                  &(cbs->next_column));
+    if(exit_register) {
+      cbs->qparam = NULLQUARK; 
+      if (table->next_tab_group) {
+        XmProcessTraversal (table->next_tab_group, XmTRAVERSE_CURRENT);
       }
-
-      /* compute the cell location */
-      rel_row = table->locators[row][col]->phys_row_offset;
-      rel_col = table->locators[row][col]->phys_col_offset;
-
-      next_row = arr->right_traverse_r[rel_row][rel_col];
-      next_col = arr->right_traverse_c[rel_row][rel_col];
-
-      /* if we are at the end of the traversal chain,
-       * hop out of this tab group, and into the next.
-       */
-      if ((0 > next_row) || (0 > next_col)) {
-         /* reverse the sign of next_row, col to be positive. */
-         cbs->next_row    = row - rel_row - next_row -1; 
-         cbs->next_column = col - rel_col - next_col -1;
-         cbs->qparam      = NULLQUARK; 
-         if (table->next_tab_group) {
-            XmProcessTraversal (table->next_tab_group, 
-                                XmTRAVERSE_CURRENT); 
-         }
-      } else {
-         cbs->next_row    = row - rel_row + next_row; 
-         cbs->next_column = col - rel_col + next_col;
-      }
-   } 
-
-   /* 
-    * OK, now we do a fancy trick to get the auto-expanding registers to 
-    * work right.  The trick is that as one transaction is expanded or
-    * collapsed, the rows all get renumbered.  Now, we can't tell the 
-    * code to directly hope to the new renumbered position, because
-    * we haven't completed all of our work yet.  In particular, we
-    * haven't left the current cell, and this means we haven't yet saved 
-    * those cell contents.  Only after saving, can we reconfigure the
-    * table.  And only after we reconfigure the table can we move the 
-    * since only then will we know where & how to move it to.  Sooo ...
-    * here's what we do. We compute where we should have hopped to
-    * in the reconfigured table, and save that off in the "reverify"
-    * fields.  Then we hop to the boring old place we would have hopped 
-    * to if there had been no reconfiguring going on.  Later on, after
-    * we've reconfigured, we will move the cursor to the "reverify"
-    * position, and viola, we'll be in the right place.
-    */
-
-   if (table->traverse) {
-      int nr = cbs->next_row;
-      int nc = cbs->next_column;
-      table->reverify_phys_row = nr;
-      table->reverify_phys_col = nc;
-      (table->traverse) (table, &nr, &nc, table->client_data);
-      cbs->next_row = nr;
-      cbs->next_column = nc;
-   }
-
-   table->prev_phys_traverse_row = cbs->next_row;
-   table->prev_phys_traverse_col = cbs->next_column;
+    }
+  }
+  LEAVE ("traverseCB()\n");
 }
 
 /* ==================================================== */
@@ -628,47 +407,20 @@ xaccCreateTable (Table *table, Widget parent, char * name)
 
    table->table_widget = reg;
 
+#if (!defined (LesstifVersion)) || (87 < LesstifVersion)
+   /* Coredump when using lesstif 0.80 or earlier; 0.88.1 definitely works */
    /* stop it from beeping */
    XtVaGetValues (reg, XmNtextField, &textw, NULL);
    XtVaSetValues (textw, XmNverifyBell, False, NULL);
+#endif /* LESSTIF_VERSION */
 
    return (reg);
 }
 
-/* ==================================================== */
-/* if any of the cells have GUI specific components that need 
- * initialization, initialize them now. The realize() callback
- * on the cursor cell is how we inform the cell handler that 
- * now is the time to initialize it's GUI.  */
-
-void
-xaccCreateCursor (Table *table, CellBlock *curs) 
-{
-   int i,j;
-   Widget reg;
-   if (!curs || !table) return;
-
-   /* if Xbae itself is not yet created, then we can't go here either */
-   reg = table->table_widget;
-   if (!reg) return;
-
-   for (i=0; i<curs->numRows; i++) {
-      for (j=0; j<curs->numCols; j++) {
-         BasicCell *cell;
-         cell = curs->cells[i][j];
-         if (cell) {
-            void (*xt_realize) (BasicCell *, 
-                                void *gui,
-                                int pixel_width);
-            xt_realize = cell->realize;
-            if (xt_realize) {
-               int pixel_width;
-               pixel_width = XbaeMatrixGetColumnPixelWidth (reg, j);
-               xt_realize (cell, ((void *) reg), pixel_width);
-            }
-         }
-      }
-   }
+int
+gnc_table_column_width(Table *table, const int col) {
+  Widget reg = table->table_widget;
+  return(XbaeMatrixGetColumnPixelWidth (reg, col));
 }
 
 
@@ -687,7 +439,7 @@ xaccMotifResizeTable (Table * table,
                new_phys_rows,
                new_phys_cols,
                (table->bg_hues),
-               Pixel,
+              Pixel,
                1,
                NOOP);
 
@@ -863,16 +615,16 @@ xaccRefreshTableGUI (Table * table)
   if (!table) return;
   if (!(table->table_widget)) return;
 
-{int i;
-printf (" refresh numphysrows=%d numphyscols=%d =========================\n", 
-table->num_phys_rows,table->num_phys_cols);
-for (i=0; i<table->num_phys_rows; i++) {
-printf ("cell %d\tcolor: 0x%x\tact:%s\tdescr: %s\tpay: %s\n", i, 
-table->bg_colors[i][3], 
-table->entries[i][2],
-table->entries[i][3],
-table->entries[i][5]);
-}}
+  DEBUGCMD ({int i;
+     printf (" refresh numphysrows=%d numphyscols=%d =========================\n", 
+     table->num_phys_rows,table->num_phys_cols);
+     for (i=0; i<table->num_phys_rows; i++) {
+     printf ("cell %d\tcolor: 0x%x\tact:%s\tdescr: %s\tpay: %s\n", i, 
+     table->bg_colors[i][3], 
+     table->entries[i][2],
+     table->entries[i][3],
+     table->entries[i][5]);
+     }});
 
   RefreshColors (table, 0, table->num_phys_rows, 0, table->num_phys_cols);
 
@@ -887,8 +639,9 @@ table->entries[i][5]);
 
 /* ==================================================== */
 
-static void        
-doRefreshCursorGUI (Table * table, CellBlock *curs, int from_row, int from_col)
+void        
+doRefreshCursorGUI (Table * table, CellBlock *curs, int from_row, int from_col,
+                    gncBoolean do_scroll)
 {
    int phys_row, phys_col;
    int to_row, to_col;
@@ -897,6 +650,9 @@ doRefreshCursorGUI (Table * table, CellBlock *curs, int from_row, int from_col)
    /* if the current cursor is undefined, there is nothing to do. */
    if (!curs) return;
    if ((0 > from_row) || (0 > from_col)) return;
+   if ((from_row >= table->num_phys_rows) ||
+       (from_col >= table->num_phys_cols))
+     return;
 
    /* compute the physical bounds of the current cursor */
    phys_row = from_row;
@@ -932,14 +688,4 @@ doRefreshCursorGUI (Table * table, CellBlock *curs, int from_row, int from_col)
     */
 }
 
-/* ==================================================== */
-
-void        
-xaccRefreshCursorGUI (Table * table)
-{
-   doRefreshCursorGUI (table, table->current_cursor,
-      table->current_cursor_phys_row,
-      table->current_cursor_phys_col);
-}
-
 /* ================== end of file ======================= */
